@@ -12,10 +12,10 @@
 ##############################################################################
 """Add security system support to Document Templates
 
-$Id: DTML.py 40320 2005-11-22 16:08:56Z tseaver $
+$Id: DTML.py 72009 2007-01-13 16:42:22Z andreasjung $
 """
 from DocumentTemplate import DT_Util
-import SecurityManagement, string, math, random
+import SecurityManagement, string, math, random, sets
 import DocumentTemplate.sequence
 
 from ZopeGuards import safe_builtins
@@ -28,6 +28,8 @@ DT_Util.TemplateDict.__allow_access_to_unprotected_subobjects__=1
 string.__allow_access_to_unprotected_subobjects__=1
 math.__allow_access_to_unprotected_subobjects__=1
 random.__allow_access_to_unprotected_subobjects__=1
+sets.__allow_access_to_unprotected_subobjects__=1
+
 
 DocumentTemplate.sequence.__allow_access_to_unprotected_subobjects__=1
 
