@@ -12,10 +12,11 @@
 ##############################################################################
 """Permissions
 
-$Id: Permission.py 94468 2009-01-03 12:23:22Z yuppie $
+$Id: Permission.py 88040 2008-07-05 15:56:57Z tseaver $
 """
 
-import string, Products
+import string, Products, Globals
+
 
 name_trans=filter(lambda c, an=string.letters+string.digits+'_': c not in an,
                   map(chr,range(256)))
@@ -132,10 +133,6 @@ def registerPermissions(permissions, defaultDefault=('Manager',)):
         Products.__ac_permissions__=(
             Products_permissions + ((perm, (), default),))
         mangled=pname(perm) # get mangled permission name
-        if not hasattr(ApplicationDefaultPermissions, mangled):
-            setattr(ApplicationDefaultPermissions,
+        if not hasattr(Globals.ApplicationDefaultPermissions, mangled):
+            setattr(Globals.ApplicationDefaultPermissions,
                     mangled, default)
-
-class ApplicationDefaultPermissions:
-    _View_Permission = ('Manager', 'Anonymous')
-    _Access_contents_information_Permission = ('Manager', 'Anonymous')
