@@ -12,7 +12,7 @@
 ##############################################################################
 """Access control support
 
-$Id: Role.py 88052 2008-07-06 01:14:06Z tseaver $
+$Id: Role.py 88051 2008-07-06 01:12:15Z tseaver $
 """
 from cgi import escape
 
@@ -20,6 +20,7 @@ from Globals import DTMLFile, MessageDialog, Dictionary
 from Acquisition import Implicit, Acquired, aq_get
 from Globals import InitializeClass
 from AccessControl import ClassSecurityInfo
+from AccessControl.SecurityManagement import newSecurityManager
 from AccessControl.Permissions import change_permissions
 import ExtensionClass, PermissionMapping, Products
 from App.Common import aq_base
@@ -27,7 +28,7 @@ from zope.interface import implements
 
 from interfaces import IRoleManager
 from Permission import Permission
-from requestmethod import postonly
+from requestmethod import requestmethod
 
 
 DEFAULTMAXLISTUSERS=250
@@ -129,7 +130,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                              help_product='OFSP')
 
     security.declareProtected(change_permissions, 'manage_role')
-    @postonly
+    @requestmethod('POST')
     def manage_role(self, role_to_manage, permissions=[], REQUEST=None):
         """Change the permissions given to the given role.
         """
@@ -148,7 +149,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                                  help_product='OFSP')
 
     security.declareProtected(change_permissions, 'manage_acquiredPermissions')
-    @postonly
+    @requestmethod('POST')
     def manage_acquiredPermissions(self, permissions=[], REQUEST=None):
         """Change the permissions that acquire.
         """
@@ -163,6 +164,66 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         if REQUEST is not None: return self.manage_access(REQUEST)
 
+    def manage_getUserRolesAndPermissions(self, user_id):
+        """ Used for permission/role reporting for a given user_id.
+            Returns a dict mapping
+
+            'user_defined_in' -> path where the user account is defined
+            'roles' -> global roles,
+            'roles_in_context' -> roles in context of the current object,
+            'allowed_permissions' -> permissions allowed for the user,
+            'disallowed_permissions' -> all other permissions 
+        """
+        
+        d = {}
+
+        current = self
+        while 1:
+            try:
+                uf = current.acl_users
+            except AttributeError:
+                raise ValueError('User %s could not be found' % user_id)
+
+            userObj = uf.getUser(user_id)
+            if userObj:
+                break
+            else:
+                current = current.aq_parent
+
+
+        newSecurityManager(None, userObj) # necessary?
+        userObj = userObj.__of__(uf)
+
+        d = {'user_defined_in' : '/' + uf.absolute_url(1)}
+
+        # roles
+        roles = list(userObj.getRoles())
+        roles.sort()
+        d['roles'] = roles
+
+
+        # roles in context
+        roles = list(userObj.getRolesInContext(self))
+        roles.sort()
+        d['roles_in_context'] = roles
+
+        # permissions
+        allowed = []
+        disallowed = []
+        permMap = self.manage_getPermissionMapping()
+        for item in permMap:
+            p = item['permission_name']
+            if userObj.has_permission(p, self):
+                allowed.append(p)
+            else:
+                disallowed.append(p)
+
+        d['allowed_permissions'] = allowed
+        d['disallowed_permissions'] = disallowed
+
+        return d
+
+
     security.declareProtected(change_permissions, 'manage_permissionForm')
     manage_permissionForm=DTMLFile('dtml/permissionEdit', globals(),
                                    management_view='Security',
@@ -170,7 +231,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                                    help_product='OFSP')
 
     security.declareProtected(change_permissions, 'manage_permission')
-    @postonly
+    @requestmethod('POST')
     def manage_permission(self, permission_to_manage,
                           roles=[], acquire=0, REQUEST=None):
         """Change the settings for the given permission.
@@ -197,6 +258,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
     _normal_manage_access=DTMLFile('dtml/access', globals())
 
     _method_manage_access=DTMLFile('dtml/methodAccess', globals())
+    manage_reportUserPermissions=DTMLFile('dtml/reportUserPermissions', globals())
 
     security.declareProtected(change_permissions, 'manage_access')
     def manage_access(self, REQUEST, **kw):
@@ -209,7 +271,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             return apply(self._normal_manage_access,(), kw)
 
     security.declareProtected(change_permissions, 'manage_changePermissions')
-    @postonly
+    @requestmethod('POST')
     def manage_changePermissions(self, REQUEST):
         """Change all permissions settings, called by management screen.
         """
@@ -364,7 +426,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         return tuple(dict.get(userid, []))
 
     security.declareProtected(change_permissions, 'manage_addLocalRoles')
-    @postonly
+    @requestmethod('POST')
     def manage_addLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
         if not roles:
@@ -383,7 +445,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
     security.declareProtected(change_permissions, 'manage_setLocalRoles')
-    @postonly
+    @requestmethod('POST')
     def manage_setLocalRoles(self, userid, roles, REQUEST=None):
         """Set local roles for a user."""
         if not roles:
@@ -398,7 +460,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             return self.manage_listLocalRoles(self, REQUEST, stat=stat)
 
     security.declareProtected(change_permissions, 'manage_delLocalRoles')
-    @postonly
+    @requestmethod('POST')
     def manage_delLocalRoles(self, userids, REQUEST=None):
         """Remove all local roles for a user."""
         dict=self.__ac_local_roles__
@@ -491,7 +553,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         return self.manage_access(REQUEST)
 
-    @postonly
+    @requestmethod('POST')
     def _addRole(self, role, REQUEST=None):
         if not role:
             return MessageDialog(
@@ -509,7 +571,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         if REQUEST is not None:
             return self.manage_access(REQUEST)
 
-    @postonly
+    @requestmethod('POST')
     def _delRoles(self, roles, REQUEST=None):
         if not roles:
             return MessageDialog(
