@@ -12,7 +12,7 @@
 ##############################################################################
 """Access control package.
 
-$Id: User.py 73389 2007-03-20 09:03:57Z mj $
+$Id: User.py 73386 2007-03-20 08:50:24Z mj $
 """
 
 import os
@@ -20,10 +20,12 @@ import re
 import socket
 from base64 import decodestring
 
-import Globals
 from Acquisition import Implicit
 from App.Management import Navigation, Tabs
 from Globals import DTMLFile, MessageDialog, Persistent, PersistentMapping
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
+from AccessControl.Permissions import manage_users as ManageUsers
 from OFS.SimpleItem import Item
 from zExceptions import Unauthorized, BadRequest
 from zope.interface import implements
@@ -31,7 +33,7 @@ from zope.interface import implements
 import AuthEncoding
 import SpecialUsers
 from interfaces import IStandardUserFolder
-from requestmethod import postonly
+from requestmethod import requestmethod
 from PermissionRole import _what_not_even_god_should_do, rolesForPermissionOn
 from Role import RoleManager, DEFAULTMAXLISTUSERS
 from SecurityManagement import getSecurityManager
@@ -460,6 +462,8 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
 
     encrypt_passwords = 1
 
+    security = ClassSecurityInfo()
+
     manage_options=(
         (
         {'label':'Contents', 'action':'manage_main',
@@ -471,32 +475,26 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         +Item.manage_options
         )
 
-    __ac_permissions__=(
-        ('Manage users',
-         ('manage_users','getUserNames', 'getUser', 'getUsers',
-          'getUserById', 'user_names', 'setDomainAuthenticationMode',
-          'userFolderAddUser', 'userFolderEditUser', 'userFolderDelUsers',
-          )
-         ),
-        )
-
-
     # ----------------------------------
     # Public UserFolder object interface
     # ----------------------------------
 
+    security.declareProtected(ManageUsers, 'getUserNames')
     def getUserNames(self):
         """Return a list of usernames"""
         raise NotImplementedError
 
+    security.declareProtected(ManageUsers, 'getUsers')
     def getUsers(self):
         """Return a list of user objects"""
         raise NotImplementedError
 
+    security.declareProtected(ManageUsers, 'getUser')
     def getUser(self, name):
         """Return the named user object or None"""
         raise NotImplementedError
 
+    security.declareProtected(ManageUsers, 'getUserById')
     def getUserById(self, id, default=None):
         """Return the user corresponding to the given id.
         """
@@ -535,7 +533,9 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     # Authors of custom user folders don't need to do anything special to
     # support these - they will just call the appropriate '_' methods that
     # user folder subclasses already implement.
-    @postonly
+
+    security.declareProtected(ManageUsers, 'userFolderAddUser')
+    @requestmethod('POST')
     def userFolderAddUser(self, name, password, roles, domains,
                           REQUEST=None, **kw):
         """API method for creating a new user object. Note that not all
@@ -545,7 +545,8 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
             return self._doAddUser(name, password, roles, domains, **kw)
         raise NotImplementedError
 
-    @postonly
+    security.declareProtected(ManageUsers, 'userFolderEditUser')
+    @requestmethod('POST')
     def userFolderEditUser(self, name, password, roles, domains,
                            REQUEST=None, **kw):
         """API method for changing user object attributes. Note that not
@@ -555,7 +556,8 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
             return self._doChangeUser(name, password, roles, domains, **kw)
         raise NotImplementedError
 
-    @postonly
+    security.declareProtected(ManageUsers, 'userFolderDelUsers')
+    @requestmethod('POST')
     def userFolderDelUsers(self, names, REQUEST=None):
         """API method for deleting one or more user objects. Note that not
            all user folder implementations support deletion of user objects."""
@@ -798,7 +800,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
             self, REQUEST, manage_tabs_message=manage_tabs_message,
             management_view='Properties')
 
-    @postonly
+    @requestmethod('POST')
     def manage_setUserFolderProperties(self, encrypt_passwords=0,
                                        update_passwords=0,
                                        maxlistusers=DEFAULTMAXLISTUSERS,
@@ -853,7 +855,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
 
         return 1
 
-    @postonly
+    @requestmethod('POST')
     def _addUser(self,name,password,confirm,roles,domains,REQUEST=None):
         if not name:
             return MessageDialog(
@@ -889,7 +891,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         self._doAddUser(name, password, roles, domains)
         if REQUEST: return self._mainUser(self, REQUEST)
 
-    @postonly
+    @requestmethod('POST')
     def _changeUser(self,name,password,confirm,roles,domains,REQUEST=None):
         if password == 'password' and confirm == 'pconfirm':
             # Protocol for editUser.dtml to indicate unchanged password
@@ -927,7 +929,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         self._doChangeUser(name, password, roles, domains)
         if REQUEST: return self._mainUser(self, REQUEST)
 
-    @postonly
+    @requestmethod('POST')
     def _delUsers(self,names,REQUEST=None):
         if not names:
             return MessageDialog(
@@ -937,6 +939,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
         self._doDelUsers(names)
         if REQUEST: return self._mainUser(self, REQUEST)
 
+    security.declareProtected(ManageUsers, 'manage_users')
     def manage_users(self,submit=None,REQUEST=None,RESPONSE=None):
         """This method handles operations on users for the web based forms
            of the ZMI. Application code (code that is outside of the forms
@@ -976,6 +979,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
 
         return self._mainUser(self, REQUEST)
 
+    security.declareProtected(ManageUsers, 'user_names')
     def user_names(self):
         return self.getUserNames()
 
@@ -1002,6 +1006,7 @@ class BasicUserFolder(Implicit, Persistent, Navigation, Tabs, RoleManager,
     # Domain authentication support. This is a good candidate to
     # become deprecated in future Zope versions.
 
+    security.declareProtected(ManageUsers, 'setDomainAuthenticationMode')
     def setDomainAuthenticationMode(self, domain_auth_mode):
         """Set the domain-based authentication mode. By default, this
            mode is off due to the high overhead of the operation that
@@ -1106,7 +1111,7 @@ class UserFolder(BasicUserFolder):
                     pass
 
 
-Globals.default__class_init__(UserFolder)
+InitializeClass(UserFolder)
 
 
 def manage_addUserFolder(self,dtself=None,REQUEST=None,**ignored):
