@@ -36,7 +36,7 @@
   USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
   DAMAGE.
 
-  $Id: cAccessControl.c 41698 2006-02-19 12:21:40Z andreasjung $
+  $Id: cAccessControl.c 79532 2007-09-08 19:29:06Z nikhil_n $
 
   If you have questions regarding this software,
   contact:
@@ -651,8 +651,7 @@ static PyExtensionClass imPermissionRoleType = {
 static PyObject *Containers = NULL;
 static PyObject *ContainerAssertions = NULL;
 static PyObject *Unauthorized = NULL;
-static PyObject *LOG = NULL;
-static PyObject *PROBLEM = NULL;
+static PyObject *warn= NULL;
 static PyObject *NoSequenceFormat = NULL;
 static PyObject *_what_not_even_god_should_do = NULL;
 static PyObject *Anonymous = NULL;
@@ -1015,7 +1014,7 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 	**| try:
 	**|    if roles is None or 'Anonymous' in roles: return 1
 	**| except TypeError:
-	**|     LOG("Zope Security Policy", PROBLEM, '"%s' passed as roles"
+	**|     LOG.warn('"%s' passed as roles"
 	**|		" during validation of '%s' is not a sequence." % 
 	**|		('roles', name))
 	**|	raise
@@ -1045,8 +1044,7 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
               m=PyObject_Repr(roles);
               if (m) ASSIGN(m, Py_BuildValue("OO", m, name));
               if (m) ASSIGN(m, PyString_Format(NoSequenceFormat, m));
-              if (m) ASSIGN(m, PyObject_CallFunction(LOG, "sOO",
-                     "Zope Security Policy", PROBLEM, m)); 
+              if (m) ASSIGN(m, PyObject_CallFunction(warn, "O", m));
               Py_XDECREF(m);
               PyErr_Restore(t, v, tb);
               goto err;
@@ -1280,7 +1278,7 @@ static PyObject *ZopeSecurityPolicy_validate(PyObject *self, PyObject *args) {
 static void ZopeSecurityPolicy_dealloc(ZopeSecurityPolicy *self) {
 
 	Py_DECREF(self->ob_type);	/* Extensionclass init incref'd */
-	PyMem_DEL(self);  
+	PyObject_DEL(self);  
 }
 
 
@@ -1364,7 +1362,7 @@ SecurityManager_dealloc(SecurityManager *self)
   Py_XDECREF(self->validate);
   Py_XDECREF(self->checkPermission);
   Py_DECREF(self->ob_type);	/* Extensionclass init incref'd */
-  PyMem_DEL(self);  
+  PyObject_DEL(self);  
 }
 
 static PyObject *
@@ -1561,7 +1559,7 @@ static void PermissionRole_dealloc(PermissionRole *self) {
 
 	Py_XDECREF(self->ob_type);	/* Extensionclass init incref'd */
 
-	PyMem_DEL(self);  
+	PyObject_DEL(self);  
 }
 
 
@@ -1707,7 +1705,7 @@ static void imPermissionRole_dealloc(imPermissionRole *self) {
 
 	Py_DECREF(self->ob_type);	/* Extensionclass init incref'd */
 
-	PyMem_DEL(self);  
+	PyObject_DEL(self);  
 }
 
 /*
@@ -2152,7 +2150,7 @@ module_guarded_getattr(PyObject *ignored, PyObject *args)
 static PyObject *
 module_aq_validate(PyObject *ignored, PyObject *args)
 {
-  PyObject *inst, *obj, *name, *v, *validate;
+  PyObject *inst=NULL, *obj=NULL, *name=NULL, *v=NULL, *validate=NULL;
 
   if (unpacktuple5(args, "validate", 0,
                    &inst, &obj, &name, &v, &validate) < 0) return NULL;
@@ -2239,7 +2237,7 @@ void initcAccessControl(void) {
 
 	module = Py_InitModule3("cAccessControl",
 		cAccessControl_methods,
-		"$Id: cAccessControl.c 41698 2006-02-19 12:21:40Z andreasjung $\n");
+		"$Id: cAccessControl.c 79532 2007-09-08 19:29:06Z nikhil_n $\n");
 
 	aq_init(); /* For Python <= 2.1.1, aq_init() should be after
                       Py_InitModule(). */
@@ -2304,12 +2302,11 @@ void initcAccessControl(void) {
 	Py_DECREF(module);
 	module = NULL;
 
-	/*| from zLOG import LOG, PROBLEM
+	/*| from logger_wrapper import warn
 	*/
 
-	IMPORT(module, "zLOG");
-	GETATTR(module, LOG);
-	GETATTR(module, PROBLEM);
+	IMPORT(module, "logger_wrapper");
+	GETATTR(module, warn);
 	Py_DECREF(module);
 	module = NULL;
 }
