@@ -12,7 +12,7 @@
 ##############################################################################
 """AccessControl z3 interfaces.
 
-$Id: interfaces.py 40460 2005-12-01 22:45:00Z tseaver $
+$Id: interfaces.py 72052 2007-01-16 13:03:46Z andreasjung $
 """
 
 from AccessControl.SimpleObjectPolicies import _noroles
@@ -265,6 +265,16 @@ class IRoleManager(IPermissionMappingSupport):
         """
         """
 
+    def manage_getUserRolesAndPermissions(user_id):
+        """ Used for permission/role reporting for a given user_id.
+            Returns a dict mapping
+
+            'user_defined_in' -> path where the user account is defined
+            'roles' -> global roles,
+            'roles_in_context' -> roles in context of the current object,
+            'allowed_permissions' -> permissions allowed for the user,
+            'disallowed_permissions' -> all other permissions 
+        """
 
 class IStandardUserFolder(Interface):
 
