@@ -11,17 +11,20 @@
 #
 ##############################################################################
 
-from ZPublisher.HTTPRequest import HTTPRequest
+from zope.interface import implements
+from zope.publisher.interfaces.browser import IBrowserRequest
 
-def makerequest(method):
-    environ = dict(SERVER_NAME='foo', SERVER_PORT='80', REQUEST_METHOD=method)
-    return HTTPRequest(None, environ, None)
+class DummyRequest:
+    implements(IBrowserRequest)
+    
+    def __init__(self, method):
+        self.method = method
 
 def test_suite():
-    from zope.testing.doctest import DocFileSuite
+    from doctest import DocFileSuite
     return DocFileSuite('../requestmethod.txt',
-                        globs=dict(GET=makerequest('GET'),
-                                   POST=makerequest('POST')))
+                        globs=dict(GET=DummyRequest('GET'),
+                                   POST=DummyRequest('POST')))
 
 if __name__ == '__main__':
     import unittest
