@@ -328,7 +328,7 @@
    http://www.bell-labs.com/people/cope/oopsla/Oopsla96TechnicalProgramAbstracts.html#GilLorenz, 
    OOPSLA '96 Proceedings, ACM SIG-PLAN, October, 1996
 
-$Id: tests.py 99193 2009-04-15 14:44:25Z witsch $
+$Id: tests.py 78385 2007-07-27 18:07:04Z philikon $
 """
 
 import ExtensionClass
@@ -1636,9 +1636,6 @@ def test_proxying():
     ...     def __contains__(self, key):
     ...         print 'contains', repr(key)
     ...         return key == 5
-    ...     def __iter__(self):
-    ...         print 'iterating...'
-    ...         return iter((42,))
 
     The naked class behaves like this:
 
@@ -1649,9 +1646,6 @@ def test_proxying():
     >>> 5 in c
     contains 5
     True
-    >>> list(c)
-    iterating...
-    [42]
 
     Let's put c in the context of i:
 
@@ -1666,88 +1660,17 @@ def test_proxying():
     >>> 5 in i.c
     contains 5
     True
-    >>> list(i.c)
-    iterating...
-    [42]
-
-    Let's let's test the same again with an explicit wrapper:
-
-    >>> import Acquisition
-    >>> class Impl(Acquisition.Explicit):
-    ...     pass
-
-    >>> class C(Acquisition.Explicit):
-    ...     def __getitem__(self, key):
-    ...         print 'getitem', key
-    ...         if key == 4:
-    ...             raise IndexError
-    ...         return key
-    ...     def __contains__(self, key):
-    ...         print 'contains', repr(key)
-    ...         return key == 5
-    ...     def __iter__(self):
-    ...         print 'iterating...'
-    ...         return iter((42,))
-
-    The naked class behaves like this:
-
-    >>> c = C()
-    >>> 3 in c
-    contains 3
-    False
-    >>> 5 in c
-    contains 5
-    True
-    >>> list(c)
-    iterating...
-    [42]
-
-    Let's put c in the context of i:
-
-    >>> i = Impl()
-    >>> i.c = c
-
-    Now check that __contains__ is properly used:
-
-    >>> 3 in i.c # c.__of__(i)
-    contains 3
-    False
-    >>> 5 in i.c
-    contains 5
-    True
-    >>> list(i.c)
-    iterating...
-    [42]
-
-    Finally let's check that https://bugs.launchpad.net/zope2/+bug/360761
-    has been fixed:
-
-    >>> class C(Acquisition.Implicit):
-    ...     l=[1,2,3]
-    ...     def __getitem__(self, i):
-    ...         return self.l[i]
-
-    >>> c1 = C()
-    >>> type(iter(c1))
-    <type 'iterator'>
-    >>> list(c1)
-    [1, 2, 3]
-
-    >>> c2 = C().__of__(c1)
-    >>> type(iter(c2))
-    <type 'iterator'>
-    >>> list(c2)
-    [1, 2, 3]
 
     """
 
 
 import unittest
-from zope.testing.doctest import DocTestSuite
+from zope.testing.doctest import DocTestSuite, DocFileSuite
 
 def test_suite():
     return unittest.TestSuite((
         DocTestSuite(),
+        DocFileSuite('README.txt', package='Acquisition'),
         ))
 
 if __name__ == '__main__':
