@@ -12,7 +12,7 @@
  ****************************************************************************/
 static char cDocumentTemplate_module_documentation[] = 
 ""
-"\n$Id: cDocumentTemplate.c 40222 2005-11-18 15:46:28Z andreasjung $"
+"\n$Id: cDocumentTemplate.c 79532 2007-09-08 19:29:06Z nikhil_n $"
 ;
 
 #include "ExtensionClass/ExtensionClass.h"
@@ -84,7 +84,7 @@ InstanceDict_dealloc(InstanceDictobject *self)
   Py_XDECREF(self->namespace);
   Py_XDECREF(self->guarded_getattr);
   Py_DECREF(self->ob_type);
-  PyMem_DEL(self);
+  PyObject_DEL(self);
 }
 
 static PyObject *
@@ -438,7 +438,7 @@ MM_dealloc(MM *self)
   Py_XDECREF(self->data);
   Py_XDECREF(self->dict);
   Py_DECREF(self->ob_type);
-  PyMem_DEL(self);
+  PyObject_DEL(self);
 }
 
 static PyObject *
@@ -514,7 +514,7 @@ static void
 DictInstance_dealloc(DictInstance *self)
 {
   Py_DECREF(self->data);
-  PyMem_DEL(self);
+  PyObject_DEL(self);
 }
 
 static PyObject *
