@@ -1,46 +1,63 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
 """Application support
 
-$Id: Application.py,v 1.191.2.14 2005/01/18 16:21:13 tseaver Exp $
+$Id: Application.py 76988 2007-06-23 15:37:09Z shh $
 """
-import Globals,Folder,os,sys,App.Product, App.ProductRegistry, misc_
-import time, traceback, os,  Products
-from DateTime import DateTime
-from AccessControl.User import UserFolder
-from App.ApplicationManager import ApplicationManager
-from webdav.NullResource import NullResource
-from FindSupport import FindSupport
+
+import os, sys, traceback
+from logging import getLogger
 from cgi import escape
-from urllib import quote
 from StringIO import StringIO
-from AccessControl.PermissionRole import PermissionRole
-from App.ProductContext import ProductContext
-from misc_ import Misc_
-import ZDOM
-from zLOG import LOG, ERROR, WARNING, INFO
-from zExceptions import Redirect as RedirectException, Forbidden
-from HelpSys.HelpSys import HelpSys
+from warnings import warn
+
+import Globals, Products, App.Product, App.ProductRegistry
+import transaction
+from Globals import InitializeClass
+from AccessControl import ClassSecurityInfo
+from AccessControl.User import UserFolder
 from Acquisition import aq_base
-from App.Product import doInstall
+from App.ApplicationManager import ApplicationManager
 from App.config import getConfiguration
+from App.Product import doInstall
+from App.ProductContext import ProductContext
+from DateTime import DateTime
+from HelpSys.HelpSys import HelpSys
+from webdav.NullResource import NullResource
+from zExceptions import Redirect as RedirectException, Forbidden
+
+from zope.interface import implements
+
+import Folder
+import misc_
+import ZDOM
+from FindSupport import FindSupport
+from interfaces import IApplication
+from misc_ import Misc_
+
+LOG = getLogger('Application')
 
 class Application(Globals.ApplicationDefaultPermissions,
                   ZDOM.Root, Folder.Folder,
                   App.ProductRegistry.ProductRegistry, FindSupport):
+
     """Top-level system object"""
-    title    ='Zope'
-    #__roles__=['Manager', 'Anonymous']
+
+    implements(IApplication)
+
+    security = ClassSecurityInfo()
+
+    title ='Zope'
     __defined_roles__=('Manager','Anonymous','Owner')
     web__form__method='GET'
     isTopLevelPrincipiaApplicationObject=1
@@ -68,9 +85,6 @@ class Application(Globals.ApplicationDefaultPermissions,
 
     _initializer_registry = None
 
-    def title_and_id(self): return self.title
-    def title_or_id(self): return self.title
-
     def __init__(self):
         # Initialize users
         uf=UserFolder()
@@ -81,20 +95,28 @@ class Application(Globals.ApplicationDefaultPermissions,
         cpl=ApplicationManager()
         cpl._init()
         self._setObject('Control_Panel', cpl)
-        get_transaction().note("Created Zope Application")
+        transaction.get().note("Created Zope Application")
 
     def id(self):
         try:    return self.REQUEST['SCRIPT_NAME'][1:]
         except: return self.title
 
-    def __class_init__(self): Globals.default__class_init__(self)
+    def title_and_id(self):
+        return self.title
 
-    def PrincipiaRedirect(self,destination,URL1):
+    def title_or_id(self):
+        return self.title
+
+    def __class_init__(self):
+        InitializeClass(self)
+
+    def PrincipiaRedirect(self, destination, URL1):
         """Utility function to allow user-controlled redirects"""
         if destination.find('//') >= 0:
             raise RedirectException, destination
         raise RedirectException, ("%s/%s" % (URL1, destination))
-    Redirect=ZopeRedirect=PrincipiaRedirect
+
+    Redirect = ZopeRedirect = PrincipiaRedirect
 
     def __bobo_traverse__(self, REQUEST, name=None):
 
@@ -115,9 +137,10 @@ class Application(Globals.ApplicationDefaultPermissions,
     def PrincipiaTime(self, *args):
         """Utility function to return current date/time"""
         return apply(DateTime, args)
-    ZopeTime=PrincipiaTime
 
-    ZopeAttributionButton__roles__=None
+    ZopeTime = PrincipiaTime
+
+    security.declarePublic('ZopeAttributionButton')
     def ZopeAttributionButton(self):
         """Returns an HTML fragment that displays the 'powered by zope'
         button along with a link to the Zope site."""
@@ -137,10 +160,11 @@ class Application(Globals.ApplicationDefaultPermissions,
         raise Forbidden, 'This resource cannot be moved.'
 
     test_url___allow_groups__=None
-    test_url=ZopeAttributionButton
+    test_url = ZopeAttributionButton
 
     def absolute_url(self, relative=0):
-        '''The absolute URL of the root object is BASE1 or "/".'''
+        """The absolute URL of the root object is BASE1 or "/".
+        """
         if relative: return ''
         try:
             # Take advantage of computed URL cache
@@ -149,27 +173,33 @@ class Application(Globals.ApplicationDefaultPermissions,
             return '/'
 
     def absolute_url_path(self):
-        '''The absolute URL path of the root object is BASEPATH1 or "/".'''
+        """The absolute URL path of the root object is BASEPATH1 or "/".
+        """
         try:
             return self.REQUEST['BASEPATH1'] or '/'
         except (AttributeError, KeyError):
             return '/'
 
     def virtual_url_path(self):
-        '''The virtual URL path of the root object is empty.'''
+        """The virtual URL path of the root object is empty.
+        """
         return ''
 
+    def getPhysicalRoot(self):
+        return self
+
     def getPhysicalPath(self):
-        '''Returns a path that can be used to access this object again
-        later, for example in a copy/paste operation.  Designed to
-        be used with getPhysicalRoot().
-        '''
+        """Get the physical path of the object.
+
+        Returns a path (an immutable sequence of strings) that can be used to
+        access this object again later, for example in a copy/paste operation.
+        getPhysicalRoot() and getPhysicalPath() are designed to operate
+        together.
+        """
         # We're at the base of the path.
         return ('',)
 
-    def getPhysicalRoot(self): return self
-
-    fixupZClassDependencies__roles__=()
+    security.declarePrivate('fixupZClassDependencies')
     def fixupZClassDependencies(self, rebuild=0):
         # Note that callers should not catch exceptions from this method
         # to ensure that the transaction gets aborted if the registry
@@ -206,9 +236,7 @@ class Application(Globals.ApplicationDefaultPermissions,
                             ob._register()
                             result=1
                             if not rebuild:
-                                LOG('Zope', INFO,
-                                    'Registered ZClass: %s' % ob.id
-                                    )
+                                LOG.info('Registered ZClass: %s' % ob.id)
                     # Include subobjects.
                     if hasattr(base, 'objectItems'):
                         m = list(ob.objectItems())
@@ -221,13 +249,12 @@ class Application(Globals.ApplicationDefaultPermissions,
                             m = list(ps.methods.objectItems())
                             items.extend(m)
                 except:
-                    LOG('Zope', WARNING,
-                        'Broken objects exist in product %s.' % product.id,
-                        error=sys.exc_info())
+                    LOG.warn('Broken objects exist in product %s.' % product.id,
+                             exc_info=sys.exc_info())
 
         return result
 
-    checkGlobalRegistry__roles__=()
+    security.declarePrivate('checkGlobalRegistry')
     def checkGlobalRegistry(self):
         """Check the global (zclass) registry for problems, which can
         be caused by things like disk-based products being deleted.
@@ -235,28 +262,32 @@ class Application(Globals.ApplicationDefaultPermissions,
         try:
             keys=list(self._p_jar.root()['ZGlobals'].keys())
         except:
-            LOG('Zope', ERROR,
+            LOG.error(
                 'A problem was found when checking the global product '\
                 'registry.  This is probably due to a Product being '\
                 'uninstalled or renamed.  The traceback follows.',
-                error=sys.exc_info())
+                exc_info=sys.exc_info())
             return 1
         return 0
 
-    _setInitializerRegistry__roles__ = ()
+    security.declarePrivate('_setInitializerFlag')
     def _setInitializerFlag(self, flag):
         if self._initializer_registry is None:
             self._initializer_registry = {}
         self._initializer_registry[flag] = 1
 
-    _getInitializerRegistry__roles__ = ()
+    security.declarePrivate('_getInitializerFlag')
     def _getInitializerFlag(self, flag):
         reg = self._initializer_registry
         if reg is None:
             reg = {}
         return reg.get(flag)
 
+InitializeClass(Application)
+
+
 class Expired(Globals.Persistent):
+
     icon='p_/broken'
 
     def __setstate__(self, s={}):
@@ -277,7 +308,9 @@ def initialize(app):
     initializer = AppInitializer(app)
     initializer.initialize()
 
+
 class AppInitializer:
+
     """ Initialze an Application object (called at startup) """
 
     def __init__(self, app):
@@ -288,8 +321,8 @@ class AppInitializer:
         return self.app[0]
 
     def commit(self, note):
-        get_transaction().note(note)
-        get_transaction().commit()
+        transaction.get().note(note)
+        transaction.commit()
 
     def initialize(self):
         app = self.getApp()
@@ -324,10 +357,10 @@ class AppInitializer:
 
     def install_tempfolder_and_sdc(self):
         app = self.getApp()
-        from Products.ZODBMountPoint.MountedObject \
-             import manage_addMounts, MountedObject
-        from Products.ZODBMountPoint.MountedObject \
-             import getConfiguration as getDBTabConfiguration
+        from Products.ZODBMountPoint.MountedObject import manage_addMounts,\
+             MountedObject
+        from Products.ZODBMountPoint.MountedObject import getConfiguration as \
+             getDBTabConfiguration
 
         dbtab_config = getDBTabConfiguration()
 
@@ -350,10 +383,9 @@ class AppInitializer:
                 mount_paths = [ x[0] for x in dbtab_config.listMountPaths() ]
                 if not '/temp_folder' in mount_paths:
                     # we won't be able to create the mount point properly
-                    LOG('Zope Default Object Creation', ERROR,
-                        ('Could not initialze a Temporary Folder because '
-                         'a database was not configured to be mounted at '
-                         'the /temp_folder mount point'))
+                    LOG.error('Could not initialze a Temporary Folder because '
+                              'a database was not configured to be mounted at '
+                              'the /temp_folder mount point')
                     return
                 try:
                     manage_addMounts(app, ('/temp_folder',))
@@ -361,10 +393,8 @@ class AppInitializer:
                     self.commit('Added temp_folder')
                     tf = app.temp_folder
                 except:
-                    LOG('Zope Default Object Creation', ERROR,
-                        ('Could not add a /temp_folder mount point due to an '
-                        'error.'),
-                        error=sys.exc_info())
+                    LOG.error('Could not add a /temp_folder mount point due to an '
+                              'error.', exc_info=sys.exc_info())
                     return
 
         # Ensure that there is a transient object container in the temp folder
@@ -379,7 +409,7 @@ class AppInitializer:
             default_period_secs = 20
             default_timeout_mins = 20
 
-            limit = getattr(config, 'maximum_number_of_session_objects', 
+            limit = getattr(config, 'maximum_number_of_session_objects',
                             default_limit)
             timeout_spec = getattr(config, 'session_timeout_minutes',
                                    default_timeout_mins)
@@ -387,25 +417,16 @@ class AppInitializer:
                                   default_period_secs)
 
             if addnotify and app.unrestrictedTraverse(addnotify, None) is None:
-                LOG('Zope Default Object Creation', WARNING,
-                    ('failed to use nonexistent "%s" script as '
-                     'session-add-notify-script-path' % addnotify))
+                LOG.warn('failed to use nonexistent "%s" script as '
+                         'session-add-notify-script-path' % addnotify)
                 addnotify=None
 
             if delnotify and app.unrestrictedTraverse(delnotify, None) is None:
-                LOG('Zope Default Object Creation', WARNING,
-                    ('failed to use nonexistent "%s" script as '
-                     'session-delete-notify-script-path' % delnotify))
+                LOG.warn('failed to use nonexistent "%s" script as '
+                         'session-delete-notify-script-path' % delnotify)
                 delnotify=None
 
-            toc = TransientObjectContainer(
-                'session_data', 'Session Data Container',
-                addNotification = addnotify,
-                delNotification = delnotify,
-                limit=limit,
-                period_secs=period_spec)
-
-            if timeout_spec is not None:
+            if 1:  # Preserve indentation for diff
                 toc = TransientObjectContainer('session_data',
                                                'Session Data Container',
                                                timeout_mins = timeout_spec,
@@ -431,6 +452,10 @@ class AppInitializer:
             from Products.Sessions.BrowserIdManager import BrowserIdManager
             bid = BrowserIdManager('browser_id_manager', 'Browser Id Manager')
             app._setObject('browser_id_manager', bid)
+            # FIXME explicitely call manage_afterAdd, as sometimes
+            # events are initialized too late
+            browser_id_manager = app.browser_id_manager
+            browser_id_manager.manage_afterAdd(browser_id_manager, app)
             app._setInitializerFlag('browser_id_manager')
             self.commit('Added browser_id_manager')
 
@@ -447,6 +472,10 @@ class AppInitializer:
                 path='/temp_folder/session_data',
                 requestName='SESSION')
             app._setObject('session_data_manager', sdm)
+            # FIXME explicitely call manage_afterAdd, as sometimes
+            # events are initialized too late
+            session_data_manager = app.session_data_manager
+            session_data_manager.manage_afterAdd(session_data_manager, app)
             app._setInitializerFlag('session_data_manager')
             self.commit('Added session_data_manager')
 
@@ -495,6 +524,10 @@ class AppInitializer:
             from Products.SiteErrorLog.SiteErrorLog import SiteErrorLog
             error_log = SiteErrorLog()
             app._setObject('error_log', error_log)
+            # FIXME explicitely call manage_afterAdd, as sometimes
+            # events are initialized too late
+            error_log = app.error_log
+            error_log.manage_afterAdd(error_log, app)
             app._setInitializerFlag('error_log')
             self.commit('Added site error_log at /error_log')
 
@@ -526,19 +559,19 @@ class AppInitializer:
         bad_things=0
         try:
             if app.checkGlobalRegistry():
-                LOG('Zope', INFO,
+                LOG.info(
                     'Beginning attempt to rebuild the global ZClass registry.')
                 app.fixupZClassDependencies(rebuild=1)
                 did_fixups=1
-                LOG('Zope', INFO,
+                LOG.info(
                     'The global ZClass registry has successfully been rebuilt.')
-                get_transaction().note('Rebuilt global product registry')
-                get_transaction().commit()
+                transaction.get().note('Rebuilt global product registry')
+                transaction.commit()
         except:
             bad_things=1
-            LOG('Zope', ERROR, 'The attempt to rebuild the registry failed.',
-                error=sys.exc_info())
-            get_transaction().abort()
+            LOG.error('The attempt to rebuild the registry failed.',
+                       exc_info=True)
+            transaction.abort()
 
         # Now we need to see if any (disk-based) products were installed
         # during intialization. If so (and the registry has no errors),
@@ -553,19 +586,16 @@ class AppInitializer:
             # product was added or updated and we are not a ZEO client.
             if getattr(Globals, '__disk_product_installed__', None):
                 try:
-                    LOG('Zope', INFO,
-                        ('New disk product detected, determining if we need '
-                        'to fix up any ZClasses.'))
+                    LOG.info('New disk product detected, determining if we need '
+                             'to fix up any ZClasses.')
                     if app.fixupZClassDependencies():
-                        LOG('Zope',INFO,
-                            'Repaired broken ZClass dependencies.')
+                        LOG.info('Repaired broken ZClass dependencies.')
                         self.commit('Repaired broked ZClass dependencies')
                 except:
-                    LOG('Zope', ERROR,
-                        ('Attempt to fixup ZClass dependencies after '
-                         'detecting an updated disk-based product failed.'),
-                        error=sys.exc_info())
-                    get_transaction().abort()
+                    LOG.error('Attempt to fixup ZClass dependencies after '
+                              'detecting an updated disk-based product failed.',
+                              exc_info=sys.exc_info())
+                    transaction.abort()
 
     def install_products(self):
         app = self.getApp()
@@ -587,8 +617,8 @@ def install_products(app):
 
     debug_mode = App.config.getConfiguration().debug_mode
 
-    get_transaction().note('Prior to product installs')
-    get_transaction().commit()
+    transaction.get().note('Prior to product installs')
+    transaction.commit()
 
     products = get_products()
 
@@ -603,8 +633,14 @@ def install_products(app):
         install_product(app, product_dir, product_name, meta_types,
                         folder_permissions, raise_exc=debug_mode)
 
+    # Delayed install of packages-as-products
+    for module, init_func in getattr(Products, '_packages_to_initialize', []):
+        install_package(app, module, init_func, raise_exc=debug_mode)
+    if hasattr(Products, '_packages_to_initialize'):
+        del Products._packages_to_initialize
+
     Products.meta_types=Products.meta_types+tuple(meta_types)
-    Globals.default__class_init__(Folder.Folder)
+    InitializeClass(Folder.Folder)
 
 def get_products():
     """ Return a list of tuples in the form:
@@ -641,10 +677,10 @@ def import_products():
 
     for priority, product_name, index, product_dir in products:
         if done.has_key(product_name):
-            LOG('OFS.Application', WARNING, 'Duplicate Product name',
-                'After loading Product %s from %s,\n'
-                'I skipped the one in %s.\n' % (
-                `product_name`, `done[product_name]`, `product_dir`) )
+            LOG.warn('Duplicate Product name: '
+                     'After loading Product %s from %s, '
+                     'I skipped the one in %s.' % (
+                    `product_name`, `done[product_name]`, `product_dir`) )
             continue
         done[product_name]=product_dir
         import_product(product_dir, product_name, raise_exc=debug_mode)
@@ -679,8 +715,8 @@ def import_product(product_dir, product_name, raise_exc=0, log_exc=1):
         except:
             exc = sys.exc_info()
             if log_exc:
-                LOG('Zope', ERROR, 'Could not import %s' % pname,
-                    error=exc)
+                LOG.error('Could not import %s' % pname,
+                          exc_info=exc)
             f=StringIO()
             traceback.print_exc(100,f)
             f=f.getvalue()
@@ -706,7 +742,6 @@ def install_product(app, product_dir, product_name, meta_types,
     path_join=os.path.join
     isdir=os.path.isdir
     exists=os.path.exists
-    DictType=type({})
     global_dict=globals()
     silly=('__doc__',)
 
@@ -727,7 +762,7 @@ def install_product(app, product_dir, product_name, meta_types,
             # like icon images.
             misc_=pgetattr(product, 'misc_', {})
             if misc_:
-                if type(misc_) is DictType:
+                if isinstance(misc_, dict):
                     misc_=Misc_(product_name, misc_)
                 Application.misc_.__dict__[product_name]=misc_
 
@@ -742,39 +777,36 @@ def install_product(app, product_dir, product_name, meta_types,
                 product, product_name, package_dir, app)
             context=ProductContext(productObject, app, product)
 
-            # Look for an 'initialize' method in the product. If it does
-            # not exist, then this is an old product that has never been
-            # updated. In that case, we will analyze the product and
-            # build up enough information to do initialization manually.
+            # Look for an 'initialize' method in the product.
             initmethod=pgetattr(product, 'initialize', None)
             if initmethod is not None:
                 initmethod(context)
 
-            # Support old-style product metadata. Older products may
-            # define attributes to name their permissions, meta_types,
-            # constructors, etc.
             permissions={}
             new_permissions={}
-            for p in pgetattr(product, '__ac_permissions__', ()):
-                permission, names, default = (
-                    tuple(p)+('Manager',))[:3]
-                if names:
-                    for name in names:
-                        permissions[name]=permission
-                elif not folder_permissions.has_key(permission):
-                    new_permissions[permission]=()
+            if pgetattr(product, '__ac_permissions__', None) is not None:
+                warn("__init__.py of %s has a long deprecated "
+                     "'__ac_permissions__' attribute. '__ac_permissions__' "
+                     "is now ignored by install_product. Please use "
+                     "registerClass instead."
+                     % product.__name__,
+                     DeprecationWarning)
 
-            for meta_type in pgetattr(product, 'meta_types', ()):
-                # Modern product initialization via a ProductContext
-                # adds 'product' and 'permission' keys to the meta_type
-                # mapping. We have to add these here for old products.
-                pname=permissions.get(meta_type['action'], None)
-                if pname is not None:
-                    meta_type['permission']=pname
-                meta_type['product']=productObject.id
-                meta_type['visibility'] = 'Global'
-                meta_types.append(meta_type)
+            if pgetattr(product, 'meta_types', None) is not None:
+                warn("__init__.py of %s has a long deprecated 'meta_types' "
+                     "attribute. 'meta_types' is now ignored by "
+                     "install_product. Please use registerClass instead."
+                     % product.__name__,
+                     DeprecationWarning)
 
+            if pgetattr(product, 'methods', None) is not None:
+                warn("__init__.py of %s has a long deprecated 'methods' "
+                     "attribute. 'methods' support might be removed in Zope "
+                     "2.11 or a later feature release. Please use the "
+                     "'legacy' argument of registerClass instead if the "
+                     "methods are constructors. Or refactor the product "
+                     "using adapters." % product.__name__,
+                     DeprecationWarning)
             for name,method in pgetattr(
                 product, 'methods', {}).items():
                 if not hasattr(Folder.Folder, name):
@@ -798,19 +830,47 @@ def install_product(app, product_dir, product_name, meta_types,
                     list(Folder.Folder.__ac_permissions__)+new_permissions)
 
             if not doInstall():
-                get_transaction().abort()
+                transaction.abort()
             else:
-                get_transaction().note('Installed product '+product_name)
-                get_transaction().commit()
+                transaction.get().note('Installed product '+product_name)
+                transaction.commit()
 
         except:
             if log_exc:
-                LOG('Zope',ERROR,'Couldn\'t install %s' % product_name,
-                    error=sys.exc_info())
-            get_transaction().abort()
+                LOG.error('Couldn\'t install %s' % product_name,
+                           exc_info=sys.exc_info())
+            transaction.abort()
             if raise_exc:
                 raise
 
+
+def install_package(app, module, init_func, raise_exc=False, log_exc=True):
+    """Installs a Python package like a product."""
+    try:
+        product = App.Product.initializeProduct(module,
+                                                module.__name__,
+                                                module.__path__[0],
+                                                app)
+        product.package_name = module.__name__
+        if init_func is not None:
+            newContext = ProductContext(product, app, module)
+            init_func(newContext)
+
+        if not doInstall():
+            transaction.abort()
+        else:
+            transaction.get().note('Installed package %s' % module.__name__)
+            transaction.commit()
+
+    except:
+        if log_exc:
+            LOG.error("Couldn't install %s" % module.__name__,
+                      exc_info=True)
+        transaction.abort()
+        if raise_exc:
+            raise
+
+
 def install_standards(app):
     # Check to see if we've already done this before
     # Don't do it twice (Casey)
@@ -836,6 +896,11 @@ def install_standards(app):
                 continue
             app.manage_addProduct['PageTemplates'].manage_addPageTemplate(
                 id=fn, title='', text=open(ob.filename))
+        elif ext in ('.ico', '.gif', '.png'):
+            if hasattr(app, fn):
+                continue
+            app.manage_addProduct['OFSP'].manage_addImage(
+                id=fn, title='', file=open(os.path.join(std_dir, fn)))
         else:
             continue
         wrote = 1
@@ -844,15 +909,15 @@ def install_standards(app):
         #setattr(Application, fn, ob)
     if wrote:
         app._standard_objects_have_been_added = 1
-        get_transaction().note('Installed standard objects')
-        get_transaction().commit()
+        transaction.get().note('Installed standard objects')
+        transaction.commit()
 
 def reinstall_product(app, product_name):
     folder_permissions = get_folder_permissions()
     meta_types=[]
 
-    get_transaction().note('Prior to product reinstall')
-    get_transaction().commit()
+    transaction.get().note('Prior to product reinstall')
+    transaction.commit()
 
     for product_dir in Products.__path__:
         product_names=os.listdir(product_dir)
@@ -864,7 +929,7 @@ def reinstall_product(app, product_name):
             break
 
     Products.meta_types=Products.meta_types+tuple(meta_types)
-    Globals.default__class_init__(Folder.Folder)
+    InitializeClass(Folder.Folder)
 
 
 def reimport_product(product_name):
@@ -878,9 +943,8 @@ def reimport_product(product_name):
 
 
 def removeProductMetaTypes(pid):
-    '''
-    Unregisters the meta types registered by a product.
-    '''
+    """Unregisters the meta types registered by a product.
+    """
     meta_types = Products.meta_types
     new_mts = []
     changed = 0
