@@ -12,7 +12,7 @@
 ##############################################################################
 """DTML Method objects.
 
-$Id: DTMLMethod.py 73388 2007-03-20 09:02:28Z mj $
+$Id: DTMLMethod.py 73386 2007-03-20 08:50:24Z mj $
 """
 import History
 from Globals import HTML, DTMLFile, MessageDialog
@@ -36,7 +36,7 @@ from AccessControl.Permissions import change_proxy_roles
 from AccessControl.Permissions import view as View
 from AccessControl.Permissions import ftp_access
 from AccessControl.DTML import RestrictedDTML
-from AccessControl.requestmethod import postonly
+from AccessControl.requestmethod import requestmethod
 from Cache import Cacheable
 from zExceptions import Forbidden
 from zExceptions.TracebackSupplement import PathTracebackSupplement
@@ -328,7 +328,7 @@ class DTMLMethod(RestrictedDTML, HTML, Acquisition.Implicit, RoleManager,
 
 
     security.declareProtected(change_proxy_roles, 'manage_proxy')
-    @postonly
+    @requestmethod('POST')
     def manage_proxy(self, roles=(), REQUEST=None):
         "Change Proxy Roles"
         self._validateProxy(REQUEST, roles)
