@@ -12,7 +12,7 @@
 ##############################################################################
 """Object Manager
 
-$Id: ObjectManager.py 88041 2008-07-05 16:12:08Z tseaver $
+$Id: ObjectManager.py 89922 2008-08-17 12:19:09Z andreasjung $
 """
 
 import warnings
@@ -55,6 +55,8 @@ from zope.event import notify
 from zope.app.container.contained import ObjectAddedEvent
 from zope.app.container.contained import ObjectRemovedEvent
 from zope.app.container.contained import notifyContainerModified
+from zope.app.container.interfaces import IContainer
+from zope.interface import implements
 from OFS.event import ObjectWillBeAddedEvent
 from OFS.event import ObjectWillBeRemovedEvent
 import OFS.subscribers
@@ -140,7 +142,10 @@ class ObjectManager(
     This class provides core behavior for collections of heterogeneous objects.
     """
 
-    implements(IObjectManager)
+    # The claim to implement IContainer has been made during the Zope3
+    # integration project called Five but hasn't been completed in full.
+
+    implements(IObjectManager, IContainer)
 
     security = ClassSecurityInfo()
     security.declareObjectProtected(access_contents_information)
@@ -648,6 +653,7 @@ class ObjectManager(
                 continue
             listing += [f for f in os.listdir(directory)
                         if f.endswith('.zexp') or f.endswith('.xml')]
+        listing.sort()
         return listing
 
     # FTP support methods
