@@ -16,11 +16,15 @@
 Directives to emulate the 'http://namespaces.zope.org/browser'
 namespace in ZCML known from zope.app.
 
-$Id: metaconfigure.py 69843 2006-08-29 14:48:01Z alecm $
+$Id: metaconfigure.py 77058 2007-06-25 15:42:39Z tseaver $
 """
 import os
+from inspect import ismethod
 
+from zope import component
 from zope.interface import Interface
+from zope.component.zcml import handler
+from zope.component.interface import provideInterface
 from zope.configuration.exceptions import ConfigurationError
 from zope.publisher.interfaces.browser import IBrowserRequest, \
      IDefaultBrowserLayer
@@ -29,16 +33,16 @@ from zope.app.publisher.browser.viewmeta import pages as zope_app_pages
 from zope.app.publisher.browser.viewmeta import view as zope_app_view
 from zope.app.publisher.browser.viewmeta import providesCallable, \
      _handle_menu, _handle_for
-from zope.app.component.metaconfigure import handler
-from zope.app.component.interface import provideInterface
 
 from Products.Five.browser import BrowserView
-from Products.Five.browser.resource import FileResourceFactory, ImageResourceFactory
+from Products.Five.browser.resource import FileResourceFactory
+from Products.Five.browser.resource import ImageResourceFactory
 from Products.Five.browser.resource import PageTemplateResourceFactory
 from Products.Five.browser.resource import DirectoryResourceFactory
 from Products.Five.browser.pagetemplatefile import ZopeTwoPageTemplateFile
 from Products.Five.metaclass import makeClass
 from Products.Five.security import getSecurityInfo, protectClass, protectName
+from Products.Five.security import CheckerPrivateId
 
 from Globals import InitializeClass as initializeClass
 
@@ -56,7 +60,7 @@ def page(_context, name, permission, for_,
         allowed_attributes = []
     if allowed_interface is not None:
         for interface in allowed_interface:
-            allowed_attributes.extend(interface.names())
+            allowed_attributes.extend(interface.names(all=True))
 
     if attribute != '__call__':
         if template:
@@ -121,8 +125,8 @@ def page(_context, name, permission, for_,
     _context.action(
         discriminator = ('view', for_, name, IBrowserRequest, layer),
         callable = handler,
-        args = ('provideAdapter',
-                (for_, layer), Interface, name, new_class, _context.info),
+        args = ('registerAdapter',
+                new_class, (for_, layer), Interface, name, _context.info),
         )
     _context.action(
         discriminator = ('five:protectClass', new_class),
@@ -136,6 +140,19 @@ def page(_context, name, permission, for_,
                 callable = protectName,
                 args = (new_class, attr, permission)
                 )
+    # Make everything else private
+    allowed = [attribute] + (allowed_attributes or [])
+    private_attrs = [name for name in dir(new_class)
+                     if (not name.startswith('_')) and
+                        (name not in allowed) and
+                        ismethod(getattr(new_class, name))]
+    for attr in private_attrs:
+        _context.action(
+            discriminator = ('five:protectName', new_class, attr),
+            callable = protectName,
+            args = (new_class, attr, CheckerPrivateId)
+            )
+    # Protect the class
     _context.action(
         discriminator = ('five:initialize:class', new_class),
         callable = initializeClass,
@@ -189,11 +206,12 @@ class view(zope_app_view):
 
                 if name in pages:
                     return getattr(self, pages[name])
-                view = zapi.queryView(self, name, request)
+                view = component.queryMultiAdapter((self, request), name=name,
+                                                   default=None)
                 if view is not None:
-                    return view
+                    return view.__of__(self)
 
-                m = class_.publishTraverse.__get__(self)
+                m = class_.publishTraverse.__get__(self).__of__(self)
                 return m(request, name)
 
         else:
@@ -202,9 +220,10 @@ class view(zope_app_view):
 
                 if name in pages:
                     return getattr(self, pages[name])
-                view = zapi.queryView(self, name, request)
+                view = component.queryMultiAdapter((self, request), name=name,
+                                                   default=None)
                 if view is not None:
-                    return view
+                    return view.__of__(self)
 
                 raise NotFoundError(self, name, request)
 
@@ -232,6 +251,7 @@ class view(zope_app_view):
         except:
             cname = "GeneratedClass"
             
+        cdict['__name__'] = name
         newclass = makeClass(cname, bases, cdict)
         
         _handle_for(_context, for_)
@@ -247,8 +267,8 @@ class view(zope_app_view):
             discriminator = ('view', for_, name, IBrowserRequest, layer,
                              self.provides),
             callable = handler,
-            args = ('provideAdapter',
-                    (for_, layer), self.provides, name, newclass,
+            args = ('registerAdapter',
+                    newclass, (for_, layer), self.provides, name,
                     _context.info),
             )
 
@@ -287,8 +307,8 @@ def resource(_context, name, layer=IDefaultBrowserLayer, permission='zope.Public
     _context.action(
         discriminator = ('resource', name, IBrowserRequest, layer),
         callable = handler,
-        args = ('provideAdapter',
-                (layer,), Interface, name, factory, _context.info),
+        args = ('registerAdapter',
+                factory, (layer,), Interface, name, _context.info),
         )
     _context.action(
         discriminator = ('five:protectClass', new_class),
@@ -354,8 +374,8 @@ def resourceDirectory(_context, name, directory, layer=IDefaultBrowserLayer,
     _context.action(
         discriminator = ('resource', name, IBrowserRequest, layer),
         callable = handler,
-        args = ('provideAdapter',
-                (layer,), Interface, name, factory, _context.info),
+        args = ('registerAdapter',
+                factory, (layer,), Interface, name, _context.info),
         )
     for new_class in new_classes:
         _context.action(
