@@ -13,13 +13,14 @@
 ##############################################################################
 """Generic Components ZCML Handlers
 
-$Id: metaconfigure.py 69446 2006-08-13 19:01:27Z philikon $
+$Id: metaconfigure.py 69765 2006-08-24 14:39:34Z regebro $
 """
+import warnings
 from zope.configuration.exceptions import ConfigurationError
 from zope.app.component import contentdirective
 from Products.Five.security import protectName, initializeClass
 
-class ContentDirective(contentdirective.ContentDirective):
+class ClassDirective(contentdirective.ClassDirective):
 
     def __protectName(self, name, permission_id):
         self.__context.action(
@@ -43,3 +44,13 @@ class ContentDirective(contentdirective.ContentDirective):
             callable = initializeClass,
             args = (self.__class,)
             )
+
+# BBB 2006/02/24, to be removed after 12 months
+class ContentDirective(contentdirective.ClassDirective):
+
+    def __init__(self, _context, class_):
+        warnings.warn_explicit(
+            "The 'content' alias for the 'class' directive has been "
+            "deprecated and will be removed in Zope 2.12.\n",
+            DeprecationWarning, _context.info.file, _context.info.line)        
+        super(ContentDirective, self).__init__(_context, class_)
