@@ -13,19 +13,17 @@
 ##############################################################################
 """Five security handling
 
-$Id: security.py 61072 2005-10-31 17:43:51Z philikon $
+$Id: security.py 71270 2006-11-22 13:37:52Z yuppie $
 """
-from zope.interface import implements, classProvides
 from zope.component import queryUtility, getUtility
-from zope.security.management import thread_local
+from zope.interface import implements, classProvides
 from zope.security.interfaces import IInteraction, ISecurityPolicy
+from zope.security.interfaces import IPermission
+from zope.security.management import thread_local
 from zope.security.simplepolicies import ParanoidSecurityPolicy
-from zope.app.security.interfaces import IPermission
-from zope.app import zapi
 
 from AccessControl import ClassSecurityInfo, getSecurityManager
 from Globals import InitializeClass as initializeClass
-from types import StringTypes
 
 CheckerPublicId = 'zope.Public'
 CheckerPrivateId = 'zope2.Private'
@@ -66,8 +64,8 @@ def checkPermission(permission, object, interaction=None):
         permission is None or permission is CheckerPublic):
         return True
 
-    if isinstance(permission, StringTypes):
-        permission = zapi.queryUtility(IPermission, unicode(permission))
+    if isinstance(permission, basestring):
+        permission = queryUtility(IPermission, unicode(permission))
         if permission is None:
             return False
 
