@@ -12,33 +12,46 @@
 ##############################################################################
 """SMTP mail objects
 
-$Id: MailHost.py 68645 2006-06-15 09:44:48Z andreasjung $
+$Id: MailHost.py 88311 2008-07-13 10:53:06Z andreasjung $
 """
 
 import mimetools
 import rfc822
+import time
+import logging
 from cStringIO import StringIO
-from smtplib import SMTP
+from threading import Lock
 
 import Acquisition
 import OFS.SimpleItem
 from AccessControl import ClassSecurityInfo
-from AccessControl.Permissions import change_configuration
+from AccessControl.Permissions import change_configuration, view
 from AccessControl.Permissions import use_mailhost_services
 from AccessControl.Permissions import view_management_screens
 from AccessControl.Role import RoleManager
 from Globals import Persistent, DTMLFile, InitializeClass
 from DateTime import DateTime
+
 from zope.interface import implements
+from zope.sendmail.maildir import Maildir
+from zope.sendmail.delivery import DirectMailDelivery, QueuedMailDelivery, \
+                            QueueProcessorThread
 
 from interfaces import IMailHost
+from decorator import synchronized
 
+# Use our own TLS/SSL-aware mailer since the zope.sendmail does
+# not support TLS/SSL in this version (should be replaced with
+# the next version)
+from mailer import SMTPMailer
+
+queue_threads = {}  # maps MailHost path -> queue processor threada
+
+LOG = logging.getLogger('MailHost')
 
 class MailHostError(Exception):
-
     pass
 
-
 manage_addMailHostForm=DTMLFile('dtml/addMailHost_form', globals())
 def manage_addMailHost( self, id, title='', smtp_host='localhost'
                       , localhost='localhost', smtp_port=25
@@ -54,8 +67,7 @@ add = manage_addMailHost
 
 
 class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
-
-    'a mailhost...?'
+    """a mailhost...?"""
 
     implements(IMailHost)
 
@@ -66,8 +78,13 @@ class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
     security = ClassSecurityInfo()
     smtp_uid='' # Class attributes for smooth upgrades
     smtp_pwd=''
+    smtp_queue = False
+    smtp_queue_directory = '/tmp'
+    force_tls = False
+    lock = Lock()
 
-    timeout=1.0
+    # timeout=1.0 # unused?
+    
 
     manage_options=(
         (
@@ -80,8 +97,9 @@ class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
         )
 
 
-    def __init__( self, id='', title='', smtp_host='localhost', smtp_port=25, 
-                  smtp_uid='', smtp_pwd=''):
+    def __init__(self, id='', title='', smtp_host='localhost', smtp_port=25, 
+                 force_tls=False, 
+                 smtp_uid='', smtp_pwd='', smtp_queue=False, smtp_queue_directory='/tmp'):
         """Initialize a new MailHost instance """
         self.id = id
         self.title = title
@@ -89,6 +107,9 @@ class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
         self.smtp_port = int(smtp_port)
         self.smtp_uid = smtp_uid
         self.smtp_pwd = smtp_pwd
+        self.force_tls = force_tls
+        self.smtp_queue = smtp_queue
+        self.smtp_queue_directory = smtp_queue_directory
 
 
     # staying for now... (backwards compatibility)
@@ -97,7 +118,10 @@ class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
         self.smtp_port=smtp_port
 
     security.declareProtected(change_configuration, 'manage_makeChanges')
-    def manage_makeChanges(self,title,smtp_host,smtp_port,smtp_uid='',smtp_pwd='', REQUEST=None):
+    def manage_makeChanges(self,title,smtp_host,smtp_port,smtp_uid='',smtp_pwd='', 
+                           smtp_queue=False, smtp_queue_directory='/tmp',
+                           force_tls=False, 
+                           REQUEST=None):
         'make the changes'
 
         title=str(title)
@@ -109,6 +133,18 @@ class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
         self.smtp_port=smtp_port
         self.smtp_uid = smtp_uid
         self.smtp_pwd = smtp_pwd
+        self.force_tls = force_tls
+        self.smtp_queue = smtp_queue
+        self.smtp_queue_directory = smtp_queue_directory
+
+        # restart queue processor thread 
+        if self.smtp_queue:
+            self._stopQueueProcessorThread() 
+            self._startQueueProcessorThread() 
+        else:
+            self._stopQueueProcessorThread() 
+
+
         if REQUEST is not None:
             msg = 'MailHost %s updated' % self.id
             return self.manage_main( self
@@ -119,15 +155,16 @@ class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
     security.declareProtected(use_mailhost_services, 'sendTemplate')
     def sendTemplate(trueself, self, messageTemplate,
                      statusTemplate=None, mto=None, mfrom=None,
-                     encode=None, REQUEST=None):
+                     encode=None, REQUEST=None, immediate=False):
         'render a mail template, then send it...'
         mtemplate = getattr(self, messageTemplate)
         messageText = mtemplate(self, trueself.REQUEST)
         messageText, mto, mfrom = _mungeHeaders( messageText, mto, mfrom)
         messageText=_encode(messageText, encode)
-        trueself._send(mfrom, mto, messageText)
+        trueself._send(mfrom, mto, messageText, immediate)
 
-        if not statusTemplate: return "SEND OK"
+        if not statusTemplate: 
+            return "SEND OK"
 
         try:
             stemplate=getattr(self, statusTemplate)
@@ -137,11 +174,11 @@ class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
 
     security.declareProtected(use_mailhost_services, 'send')
     def send(self, messageText, mto=None, mfrom=None, subject=None,
-             encode=None):
+             encode=None, immediate=False):
 
         messageText, mto, mfrom = _mungeHeaders( messageText, mto, mfrom, subject)
         messageText = _encode(messageText, encode)
-        self._send(mfrom, mto, messageText)
+        self._send(mfrom, mto, messageText, immediate)
 
     # This is here for backwards compatibility only. Possibly it could
     # be used to send messages at a scheduled future time, or via a mail queue?
@@ -149,27 +186,108 @@ class MailBase(Acquisition.Implicit, OFS.SimpleItem.Item, RoleManager):
     scheduledSend = send
 
     security.declareProtected(use_mailhost_services, 'simple_send')
-    def simple_send(self, mto, mfrom, subject, body):
+    def simple_send(self, mto, mfrom, subject, body, immediate=False):
         body="From: %s\nTo: %s\nSubject: %s\n\n%s" % (
             mfrom, mto, subject, body)
 
-        self._send( mfrom, mto, body )
+        self._send(mfrom, mto, body, immediate)
+
+
+    def _makeMailer(self):
+        """ Create a SMTPMailer """
+        return SMTPMailer(hostname=self.smtp_host,
+                          port=int(self.smtp_port),
+                          username=self.smtp_uid or None,
+                          password=self.smtp_pwd or None,
+                          force_tls=self.force_tls
+                          )
+
+    @synchronized(lock)
+    def _stopQueueProcessorThread(self):
+        """ Stop thread for processing the mail queue """
+
+        path = self.absolute_url(1)
+        if queue_threads.has_key(path):
+            thread = queue_threads[path]
+            thread.stop()
+            while thread.isAlive():
+                # wait until thread is really dead
+                time.sleep(0.3)
+            del queue_threads[path]
+            LOG.info('Thread for %s stopped' % path)
+
+    @synchronized(lock)
+    def _startQueueProcessorThread(self):
+        """ Start thread for processing the mail queue """
+
+        path = self.absolute_url(1)
+        if not queue_threads.has_key(path):
+            thread = QueueProcessorThread()
+            thread.setMailer(self._makeMailer())
+            thread.setQueuePath(self.smtp_queue_directory)
+            thread.start()
+            queue_threads[path] = thread     
+            LOG.info('Thread for %s started' % path)
+
+    security.declareProtected(view, 'queueLength')
+    def queueLength(self):
+        """ return length of mail queue """
+
+        try:
+            maildir = Maildir(self.smtp_queue_directory)
+            return len([item for item in maildir])
+        except ValueError:
+            return 'n/a - %s is not a maildir - please verify your ' \
+                   'configuration' % self.smtp_queue_directory
+
+
+    security.declareProtected(view, 'queueThreadAlive')
+    def queueThreadAlive(self):
+        """ return True/False is queue thread is working """
+
+        th = queue_threads.get(self.absolute_url(1))
+        if th:
+            return th.isAlive()
+        return False
+
+    security.declareProtected(change_configuration, 'manage_restartQueueThread')
+    def manage_restartQueueThread(self, action='start', REQUEST=None):
+        """ Restart the queue processor thread """
+
+        if action == 'stop':
+            self._stopQueueProcessorThread()
+        elif action == 'start':
+            self._startQueueProcessorThread()
+        else:
+            raise ValueError('Unsupported action %s' % action)
+
+        if REQUEST is not None:
+            msg = 'Queue processor thread %s' % \
+                  (action == 'stop' and 'stopped' or 'started')
+            return self.manage_main(self, REQUEST, manage_tabs_message=msg)
+
 
     security.declarePrivate('_send')
-    def _send( self, mfrom, mto, messageText ):
+    def _send(self, mfrom, mto, messageText, immediate=False):
         """ Send the message """
-        smtpserver = SMTP(self.smtp_host, int(self.smtp_port) )
-        if self.smtp_uid:
-            smtpserver.login(self.smtp_uid, self.smtp_pwd)
-        smtpserver.sendmail( mfrom, mto, messageText )
-        smtpserver.quit()
+
+        if immediate:
+            self._makeMailer().send(mfrom, mto, messageText)
+        else:
+            if self.smtp_queue:
+                # Start queue processor thread, if necessary
+                self._startQueueProcessorThread()
+                delivery = QueuedMailDelivery(self.smtp_queue_directory)
+            else:
+                delivery = DirectMailDelivery(self._makeMailer())
+
+            delivery.send(mfrom, mto, messageText)
 
 InitializeClass(MailBase)
 
 
 class MailHost(Persistent, MailBase):
-
-    "persistent version"
+    """persistent version"""
 
 
 def _encode(body, encode=None):
