@@ -86,9 +86,6 @@ class PageTemplateFile(SimpleItem, Script, PageTemplate, Traversable):
 
         self.filename = filename
 
-        content = open(filename).read()
-        self.pt_edit( content, guess_type(filename, content))
-
     def pt_getContext(self):
         root = self.getPhysicalRoot()
         context = self._getContext()
