@@ -192,10 +192,28 @@ class LineEndingsTestCase(unittest.TestCase):
     def test_mac(self):
         self.assertEqual(self.runPTWithLineEndings('\r'), self.OUTPUT)
 
+
+class LazyLoadingTestCase(unittest.TestCase):
+
+    TEMPFILENAME = tempfile.mktemp(".zpt")
+
+    def tearDown(self):
+        if os.path.exists(self.TEMPFILENAME):
+            os.unlink(self.TEMPFILENAME)
+
+    def test_lazy(self):
+        f = open(self.TEMPFILENAME, 'w')
+        print >> f, 'Lazyness'
+        f.close()
+        pt = PageTemplateFile(self.TEMPFILENAME)
+        self.failUnless(not pt._text and not pt._v_program)
+
+
 def test_suite():
     return unittest.TestSuite((
         unittest.makeSuite(TypeSniffingTestCase),
         unittest.makeSuite(LineEndingsTestCase),
+        unittest.makeSuite(LazyLoadingTestCase),
     ))
 
 if __name__ == "__main__":
