@@ -1,9 +1,9 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,13 +12,13 @@
 ##############################################################################
 """Date index.
 
-$Id: DateIndex.py 40427 2005-11-30 12:32:05Z mj $
+$Id: DateIndex.py 72262 2007-01-30 18:58:05Z yuppie $
 """
 
 import time
+from logging import getLogger
 from datetime import date, datetime
 from datetime import tzinfo, timedelta
-from types import StringType, FloatType, IntType
 
 import BTrees.Length
 from BTrees.IIBTree import IISet, union, intersection, multiunion
@@ -28,7 +28,6 @@ from DateTime.DateTime import DateTime
 from Globals import DTMLFile
 from OFS.PropertyManager import PropertyManager
 from ZODB.POSException import ConflictError
-from zLOG import LOG, ERROR
 from zope.interface import implements
 
 from Products.PluginIndexes.common import safe_callable
@@ -36,6 +35,7 @@ from Products.PluginIndexes.common.UnIndex import UnIndex
 from Products.PluginIndexes.common.util import parseIndexRequest
 from Products.PluginIndexes.interfaces import IDateIndex
 
+LOG = getLogger('DateIndex')
 _marker = []
 
 ###############################################################################
@@ -89,7 +89,7 @@ class DateIndex(UnIndex, PropertyManager):
     implements(IDateIndex)
 
     meta_type = 'DateIndex'
-    query_options = ['query', 'range']
+    query_options = ('query', 'range')
 
     index_naive_time_as_local = True # False means index as UTC
     _properties=({'id':'index_naive_time_as_local',
@@ -144,10 +144,9 @@ class DateIndex(UnIndex, PropertyManager):
                     except ConflictError:
                         raise
                     except:
-                        LOG('UnIndex', ERROR,
-                            ("Should not happen: ConvertedDate was there,"
-                             " now it's not, for document with id %s" %
-                             documentId))
+                        LOG.error("Should not happen: ConvertedDate was there,"
+                                  " now it's not, for document with id %s" %
+                                  documentId)
 
             if ConvertedDate is not _marker:
                 self.insertForwardIndexEntry( ConvertedDate, documentId )
@@ -157,14 +156,14 @@ class DateIndex(UnIndex, PropertyManager):
 
         return returnStatus
 
-    def _apply_index( self, request, cid='', type=type ):
+    def _apply_index(self, request):
         """Apply the index to query parameters given in the argument
 
         Normalize the 'query' arguments into integer values at minute
         precision before querying.
         """
-        record = parseIndexRequest( request, self.id, self.query_options )
-        if record.keys == None:
+        record = parseIndexRequest(request, self.id, self.query_options)
+        if record.keys is None:
             return None
 
         keys = map( self._convert, record.keys )
@@ -215,22 +214,17 @@ class DateIndex(UnIndex, PropertyManager):
             else:
                 setlist = index.values(lo)
 
-            #for k, set in setlist:
-                #if type(set) is IntType:
-                    #set = IISet((set,))
-                #r = set_func(r, set)
-            # XXX: Use multiunion!
             r = multiunion(setlist)
 
         else: # not a range search
             for key in keys:
                 set = index.get(key, None)
                 if set is not None:
-                    if type(set) is IntType:
+                    if isinstance(set, int):
                         set = IISet((set,))
                     r = set_func(r, set)
 
-        if type(r) is IntType:
+        if isinstance(r, int):
             r = IISet((r,))
 
         if r is None:
@@ -242,20 +236,20 @@ class DateIndex(UnIndex, PropertyManager):
         """Convert Date/Time value to our internal representation"""
         # XXX: Code patched 20/May/2003 by Kiran Jonnalagadda to
         # convert dates to UTC first.
-        if isinstance( value, DateTime ):
+        if isinstance(value, DateTime):
             t_tup = value.toZone('UTC').parts()
-        elif type( value ) in (FloatType, IntType):
+        elif isinstance(value, (float, int)):
             t_tup = time.gmtime( value )
-        elif type( value ) is StringType and value:
+        elif isinstance(value, str) and value:
             t_obj = DateTime( value ).toZone('UTC')
             t_tup = t_obj.parts()
-        elif type( value ) is date:
-            t_tup = value.timetuple()
-        elif type( value ) is datetime:
+        elif isinstance(value, datetime):
             if self.index_naive_time_as_local and value.tzinfo is None:
                 value = value.replace(tzinfo=Local)
             # else if tzinfo is None, naive time interpreted as UTC
             t_tup = value.utctimetuple()
+        elif isinstance(value, date):
+            t_tup = value.timetuple()
         else:
             return default
 
