@@ -1,68 +1,69 @@
 ##############################################################################
 #
-# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
+# Copyright (c) 2002 Zope Corporation and Contributors. All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
-# FOR A PARTICULAR PURPOSE
+# FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
+"""Date range index.
 
-"""$Id: DateRangeIndex.py,v 1.6.2.5 2005/03/18 15:31:02 andreasjung Exp $
+$Id: DateRangeIndex.py 72262 2007-01-30 18:58:05Z yuppie $
 """
 
 import os
-import sys
 
-from Products.PluginIndexes import PluggableIndex
+from AccessControl import ClassSecurityInfo
+from BTrees.IIBTree import IISet, IITreeSet, union, intersection, multiunion
+from BTrees.IOBTree import IOBTree
+import BTrees.Length
+from DateTime.DateTime import DateTime
+from Globals import package_home, DTMLFile, InitializeClass
+from zope.interface import implements
+
+from Products.PluginIndexes.common import safe_callable
 from Products.PluginIndexes.common.UnIndex import UnIndex
 from Products.PluginIndexes.common.util import parseIndexRequest
-from Products.PluginIndexes.common import safe_callable
-from OFS.SimpleItem import SimpleItem
-
-import BTrees.Length
-from BTrees.IOBTree import IOBTree
-from BTrees.IIBTree import IISet, IITreeSet, union, intersection, multiunion
-
-from Globals import package_home, DTMLFile, InitializeClass
-from AccessControl import ClassSecurityInfo
-from DateTime.DateTime import DateTime
+from Products.PluginIndexes.interfaces import IDateRangeIndex
 
 _dtmldir = os.path.join( package_home( globals() ), 'dtml' )
 
 VIEW_PERMISSION         = 'View'
 INDEX_MGMT_PERMISSION   = 'Manage ZCatalogIndex Entries'
 
+
 class DateRangeIndex(UnIndex):
-    """
-        Index a date range, such as the canonical "effective-expiration"
-        range in the CMF.  Any object may return None for either the
-        start or the end date:  for the start date, this should be
-        the logical equivalent of "since the beginning of time";  for the
-        end date, "until the end of time".
 
-        Therefore, divide the space of indexed objects into four containers:
+    """Index for date ranges, such as the "effective-expiration" range in CMF.
 
-        - Objects which always match ( i.e., they returned None for both );
+    Any object may return None for either the start or the end date: for the
+    start date, this should be the logical equivalent of "since the beginning
+    of time"; for the end date, "until the end of time".
 
-        - Objects which match after a given time ( i.e., they returned None
-          for the end date );
+    Therefore, divide the space of indexed objects into four containers:
 
-        - Objects which match until a given time ( i.e., they returned None
-          for the start date );
+    - Objects which always match (i.e., they returned None for both);
 
-        - Objects which match only during a specific interval.
+    - Objects which match after a given time (i.e., they returned None for the
+      end date);
+
+    - Objects which match until a given time (i.e., they returned None for the
+      start date);
+
+    - Objects which match only during a specific interval.
     """
 
-    __implements__ = (PluggableIndex.UniqueValueIndex,
-                      PluggableIndex.SortIndex)
+    __implements__ = UnIndex.__implements__
+    implements(IDateRangeIndex)
 
     security = ClassSecurityInfo()
 
     meta_type = "DateRangeIndex"
+    query_options = ('query',)
 
     manage_options= ( { 'label'     : 'Properties'
                       , 'action'    : 'manage_indexProperties'
@@ -70,8 +71,6 @@ class DateRangeIndex(UnIndex):
                     ,
                     )
 
-    query_options = ['query']
-
     since_field = until_field = None
 
     def __init__(self, id, since_field=None, until_field=None,
@@ -85,27 +84,21 @@ class DateRangeIndex(UnIndex):
         self._edit(since_field, until_field)
         self.clear()
 
-    security.declareProtected( VIEW_PERMISSION
-                             , 'getSinceField'
-                             )
-    def getSinceField( self ):
-        """
+    security.declareProtected(VIEW_PERMISSION, 'getSinceField')
+    def getSinceField(self):
+        """Get the name of the attribute indexed as start date.
         """
         return self._since_field
 
-    security.declareProtected( VIEW_PERMISSION
-                             , 'getUntilField'
-                             )
-    def getUntilField( self ):
-        """
+    security.declareProtected(VIEW_PERMISSION, 'getUntilField')
+    def getUntilField(self):
+        """Get the name of the attribute indexed as end date.
         """
         return self._until_field
 
     manage_indexProperties = DTMLFile( 'manageDateRangeIndex', _dtmldir )
 
-    security.declareProtected( INDEX_MGMT_PERMISSION
-                             , 'manage_edit'
-                             )
+    security.declareProtected(INDEX_MGMT_PERMISSION, 'manage_edit')
     def manage_edit( self, since_field, until_field, REQUEST ):
         """
         """
@@ -115,7 +108,7 @@ class DateRangeIndex(UnIndex):
                                       % REQUEST.get('URL2')
                                       )
 
-    security.declarePrivate( '_edit' )
+    security.declarePrivate('_edit')
     def _edit( self, since_field, until_field ):
         """
             Update the fields used to compute the range.
@@ -123,22 +116,18 @@ class DateRangeIndex(UnIndex):
         self._since_field = since_field
         self._until_field = until_field
 
-
-    security.declareProtected( INDEX_MGMT_PERMISSION
-                             , 'clear'
-                             )
+    security.declareProtected(INDEX_MGMT_PERMISSION, 'clear')
     def clear( self ):
         """
             Start over fresh.
         """
-        try: self.__len__.set(0)
-        except AttributeError: self.__len__=BTrees.Length.Length()
         self._always        = IITreeSet()
         self._since_only    = IOBTree()
         self._until_only    = IOBTree()
         self._since         = IOBTree()
         self._until         = IOBTree()
         self._unindex       = IOBTree() # 'datum' will be a tuple of date ints
+        self._length        = BTrees.Length.Length()
 
     #
     #   PluggableIndexInterface implementation (XXX inherit assertions?)
@@ -223,7 +212,6 @@ class DateRangeIndex(UnIndex):
             t2 = self._until_only
 
         result = []
-        IntType = type( 0 )
 
         if not withLengths:
 
@@ -234,7 +222,7 @@ class DateRangeIndex(UnIndex):
 
             for key in t1.keys():
                 set = t1[ key ]
-                if type( set ) is IntType:
+                if isinstance(set, int):
                     length = 1
                 else:
                     length = len( set )
@@ -242,7 +230,7 @@ class DateRangeIndex(UnIndex):
 
             for key in t2.keys():
                 set = t2[ key ]
-                if type( set ) is IntType:
+                if isinstance(set, int):
                     length = 1
                 else:
                     length = len( set )
@@ -250,12 +238,12 @@ class DateRangeIndex(UnIndex):
 
         return tuple( result )
 
-    def _apply_index( self, request, cid='' ):
+    def _apply_index(self, request):
         """
             Apply the index to query parameters given in 'request', which
             should be a mapping object.
 
-            If the request does not contain the needed parametrs, then
+            If the request does not contain the needed parameters, then
             return None.
 
             If the request contains a parameter with the name of the
@@ -267,7 +255,7 @@ class DateRangeIndex(UnIndex):
             second object is a tuple containing the names of all data fields
             used.
         """
-        record = parseIndexRequest( request, self.getId() )
+        record = parseIndexRequest(request, self.id, self.query_options)
         if record.keys is None:
             return None
 
@@ -311,12 +299,15 @@ class DateRangeIndex(UnIndex):
     #
     #   ZCatalog needs this, although it isn't (yet) part of the interface.
     #
-    security.declareProtected( VIEW_PERMISSION , 'numObjects' )
+    security.declareProtected(VIEW_PERMISSION , 'numObjects')
     def numObjects( self ):
-        """
-        """
+        """ """
         return len( self._unindex )
 
+    def indexSize(self):
+        """ """
+        return len(self)
+
     #
     #   Helper functions.
     #
@@ -403,10 +394,10 @@ class DateRangeIndex(UnIndex):
     def _convertDateTime( self, value ):
         if value is None:
             return value
-        if type( value ) == type( '' ):
+        if isinstance(value, str):
             dt_obj = DateTime( value )
             value = dt_obj.millis() / 1000 / 60 # flatten to minutes
-        if isinstance( value, DateTime ):
+        if isinstance(value, DateTime):
             value = value.millis() / 1000 / 60 # flatten to minutes
         result = int( value )
         if isinstance(result, long): # this won't work (Python 2.3)
