@@ -12,17 +12,16 @@
 ##############################################################################
 """Path index.
 
-$Id: PathIndex.py 94052 2008-12-14 13:27:33Z mj $
+$Id: PathIndex.py 72262 2007-01-30 18:58:05Z yuppie $
 """
 
-from types import StringType, ListType, TupleType
 from logging import getLogger
 
 from Globals import Persistent, DTMLFile
 from OFS.SimpleItem import SimpleItem
 from BTrees.IOBTree import IOBTree
 from BTrees.OOBTree import OOBTree
-from BTrees.IIBTree import IITreeSet, IISet, intersection, union, multiunion
+from BTrees.IIBTree import IITreeSet, IISet, intersection, union
 from BTrees.Length import Length
 from zope.interface import implements
 
@@ -56,6 +55,7 @@ class PathIndex(Persistent, SimpleItem):
     implements(IPathIndex, IUniqueValueIndex)
 
     meta_type="PathIndex"
+    query_options = ('query', 'level', 'operator')
 
     manage_options= (
         {'label': 'Settings',
@@ -63,8 +63,6 @@ class PathIndex(Persistent, SimpleItem):
          'help': ('PathIndex','PathIndex_Settings.stx')},
     )
 
-    query_options = ("query", "level", "operator")
-
     def __init__(self,id,caller=None):
         self.id = id
         self.operators = ('or','and')
@@ -108,7 +106,7 @@ class PathIndex(Persistent, SimpleItem):
             else:
                 path = f
 
-            if not isinstance(path, (StringType, TupleType)):
+            if not isinstance(path, (str, tuple)):
                 raise TypeError('path value must be string or tuple of strings')
         else:
             try:
@@ -116,7 +114,7 @@ class PathIndex(Persistent, SimpleItem):
             except AttributeError:
                 return 0
 
-        if isinstance(path, (ListType, TupleType)):
+        if isinstance(path, (list, tuple)):
             path = '/'+ '/'.join(path[1:])
         comps = filter(None, path.split('/'))
 
@@ -163,34 +161,36 @@ class PathIndex(Persistent, SimpleItem):
         a tuple (path,level).
 
         level >= 0  starts searching at the given level
-        level <  0  match at *any* level
+        level <  0  not implemented yet
         """
-
-        if isinstance(path, StringType):
+        if isinstance(path, str):
             level = default_level
         else:
             level = int(path[1])
             path  = path[0]
-        
-        if level < 0:
-            # Search at every level, return the union of all results
-            return multiunion(
-                [self.search(path, level) 
-                 for level in xrange(self._depth + 1)])
 
         comps = filter(None, path.split('/'))
-        
-        if level + len(comps) - 1 > self._depth:
-            # Our search is for a path longer than anything in the index
-            return IISet()
 
         if len(comps) == 0:
             return IISet(self._unindex.keys())
 
         results = None
-        for i, comp in reversed(list(enumerate(comps))):
-            if not self._index.get(comp, {}).has_key(level+i): return IISet()
-            results = intersection(results, self._index[comp][level+i])
+        if level >= 0:
+            for i, comp in enumerate(comps):
+                if not self._index.has_key(comp): return IISet()
+                if not self._index[comp].has_key(level+i): return IISet()
+                results = intersection(results, self._index[comp][level+i])
+
+        else:
+            for level in range(self._depth + 1):
+                ids = None
+                for i, comp in enumerate(comps):
+                    try:
+                        ids = intersection(ids, self._index[comp][level+i])
+                    except KeyError:
+                        break
+                else:    
+                    results = union(results, ids)
         return results
 
     def numObjects(self):
@@ -204,17 +204,15 @@ class PathIndex(Persistent, SimpleItem):
     def __len__(self):
         return self._length()
 
-    def _apply_index(self, request, cid=''):
+    def _apply_index(self, request):
         """ hook for (Z)Catalog
             'request' --  mapping type (usually {"path": "..." }
              additionaly a parameter "path_level" might be passed
              to specify the level (see search())
-
-            'cid' -- ???
         """
-
-        record = parseIndexRequest(request,self.id,self.query_options)
-        if record.keys==None: return None
+        record = parseIndexRequest(request, self.id, self.query_options)
+        if record.keys is None:
+            return None
 
         level    = record.get("level",0)
         operator = record.get('operator',self.useOperator).lower()
