@@ -100,7 +100,7 @@ Splitter_dealloc(Splitter *self)
 {
     Py_XDECREF(self->text);
     Py_XDECREF(self->synstop);
-    PyMem_DEL(self);
+    PyObject_DEL(self);
 }
 
 static int
@@ -574,7 +574,7 @@ static char Splitter_module_documentation[] =
     "\n"
     "for use in an inverted index\n"
     "\n"
-    "$Id: ISO_8859_1_Splitter.c 40218 2005-11-18 14:39:19Z andreasjung $\n"
+    "$Id: ISO_8859_1_Splitter.c 79529 2007-09-08 12:21:13Z nikhil_n $\n"
     ;
 
 
