@@ -12,7 +12,7 @@
 ##############################################################################
 """Deprecated text index. Please use ZCTextIndex instead.
 
-$Id: TextIndex.py 41190 2006-01-08 10:22:28Z andreasjung $
+$Id: TextIndex.py 72262 2007-01-30 18:58:05Z yuppie $
 """
 
 import operator, warnings
@@ -82,6 +82,7 @@ class TextIndex(Persistent, Implicit, SimpleItem):
     implements(ITextIndex, IPluggableIndex)
 
     meta_type='TextIndex'
+    query_options = ('query', 'operator')
 
     manage_options= (
         {'label': 'Settings',
@@ -89,8 +90,6 @@ class TextIndex(Persistent, Implicit, SimpleItem):
          'help': ('TextIndex','TextIndex_Settings.stx')},
     )
 
-    query_options = ["query","operator"]
-
     def __init__(self, id, ignore_ex=None, call_methods=None, lexicon=None,
                  caller=None, extra=None):
         """Create an index
@@ -440,7 +439,7 @@ class TextIndex(Persistent, Implicit, SimpleItem):
 
             return r
 
-    def _apply_index(self, request, cid=''):
+    def _apply_index(self, request):
         """ Apply the index to query parameters given in the argument,
         request
 
@@ -454,9 +453,9 @@ class TextIndex(Persistent, Implicit, SimpleItem):
         records.  The second object is a tuple containing the names of
         all data fields used.
         """
-
-        record = parseIndexRequest(request,self.id,self.query_options)
-        if record.keys==None: return None
+        record = parseIndexRequest(request, self.id, self.query_options)
+        if record.keys is None:
+            return None
 
         # Changed for 2.4
         # We use the default operator that can me managed via the ZMI
