@@ -12,14 +12,14 @@
 ##############################################################################
 """ ZCatalog product
 
-$Id: ZCatalog.py 71168 2006-11-17 20:01:22Z lra $
+$Id: ZCatalog.py 81975 2007-11-24 13:25:49Z hannosch $
 """
 
 from warnings import warn
 import urllib, time, sys, string, logging
 
 from Globals import DTMLFile, MessageDialog
-import Globals
+from Globals import InitializeClass
 from OFS.Folder import Folder
 from OFS.ObjectManager import ObjectManager
 from DateTime import DateTime
@@ -27,6 +27,7 @@ from Acquisition import Implicit
 from Persistence import Persistent
 from DocumentTemplate.DT_Util import InstanceDict, TemplateDict
 from DocumentTemplate.DT_Util import Eval
+from AccessControl import ClassSecurityInfo
 from AccessControl.Permission import name_trans
 from AccessControl.DTML import RestrictedDTML
 from AccessControl.Permissions import \
@@ -36,7 +37,6 @@ import transaction
 from Products.PluginIndexes.common.PluggableIndex \
      import PluggableIndexInterface
 from Products.PluginIndexes.interfaces import IPluggableIndex
-from Products.PluginIndexes.TextIndex import Splitter
 from zope.interface import implements
 
 from Catalog import Catalog, CatalogError
@@ -46,10 +46,7 @@ from ProgressHandler import ZLogHandler
 from ZCatalogIndexes import ZCatalogIndexes
 
 
-logger = logging.getLogger('Zope.ZCatalog')
-
-# BBB:  some products apparently import 'LOG' from here!  Gone in 2.10.
-LOG = logger
+LOG = logging.getLogger('Zope.ZCatalog')
 
 manage_addZCatalogForm=DTMLFile('dtml/addZCatalog',globals())
 
@@ -88,6 +85,11 @@ class ZCatalog(Folder, Persistent, Implicit):
     __implements__ = z2IZCatalog
     implements(z3IZCatalog)
 
+    security = ClassSecurityInfo()
+    security.setPermissionDefault(manage_zcatalog_entries, ('Manager',))
+    security.setPermissionDefault(manage_zcatalog_indexes, ('Manager',))
+    security.setPermissionDefault(search_zcatalog, ('Anonymous', 'Manager'))
+
     meta_type = "ZCatalog"
     icon='misc_/ZCatalog/ZCatalog.gif'
 
@@ -124,46 +126,30 @@ class ZCatalog(Folder, Persistent, Implicit):
          'help': ('OFSP','Ownership.stx'),}
         )
 
-    __ac_permissions__=(
-
-        (manage_zcatalog_entries,
-         ['manage_catalogObject', 'manage_uncatalogObject',
-          'catalog_object', 'uncatalog_object', 'refreshCatalog',
-
-          'manage_catalogView', 'manage_catalogFind',
-          'manage_catalogSchema', 'manage_catalogIndexes',
-          'manage_catalogAdvanced', 'manage_objectInformation',
-
-          'manage_catalogReindex', 'manage_catalogFoundItems',
-          'manage_catalogClear', 'manage_addColumn', 'manage_delColumn',
-          'manage_addIndex', 'manage_delIndex', 'manage_clearIndex',
-          'manage_reindexIndex', 'manage_main', 'availableSplitters',
-          'manage_setProgress',
-
-          # these two are deprecated:
-          'manage_delColumns', 'manage_deleteIndex'
-          ],
-         ['Manager']),
-
-        (search_zcatalog,
-         ['searchResults', '__call__', 'uniqueValuesFor',
-          'getpath', 'schema', 'indexes', 'index_objects',
-          'all_meta_types', 'valid_roles', 'resolve_url',
-          'getobject', 'search'],
-         ['Anonymous', 'Manager']),
-
-        (manage_zcatalog_indexes,
-         ['getIndexObjects'],
-         ['Manager']),
-        )
+    security.declareProtected(manage_zcatalog_entries, 'manage_main')
 
+    security.declareProtected(search_zcatalog, 'all_meta_types')
 
     manage_catalogAddRowForm = DTMLFile('dtml/catalogAddRowForm', globals())
+
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogView')
     manage_catalogView = DTMLFile('dtml/catalogView',globals())
+
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogFind')
     manage_catalogFind = DTMLFile('dtml/catalogFind',globals())
+
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogSchema')
     manage_catalogSchema = DTMLFile('dtml/catalogSchema', globals())
+
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogIndexes')
     manage_catalogIndexes = DTMLFile('dtml/catalogIndexes', globals())
+
+    security.declareProtected(manage_zcatalog_entries,
+                              'manage_catalogAdvanced')
     manage_catalogAdvanced = DTMLFile('dtml/catalogAdvanced', globals())
+
+    security.declareProtected(manage_zcatalog_entries,
+                              'manage_objectInformation')
     manage_objectInformation = DTMLFile('dtml/catalogObjectInformation',
                                         globals())
 
@@ -191,6 +177,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         self._v_total = 0
 
         self._catalog = Catalog()
+        self._migrated_280 = True
 
     def __len__(self):
         # Perform a migration of _catalog.__len__ to _catalog._length
@@ -226,6 +213,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             URL1 +
             '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Changed')
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogObject')
     def manage_catalogObject(self, REQUEST, RESPONSE, URL1, urls=None):
         """ index Zope object(s) that 'urls' point to """
         if urls:
@@ -244,6 +232,8 @@ class ZCatalog(Folder, Persistent, Implicit):
             '/manage_catalogView?manage_tabs_message=Object%20Cataloged')
 
 
+    security.declareProtected(manage_zcatalog_entries,
+                              'manage_uncatalogObject')
     def manage_uncatalogObject(self, REQUEST, RESPONSE, URL1, urls=None):
         """ removes Zope object(s) 'urls' from catalog """
 
@@ -259,6 +249,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             '/manage_catalogView?manage_tabs_message=Object%20Uncataloged')
 
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogReindex')
     def manage_catalogReindex(self, REQUEST, RESPONSE, URL1):
         """ clear the catalog, then re-index everything """
 
@@ -280,6 +271,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                          'Total CPU time: %s' % (`elapse`, `c_elapse`)))
 
 
+    security.declareProtected(manage_zcatalog_entries, 'refreshCatalog')
     def refreshCatalog(self, clear=0, pghandler=None):
         """ re-index everything we can find """
 
@@ -306,11 +298,12 @@ class ZCatalog(Folder, Persistent, Implicit):
                 except ConflictError:
                     raise
                 except:
-                    logger.error('Recataloging object at %s failed' % p,
-                                 exc_info=sys.exc_info())
+                    LOG.error('Recataloging object at %s failed' % p,
+                              exc_info=sys.exc_info())
 
         if pghandler: pghandler.finish()
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_catalogClear')
     def manage_catalogClear(self, REQUEST=None, RESPONSE=None, URL1=None):
         """ clears the whole enchilada """
         self._catalog.clear()
@@ -321,6 +314,8 @@ class ZCatalog(Folder, Persistent, Implicit):
               '/manage_catalogAdvanced?manage_tabs_message=Catalog%20Cleared')
 
 
+    security.declareProtected(manage_zcatalog_entries,
+                              'manage_catalogFoundItems')
     def manage_catalogFoundItems(self, REQUEST, RESPONSE, URL2, URL1,
                                  obj_metatypes=None,
                                  obj_ids=None, obj_searchterm=None,
@@ -366,6 +361,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             )
 
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_addColumn')
     def manage_addColumn(self, name, REQUEST=None, RESPONSE=None, URL1=None):
         """ add a column """
         self.addColumn(name)
@@ -376,24 +372,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogSchema?manage_tabs_message=Column%20Added')
 
 
-    def manage_delColumns(self, names, REQUEST=None, RESPONSE=None, URL1=None):
-        """ Deprecated method. Use manage_delColumn instead. """
-        # log a deprecation warning
-        import warnings
-        warnings.warn(
-            "The manage_delColumns method of ZCatalog is deprecated"
-            "since Zope 2.4.2.\n"
-            "This method is only kept for backwards compatibility "
-            "for a while\n"
-            "and will go away in a future release.\n"
-            "\n"
-            "Please use instead the manage_delColumn method.\n"
-            ,DeprecationWarning)
-
-        self.manage_delColumn(names, REQUEST=REQUEST, RESPONSE=RESPONSE,
-                              URL1=URL1)
-
-
+    security.declareProtected(manage_zcatalog_entries, 'manage_delColumn')
     def manage_delColumn(self, names, REQUEST=None, RESPONSE=None, URL1=None):
         """ delete a column or some columns """
         if isinstance(names, str):
@@ -408,6 +387,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogSchema?manage_tabs_message=Column%20Deleted')
 
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_addIndex')
     def manage_addIndex(self, name, type, extra=None,
                         REQUEST=None, RESPONSE=None, URL1=None):
         """add an index """
@@ -419,25 +399,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogIndexes?manage_tabs_message=Index%20Added')
 
 
-    def manage_deleteIndex(self, ids=None, REQUEST=None, RESPONSE=None,
-        URL1=None):
-        """ Deprecated method. Use manage_delIndex instead. """
-        # log a deprecation warning
-        import warnings
-        warnings.warn(
-            "The manage_deleteIndex method of ZCatalog is deprecated"
-            "since Zope 2.4.2.\n"
-            "This method is only kept for backwards compatibility for a "
-            "while\n"
-            "and will go away in a future release.\n"
-            "\n"
-            "Please use instead the manage_delIndex method.\n"
-            ,DeprecationWarning)
-
-        self.manage_delIndex(ids=ids, REQUEST=REQUEST, RESPONSE=RESPONSE,
-                             URL1=URL1)
-
-
+    security.declareProtected(manage_zcatalog_entries, 'manage_delIndex')
     def manage_delIndex(self, ids=None, REQUEST=None, RESPONSE=None,
         URL1=None):
         """ delete an index or some indexes """
@@ -458,6 +420,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '/manage_catalogIndexes?manage_tabs_message=Index%20Deleted')
 
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_clearIndex')
     def manage_clearIndex(self, ids=None, REQUEST=None, RESPONSE=None,
         URL1=None):
         """ clear an index or some indexes """
@@ -496,8 +459,8 @@ class ZCatalog(Folder, Persistent, Implicit):
             if obj is None:
                 obj = self.resolve_url(p, REQUEST)
             if obj is None:
-                logger.error('reindexIndex could not resolve '
-                             'an object from the uid %r.' % p)
+                LOG.error('reindexIndex could not resolve '
+                          'an object from the uid %r.' % p)
             else:
                 # don't update metadata when only reindexing a single
                 # index via the UI
@@ -525,6 +488,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         if pghandler:
             pghandler.finish()
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_reindexIndex')
     def manage_reindexIndex(self, ids=None, REQUEST=None, RESPONSE=None,
                             URL1=None):
         """Reindex indexe(s) from a ZCatalog"""
@@ -544,11 +508,16 @@ class ZCatalog(Folder, Persistent, Implicit):
                 '?manage_tabs_message=Reindexing%20Performed')
 
 
+    # BBB: will be removed in Zope 2.12 (like TextIndex itself)
+    security.declareProtected(manage_zcatalog_entries, 'availableSplitters')
     def availableSplitters(self):
         """ splitter we can add """
+        # This import will trigger a deprecation warning about TextIndex
+        from Products.PluginIndexes.TextIndex import Splitter
         return Splitter.availableSplitters
 
 
+    security.declareProtected(manage_zcatalog_entries, 'catalog_object')
     def catalog_object(self, obj, uid=None, idxs=None, update_metadata=1, pghandler=None):
         """ wrapper around catalog """
 
@@ -594,14 +563,17 @@ class ZCatalog(Folder, Persistent, Implicit):
                 if pghandler:
                     pghandler.info('commiting subtransaction')
 
+    security.declareProtected(manage_zcatalog_entries, 'uncatalog_object')
     def uncatalog_object(self, uid):
         """Wrapper around catalog """
         self._catalog.uncatalogObject(uid)
 
+    security.declareProtected(search_zcatalog, 'uniqueValuesFor')
     def uniqueValuesFor(self, name):
         """Return the unique values for a given FieldIndex """
         return self._catalog.uniqueValuesFor(name)
 
+    security.declareProtected(search_zcatalog, 'getpath')
     def getpath(self, rid):
         """Return the path to a cataloged object given a 'data_record_id_'
         """
@@ -612,6 +584,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         """
         return self._catalog.uids.get(path, default)
 
+    security.declareProtected(search_zcatalog, 'getobject')
     def getobject(self, rid, REQUEST=None):
         """Return a cataloged object given a 'data_record_id_'
         """
@@ -635,17 +608,21 @@ class ZCatalog(Folder, Persistent, Implicit):
         """return the current index contents for the specific rid"""
         return self._catalog.getIndexDataForRID(rid)
 
+    security.declareProtected(search_zcatalog, 'schema')
     def schema(self):
         return self._catalog.schema.keys()
 
+    security.declareProtected(search_zcatalog, 'indexes')
     def indexes(self):
         return self._catalog.indexes.keys()
 
+    security.declareProtected(search_zcatalog, 'index_objects')
     def index_objects(self):
         # This method returns unwrapped indexes!
         # You should probably use getIndexObjects instead
         return self._catalog.indexes.values()
 
+    security.declareProtected(manage_zcatalog_indexes, 'getIndexObjects')
     def getIndexObjects(self):
         # Return a list of wrapped(!) indexes
         getIndex = self._catalog.getIndex
@@ -673,6 +650,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                   'width': 8})
         return r
 
+    security.declareProtected(search_zcatalog, 'searchResults')
     def searchResults(self, REQUEST=None, used=None, **kw):
         """Search the catalog
 
@@ -684,8 +662,10 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         return self._catalog.searchResults(REQUEST, used, **kw)
 
+    security.declareProtected(search_zcatalog, '__call__')
     __call__=searchResults
 
+    security.declareProtected(search_zcatalog, 'search')
     def search(
         self, query_request, sort_index=None, reverse=0, limit=None, merge=1):
         """Programmatic search interface, use for searching the catalog from
@@ -716,6 +696,7 @@ class ZCatalog(Folder, Persistent, Implicit):
     #        except AttributeError:  pass
     #    return self.meta_types+Products.meta_types+pmt
 
+    security.declareProtected(search_zcatalog, 'valid_roles')
     def valid_roles(self):
         "Return list of valid roles"
         obj=self
@@ -834,6 +815,7 @@ class ZCatalog(Folder, Persistent, Implicit):
 
         return result
 
+    security.declareProtected(search_zcatalog, 'resolve_url')
     def resolve_url(self, path, REQUEST):
         """
         Attempt to resolve a url into an object in the Zope
@@ -898,6 +880,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                   '%s unchanged.' % (len(fixed), len(removed), unchanged),
           action='./manage_main')
 
+    security.declareProtected(manage_zcatalog_entries, 'manage_setProgress')
     def manage_setProgress(self, pgthreshold=0, RESPONSE=None, URL1=None):
         """Set parameter to perform logging of reindexing operations very 
            'pgthreshold' objects
@@ -921,7 +904,7 @@ class ZCatalog(Folder, Persistent, Implicit):
            classes. 
         """
 
-        logger.info('Start migration of indexes for %s' % self.absolute_url(1))
+        LOG.info('Start migration of indexes for %s' % self.absolute_url(1))
 
         for idx in self.Indexes.objectValues():
             bases = [str(name) for name in idx.__class__.__bases__]
@@ -938,7 +921,7 @@ class ZCatalog(Folder, Persistent, Implicit):
             if found:
                 idx_type = idx.meta_type
                 idx_id = idx.getId()
-                logger.info('processing index %s' % idx_id)
+                LOG.info('processing index %s' % idx_id)
 
                 indexed_attrs = getattr(idx, 'indexed_attrs', None)
 
@@ -960,8 +943,7 @@ class ZCatalog(Folder, Persistent, Implicit):
                 self.manage_reindexIndex(idx_id, REQUEST)
 
         self._migrated_280 = True
-        logger.info('Finished migration of indexes for %s'
-                    % self.absolute_url(1))
+        LOG.info('Finished migration of indexes for %s' % self.absolute_url(1))
 
         if RESPONSE:
             RESPONSE.redirect( URL1 +
@@ -1023,7 +1005,7 @@ class ZCatalog(Folder, Persistent, Implicit):
         return self._catalog.delColumn(name)
 
 
-Globals.default__class_init__(ZCatalog)
+InitializeClass(ZCatalog)
 
 
 def p_name(name):
