@@ -1,15 +1,35 @@
+# -*- coding: iso-8859-15 -*-
+
 """ Unit tests for ZReST objects
 
-$Id: test_ZReST.py 69290 2006-07-29 12:59:31Z andreasjung $
+$Id: test_ZReST.py 78333 2007-07-25 20:53:58Z philikon $
 """
 import unittest
+import cgi
 import tempfile
 
+txt = """Hello World
+============
+
+text text
+
+Von Vgeln und fen
+===================
+
+- some
+- more
+- text
+
+"""
+
 
 csv_text = """bin:x:1:1:bin:/bin:/bin/bash
 daemon:x:2:2:Daemon:/sbin:/bin/bash
 """
 
+docutils_include_warning = '(WARNING/2) "include" directive disabled.'
+docutils_raw_warning = '(WARNING/2) "raw" directive disabled.'
+
 class TestZReST(unittest.TestCase):
 
     def _getTargetClass(self):
@@ -40,38 +60,73 @@ class TestZReST(unittest.TestCase):
 
         self.failIf('IGNORE ME' in resty.index_html())
 
+    def testConversion(self):
+        resty = self._makeOne()
+        resty.source = txt
+        resty.input_encoding = 'iso-8859-15'
+        resty.output_encoding = 'iso-8859-15'
+        resty.render()
+        html = resty.index_html()
+
+        s = '<h1><a id="hello-world" name="hello-world">Hello World</a></h1>'
+        self.assertEqual(s in html, True)
+
+        s = '<h1><a id="von-v-geln-und-fen" name="von-v-geln-und-fen">'\
+            'Von Vgeln und fen</a></h1>'
+        self.assertEqual(s in html, True)
+
+        # ZReST should render a complete HTML document
+        self.assertEqual('<html' in html, True)
+        self.assertEqual('<body>' in html, True)
+
     def test_include_directive_raises(self):
         resty = self._makeOne()
         resty.source = 'hello world\n .. include:: /etc/passwd'
-        self.assertRaises(NotImplementedError, resty.render)
+        result = resty.render()
+        warnings = ''.join(resty._v_warnings.messages)
+
+        # The include: directive hasn't been rendered, it remains
+        # verbatimly in the rendered output.  Instead a warning
+        # message is presented:
+        self.assert_(docutils_include_warning in warnings)
 
     def test_raw_directive_disabled(self):
-
-        EXPECTED = '<h1>HELLO WORLD</h1>'
-
         resty = self._makeOne()
+        EXPECTED = '<h1>HELLO WORLD</h1>'
         resty.source = '.. raw:: html\n\n  %s\n' % EXPECTED
-        result = resty.render() # don't raise, but don't work either
-        self.failIf(EXPECTED in result)
+        result = resty.render()
+        warnings = ''.join(resty._v_warnings.messages)
 
-        self.failUnless("&quot;raw&quot; directive disabled" in result)
-        from cgi import escape
-        self.failUnless(escape(EXPECTED) in result)
+        # The raw: directive hasn't been rendered, it remains
+        # verbatimly in the rendered output.  Instead a warning
+        # message is presented:
+        self.assert_(EXPECTED not in result)
+        self.assert_(cgi.escape(EXPECTED) in result)
+        self.assert_(docutils_raw_warning in warnings)
 
     def test_raw_directive_file_directive_raises(self):
-
         resty = self._makeOne()
         resty.source = '.. raw:: html\n  :file: inclusion.txt'
-        self.assertRaises(NotImplementedError, resty.render)
+        result = resty.render()
+        warnings = ''.join(resty._v_warnings.messages)
+
+        # The raw: directive hasn't been rendered, it remains
+        # verbatimly in the rendered output.  Instead a warning
+        # message is presented:
+        self.assert_(docutils_raw_warning in warnings)
 
     def test_raw_directive_url_directive_raises(self):
-
         resty = self._makeOne()
         resty.source = '.. raw:: html\n  :url: http://www.zope.org/'
-        self.assertRaises(NotImplementedError, resty.render)
+        result = resty.render()
+        warnings = ''.join(resty._v_warnings.messages)
+
+        # The raw: directive hasn't been rendered, it remains
+        # verbatimly in the rendered output.  Instead a warning
+        # message is presented:
+        self.assert_(docutils_raw_warning in warnings)
 
     def test_csv_table_file_option_raise(self):
-
         resty = self._makeOne()
         csv_file = self._csvfile()
         resty.source = '.. csv-table:: \n  :file: %s' % csv_file
