@@ -22,10 +22,11 @@ Typically used as in
   Zope2.installProduct('SomeProduct')
   app = Zope2.app()
 
-$Id: ZopeLite.py 40819 2005-12-16 18:40:14Z shh $
+$Id: ZopeLite.py 89897 2008-08-15 21:52:31Z shh $
 """
 
 import os, sys, time
+import layer
 
 # Allow code to tell it is run by the test framework
 os.environ['ZOPETESTCASE'] = '1'
@@ -61,6 +62,9 @@ def _configure_logging():
     import logging
     root = logging.getLogger()
     if not root.handlers:
+        class NullHandler(logging.Handler):
+            def emit(self, record): pass
+        root.addHandler(NullHandler())
         logging.basicConfig()
 
 def _configure_debug_mode():
@@ -102,7 +106,12 @@ _write('.')
 
 _patched = False
 
+@layer.onsetup
 def _apply_patches():
+    # Do not patch a running Zope
+    if Zope2._began_startup:
+        return
+
     # Avoid expensive product import
     def null_import_products(): pass
     OFS.Application.import_products = null_import_products
@@ -123,24 +132,39 @@ def _apply_patches():
     global _patched
     _patched = True
 
-# Do not patch a running Zope
-if not Zope2._began_startup:
-    _apply_patches()
+_apply_patches()
+
+_theApp = None
+
+@layer.onsetup
+def _startup():
+    global _theApp
+    _theApp = Zope2.app()
+
+# Start ZopeLite
+_startup()
 
 # Allow test authors to install Zope products into the test environment. Note
 # that installProduct() must be called at module level -- never from tests.
-from OFS.Application import get_folder_permissions, get_products, install_product
+from OFS.Application import get_folder_permissions, get_products
+from OFS.Application import install_product, install_package
 from OFS.Folder import Folder
 import Products
 
-_theApp = Zope2.app()
 _installedProducts = {}
+_installedPackages = {}
 
 def hasProduct(name):
     '''Checks if a product can be found along Products.__path__'''
     return name in [n[1] for n in get_products()]
 
+@layer.onsetup
 def installProduct(name, quiet=0):
+    '''Installs a Zope product at layer setup time.'''
+    quiet = 1 # Ignore argument
+    _installProduct(name, quiet)
+
+def _installProduct(name, quiet=0):
     '''Installs a Zope product.'''
     start = time.time()
     meta_types = []
@@ -154,34 +178,41 @@ def installProduct(name, quiet=0):
                                 get_folder_permissions(), raise_exc=1)
                 _installedProducts[product_name] = 1
                 Products.meta_types = Products.meta_types + tuple(meta_types)
-                Globals.default__class_init__(Folder)
+                Globals.InitializeClass(Folder)
                 if not quiet: _print('done (%.3fs)\n' % (time.time() - start))
                 break
         else:
             if name != 'SomeProduct':   # Ignore the skeleton tests :-P
                 if not quiet: _print('Installing %s ... NOT FOUND\n' % name)
 
-def _load_control_panel():
-    # Loading the Control_Panel of an existing ZODB may take
-    # a while; print another dot if it does.
+def hasPackage(name):
+    '''Checks if a package has been registered with five:registerPackage.'''
+    return name in [m.__name__ for m in getattr(Products, '_registered_packages', [])]
+
+def installPackage(name, quiet=0):
+    '''Installs a registered Python package.'''
+    quiet = 1 # Ignore argument
+    _installPackage(name, quiet)
+
+def _installPackage(name, quiet=0):
+    '''Installs a registered Python package.'''
     start = time.time()
-    max = (start - _start) / 4
-    _exec('_theApp.Control_Panel')
-    _theApp.Control_Panel
-    if (time.time() - start) > max:
-        _write('.')
+    if _patched and not _installedPackages.has_key(name):
+        for module, init_func in getattr(Products, '_packages_to_initialize', []):
+            if module.__name__ == name:
+                if not quiet: _print('Installing %s ... ' % module.__name__)
+                # We want to fail immediately if a package throws an exception
+                # during install, so we set the raise_exc flag.
+                install_package(_theApp, module, init_func, raise_exc=1)
+                _installedPackages[module.__name__] = 1
+                Products._packages_to_initialize.remove((module, init_func))
+                if not quiet: _print('done (%.3fs)\n' % (time.time() - start))
+                break
+        else:
+            if not quiet: _print('Installing %s ... NOT FOUND\n' % name)
 
-def _install_products():
-    installProduct('PluginIndexes', 1)  # Must install first
-    installProduct('OFSP', 1)
-    #installProduct('ExternalMethod', 1)
-    #installProduct('ZSQLMethods', 1)
-    #installProduct('ZGadflyDA', 1)
-    #installProduct('MIMETools', 1)
-    #installProduct('MailHost', 1)
-
-_load_control_panel()
-_install_products()
+installProduct('PluginIndexes', 1)  # Must install first
+installProduct('OFSP', 1)
 
 # So people can use ZopeLite.app()
 app = Zope2.app
@@ -190,6 +221,7 @@ DB = Zope2.DB
 configure = Zope2.configure
 def startup(): pass
 Zope = Zope2
+active = _patched
 
 # ZODB sandbox factory
 from ZODB.DemoStorage import DemoStorage
