@@ -12,14 +12,17 @@
 ##############################################################################
 """Names exported by the ZopeTestCase package
 
-$Id: __init__.py 66218 2006-03-27 01:29:02Z shh $
+$Id: __init__.py 80864 2007-10-13 16:15:38Z shh $
 """
 
 import ZopeLite as Zope2
 import utils
+import layer
 
 from ZopeLite import hasProduct
 from ZopeLite import installProduct
+from ZopeLite import hasPackage
+from ZopeLite import installPackage
 from ZopeLite import _print
 
 from ZopeTestCase import folder_name
@@ -33,7 +36,6 @@ from ZopeTestCase import FunctionalTestCase
 from PortalTestCase import portal_name
 from PortalTestCase import PortalTestCase
 
-from profiler import Profiled
 from sandbox import Sandboxed
 from functional import Functional
 
