@@ -14,7 +14,7 @@
 
 After Marius Gedminas' functional.py module for Zope3.
 
-$Id: functional.py 38609 2005-09-25 08:45:18Z tseaver $
+$Id: functional.py 73185 2007-03-15 10:04:04Z shh $
 """
 
 import sys, re, base64
@@ -22,6 +22,27 @@ import transaction
 import sandbox
 import interfaces
 
+from zope.interface import implements
+
+
+def savestate(func):
+    '''Decorator saving thread local state before executing func
+       and restoring it afterwards.
+    '''
+    from AccessControl.SecurityManagement import getSecurityManager
+    from AccessControl.SecurityManagement import setSecurityManager
+    from zope.app.component.hooks import getSite
+    from zope.app.component.hooks import setSite
+
+    def wrapped_func(*args, **kw):
+        sm, site = getSecurityManager(), getSite()
+        try:
+            return func(*args, **kw)
+        finally:
+            setSecurityManager(sm)
+            setSite(site)
+    return wrapped_func
+
 
 class Functional(sandbox.Sandboxed):
     '''Derive from this class and an xTestCase to get functional
@@ -31,8 +52,9 @@ class Functional(sandbox.Sandboxed):
                ...
     '''
 
-    __implements__ = (interfaces.IFunctional,)
+    implements(interfaces.IFunctional)
 
+    @savestate
     def publish(self, path, basic=None, env=None, extra=None,
                 request_method='GET', stdin=None, handle_errors=True):
         '''Publishes the object at 'path' returning a response object.'''
@@ -41,12 +63,6 @@ class Functional(sandbox.Sandboxed):
         from ZPublisher.Response import Response
         from ZPublisher.Test import publish_module
 
-        from AccessControl.SecurityManagement import getSecurityManager
-        from AccessControl.SecurityManagement import setSecurityManager
-
-        # Save current security manager
-        sm = getSecurityManager()
-
         # Commit the sandbox for good measure
         transaction.commit()
 
@@ -86,9 +102,6 @@ class Functional(sandbox.Sandboxed):
                        debug=not handle_errors,
                       )
 
-        # Restore security manager
-        setSecurityManager(sm)
-
         return ResponseWrapper(response, outstream, path)
 
 
