@@ -13,7 +13,7 @@
 """Parts of ZServer support are in this module so they can
 be imported more selectively.
 
-$Id: threadutils.py 40219 2005-11-18 15:05:47Z andreasjung $
+$Id: threadutils.py 73190 2007-03-15 13:29:18Z shh $
 """
 
 from threading import Thread
@@ -22,6 +22,15 @@ from StringIO import StringIO
 dummyLOG = StringIO()
 
 
+def setNumberOfThreads(number_of_threads):
+    '''Sets number of ZServer threads.'''
+    try:
+        from ZServer.PubCore import setNumberOfThreads
+        setNumberOfThreads(number_of_threads)
+    except ImportError:
+        pass
+
+
 def zserverRunner(host, port, log=None):
     '''Runs an HTTP ZServer on host:port.'''
     from ZServer import logger, asyncore
