@@ -12,7 +12,7 @@
 ##############################################################################
 """Example Zope doctest
 
-$Id: testZopeDocTest.py 40219 2005-11-18 15:05:47Z andreasjung $
+$Id: testZopeDocTest.py 76989 2007-06-23 17:08:16Z shh $
 """
 
 import os, sys
@@ -29,8 +29,12 @@ def setUp(self):
 
     >>> 'object' in folder.objectIds()
     True
+
+    >>> foo
+    1
     '''
     self.folder.manage_addFolder('object', '')
+    self.globs['foo'] = 1
 
 
 def test_suite():
