@@ -4,13 +4,14 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
+import asyncore
 import errno
 import select
 import socket
@@ -18,14 +19,13 @@ import sys
 import threading
 import time
 import types
-
-import ThreadedAsync
-import zLOG
+import logging
 
 from ZODB.POSException import ReadOnlyError
+from ZODB.loglevels import BLATHER
 
 from ZEO.zrpc.log import log
-from ZEO.zrpc.trigger import trigger
+import ZEO.zrpc.trigger
 from ZEO.zrpc.connection import ManagedClientConnection
 
 class ConnectionManager(object):
@@ -42,9 +42,6 @@ class ConnectionManager(object):
         # If thread is not None, then there is a helper thread
         # attempting to connect.
         self.thread = None # Protected by self.cond
-        self.trigger = None
-        self.thr_async = 0
-        ThreadedAsync.register_loop_callback(self.set_async)
 
     def __repr__(self):
         return "<%s for %s>" % (self.__class__.__name__, self.addrlist)
@@ -65,8 +62,7 @@ class ConnectionManager(object):
             for addr in addrs:
                 addr_type = self._guess_type(addr)
                 if addr_type is None:
-                    raise ValueError, (
-                        "unknown address in list: %s" % repr(addr))
+                    raise ValueError("unknown address in list: %s" % repr(addr))
                 addrlist.append((addr_type, addr))
             return addrlist
 
@@ -85,7 +81,6 @@ class ConnectionManager(object):
     def close(self):
         """Prevent ConnectionManager from opening new connections"""
         self.closed = 1
-        ThreadedAsync.remove_loop_callback(self.set_async)
         self.cond.acquire()
         try:
             t = self.thread
@@ -99,33 +94,10 @@ class ConnectionManager(object):
             t.join(30)
             if t.isAlive():
                 log("CM.close(): self.thread.join() timed out",
-                    level=zLOG.WARNING)
+                    level=logging.WARNING)
         if conn is not None:
             # This will call close_conn() below which clears self.connection
             conn.close()
-        if self.trigger is not None:
-            self.trigger.close()
-            self.trigger = None
-        ThreadedAsync.remove_loop_callback(self.set_async)
-
-    def set_async(self, map):
-        # This is the callback registered with ThreadedAsync.  The
-        # callback might be called multiple times, so it shouldn't
-        # create a trigger every time and should never do anything
-        # after it's closed.
-
-        # It may be that the only case where it is called multiple
-        # times is in the test suite, where ThreadedAsync's loop can
-        # be started in a child process after a fork.  Regardless,
-        # it's good to be defensive.
-
-        # XXX need each connection started with async==0 to have a
-        # callback
-        log("CM.set_async(%s)" % repr(map), level=zLOG.DEBUG)
-        if not self.closed and self.trigger is None:
-            log("CM.set_async(): first call")
-            self.trigger = trigger()
-            self.thr_async = 1 # XXX needs to be set on the Connection
 
     def attempt_connect(self):
         """Attempt a connection to the server without blocking too long.
@@ -138,8 +110,8 @@ class ConnectionManager(object):
         finishes quickly.
         """
 
-        # XXX Will a single attempt take too long?
-        # XXX Answer: it depends -- normally, you'll connect or get a
+        # Will a single attempt take too long?
+        # Answer:  it depends -- normally, you'll connect or get a
         # connection refused error very quickly.  Packet-eating
         # firewalls and other mishaps may cause the connect to take a
         # long time to time out though.  It's also possible that you
@@ -204,7 +176,7 @@ class ConnectionManager(object):
         try:
             if conn is not self.connection:
                 # Closing a non-current connection
-                log("CM.close_conn() non-current", level=zLOG.BLATHER)
+                log("CM.close_conn() non-current", level=BLATHER)
                 return
             log("CM.close_conn()")
             self.connection = None
@@ -227,7 +199,7 @@ class ConnectionManager(object):
 # to the errno value(s) expected if the connect succeeds *or* if it's
 # already connected (our code can attempt redundant connects).
 if hasattr(errno, "WSAEWOULDBLOCK"):    # Windows
-    # XXX The official Winsock docs claim that WSAEALREADY should be
+    # Caution:  The official Winsock docs claim that WSAEALREADY should be
     # treated as yet another "in progress" indicator, but we've never
     # seen this.
     _CONNECT_IN_PROGRESS = (errno.WSAEWOULDBLOCK,)
@@ -286,7 +258,7 @@ class ConnectThread(threading.Thread):
         delay = self.tmin
         success = 0
         # Don't wait too long the first time.
-        # XXX make timeout configurable?
+        # TODO: make timeout configurable?
         attempt_timeout = 5
         while not self.stopped:
             success = self.try_connecting(attempt_timeout)
@@ -298,7 +270,7 @@ class ConnectThread(threading.Thread):
             time.sleep(delay)
             if self.mgr.is_connected():
                 log("CT: still trying to replace fallback connection",
-                    level=zLOG.INFO)
+                    level=logging.INFO)
             delay = min(delay*2, self.tmax)
         log("CT: exiting thread: %s" % self.getName())
 
@@ -372,7 +344,7 @@ class ConnectThread(threading.Thread):
                 log("CT: select() %d, %d, %d" % tuple(map(len, (r,w,x))))
             except select.error, msg:
                 log("CT: select failed; msg=%s" % str(msg),
-                    level=zLOG.WARNING) # XXX Is this the right level?
+                    level=logging.WARNING)
                 continue
             # Exceptable wrappers are in trouble; close these suckers
             for wrap in x:
@@ -407,7 +379,7 @@ class ConnectThread(threading.Thread):
             assert wrap.state == "closed"
             del wrappers[wrap]
 
-            # XXX should check deadline
+            # TODO: should check deadline
 
 
 class ConnectWrapper:
@@ -438,7 +410,7 @@ class ConnectWrapper:
             self.sock = socket.socket(domain, socket.SOCK_STREAM)
         except socket.error, err:
             log("CW: can't create socket, domain=%s: %s" % (domain, err),
-                level=zLOG.ERROR)
+                level=logging.ERROR)
             self.close()
             return
         self.sock.setblocking(0)
@@ -451,7 +423,7 @@ class ConnectWrapper:
                 err = self.sock.connect_ex(self.addr)
             except socket.error, msg:
                 log("CW: connect_ex(%r) failed: %s" % (self.addr, msg),
-                    level=zLOG.ERROR)
+                    level=logging.ERROR)
                 self.close()
                 return
             log("CW: connect_ex(%s) returned %s" %
@@ -462,7 +434,7 @@ class ConnectWrapper:
             if err not in _CONNECT_OK:
                 log("CW: error connecting to %s: %s" %
                     (self.addr, errno.errorcode.get(err) or str(err)),
-                    level=zLOG.WARNING)
+                    level=logging.WARNING)
                 self.close()
                 return
             self.state = "connected"
@@ -475,8 +447,7 @@ class ConnectWrapper:
         Call the client's testConnection(), giving the client a chance
         to do app-level check of the connection.
         """
-        self.conn = ManagedClientConnection(self.sock, self.addr,
-                                            self.client, self.mgr)
+        self.conn = ManagedClientConnection(self.sock, self.addr, self.mgr)
         self.sock = None # The socket is now owned by the connection
         try:
             self.preferred = self.client.testConnection(self.conn)
@@ -487,7 +458,7 @@ class ConnectWrapper:
             return
         except:
             log("CW: error in testConnection (%s)" % repr(self.addr),
-                level=zLOG.ERROR, error=sys.exc_info())
+                level=logging.ERROR, exc_info=True)
             self.close()
             return
         if self.preferred:
@@ -506,7 +477,7 @@ class ConnectWrapper:
             self.client.notifyConnected(self.conn)
         except:
             log("CW: error in notifyConnected (%s)" % repr(self.addr),
-                level=zLOG.ERROR, error=sys.exc_info())
+                level=logging.ERROR, exc_info=True)
             self.close()
             return
         self.state = "notified"
@@ -519,8 +490,7 @@ class ConnectWrapper:
         self.preferred = 0
         if self.conn is not None:
             # Closing the ZRPC connection will eventually close the
-            # socket, somewhere in asyncore.
-            # XXX Why do we care? --Guido
+            # socket, somewhere in asyncore.  Guido asks: Why do we care?
             self.conn.close()
             self.conn = None
         if self.sock is not None:
