@@ -4,49 +4,29 @@
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
-# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
 # THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
 # WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 # WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
 # FOR A PARTICULAR PURPOSE.
 #
 ##############################################################################
-
+import os, unittest
+import transaction
 import ZODB.FileStorage
-import sys, os, unittest
-import errno
-import filecmp
-import StringIO
-from ZODB.Transaction import Transaction
+import ZODB.tests.util
 from ZODB import POSException
-from ZODB.fsrecover import recover
+from ZODB import DB
 
-from ZODB.tests import StorageTestBase, BasicStorage, \
-     TransactionalUndoStorage, VersionStorage, \
-     TransactionalUndoVersionStorage, PackableStorage, \
-     Synchronization, ConflictResolution, HistoryStorage, \
-     IteratorStorage, Corruption, RevisionStorage, PersistentStorage, \
-     MTStorage, ReadOnlyStorage, RecoveryStorage
-from ZODB.tests.StorageTestBase import MinPO, zodb_unpickle
+from ZODB.tests import StorageTestBase, BasicStorage, TransactionalUndoStorage
+from ZODB.tests import VersionStorage, TransactionalUndoVersionStorage
+from ZODB.tests import PackableStorage, Synchronization, ConflictResolution
+from ZODB.tests import HistoryStorage, IteratorStorage, Corruption
+from ZODB.tests import RevisionStorage, PersistentStorage, MTStorage
+from ZODB.tests import ReadOnlyStorage, RecoveryStorage
+from ZODB.tests.StorageTestBase import MinPO, zodb_pickle
 
-class FileStorageTests(
-    StorageTestBase.StorageTestBase,
-    BasicStorage.BasicStorage,
-    TransactionalUndoStorage.TransactionalUndoStorage,
-    RevisionStorage.RevisionStorage,
-    VersionStorage.VersionStorage,
-    TransactionalUndoVersionStorage.TransactionalUndoVersionStorage,
-    PackableStorage.PackableStorage,
-    Synchronization.SynchronizedStorage,
-    ConflictResolution.ConflictResolvingStorage,
-    ConflictResolution.ConflictResolvingTransUndoStorage,
-    HistoryStorage.HistoryStorage,
-    IteratorStorage.IteratorStorage,
-    IteratorStorage.ExtendedIteratorStorage,
-    PersistentStorage.PersistentStorage,
-    MTStorage.MTStorage,
-    ReadOnlyStorage.ReadOnlyStorage
-    ):
+class BaseFileStorageTests(StorageTestBase.StorageTestBase):
 
     def open(self, **kwargs):
         self._storage = ZODB.FileStorage.FileStorage('FileStorageTests.fs',
@@ -57,7 +37,27 @@ class FileStorageTests(
 
     def tearDown(self):
         self._storage.close()
-        StorageTestBase.removefs("FileStorageTests.fs")
+        self._storage.cleanup()
+
+class FileStorageTests(
+    BaseFileStorageTests,
+    BasicStorage.BasicStorage,
+    TransactionalUndoStorage.TransactionalUndoStorage,
+    RevisionStorage.RevisionStorage,
+    VersionStorage.VersionStorage,
+    TransactionalUndoVersionStorage.TransactionalUndoVersionStorage,
+    PackableStorage.PackableStorage,
+    PackableStorage.PackableUndoStorage,
+    Synchronization.SynchronizedStorage,
+    ConflictResolution.ConflictResolvingStorage,
+    ConflictResolution.ConflictResolvingTransUndoStorage,
+    HistoryStorage.HistoryStorage,
+    IteratorStorage.IteratorStorage,
+    IteratorStorage.ExtendedIteratorStorage,
+    PersistentStorage.PersistentStorage,
+    MTStorage.MTStorage,
+    ReadOnlyStorage.ReadOnlyStorage
+    ):
 
     def checkLongMetadata(self):
         s = "X" * 75000
@@ -139,6 +139,41 @@ class FileStorageTests(
         # Python dict.
         self.check_conversion_to_fsIndex(read_only=True)
 
+    def check_conversion_from_dict_to_btree_data_in_fsIndex(self):
+        # To support efficient range searches on its keys as part of
+        # implementing a record iteration protocol in FileStorage, we
+        # converted the fsIndex class from using a dictionary as its
+        # self._data attribute to using an OOBTree in its stead.
+
+        from ZODB.fsIndex import fsIndex
+        from BTrees.OOBTree import OOBTree
+
+        # Create some data, and remember the index.
+        for i in range(10):
+            self._dostore()
+        data_dict = dict(self._storage._index._data)
+
+        # Replace the OOBTree with a dictionary and commit it.
+        self._storage._index._data = data_dict
+        transaction.commit()
+
+        # Save the index.
+        self._storage.close()
+
+        # Verify it's converted to fsIndex in memory upon open.
+        self.open()
+        self.assert_(isinstance(self._storage._index, fsIndex))
+        self.assert_(isinstance(self._storage._index._data, OOBTree))
+
+        # Verify it has the right content.
+        new_data_dict = dict(self._storage._index._data)
+        self.assertEqual(len(data_dict), len(new_data_dict))
+
+        for k in data_dict:
+            old_tree = data_dict[k]
+            new_tree = new_data_dict[k]
+            self.assertEqual(list(old_tree.items()), list(new_tree.items()))
+
     def check_save_after_load_with_no_index(self):
         for i in range(10):
             self._dostore()
@@ -147,37 +182,6 @@ class FileStorageTests(
         self.open()
         self.assertEqual(self._storage._saved, 1)
 
-
-    # This would make the unit tests too slow
-    # check_save_after_load_that_worked_hard(self)
-
-    def check_periodic_save_index(self):
-
-        # Check the basic algorithm
-        oldsaved = self._storage._saved
-        self._storage._records_before_save = 10
-        for i in range(4):
-            self._dostore()
-        self.assertEqual(self._storage._saved, oldsaved)
-        self._dostore()
-        self.assertEqual(self._storage._saved, oldsaved+1)
-
-        # Now make sure the parameter changes as we get bigger
-        for i in range(20):
-            self._dostore()
-
-        self.failUnless(self._storage._records_before_save > 20)
-
-    # There are a bunch of tests that the current pack() implementation
-    # does not past.  We need to fix pack(), but don't want tests to
-    # fail until then.
-
-    def checkPackVersionsInPast(self):
-        pass
-
-    def checkPackAfterUndoDeletion(self):
-        pass
-
     def check_index_oid_ignored(self):
         # Prior to ZODB 3.2.6, the 'oid' value stored in the .index file
         # was believed.  But there were cases where adding larger oids
@@ -190,13 +194,12 @@ class FileStorageTests(
         # Now the cached 'oid' value is ignored:  verify that this is so.
         import cPickle as pickle
         from ZODB.utils import z64
-        from ZODB.DB import DB
 
         # Create some data.
         db = DB(self._storage)
         conn = db.open()
         conn.root()['xyz'] = 1
-        get_transaction().commit()
+        transaction.commit()
         true_max_oid = self._storage._oid
 
         # Save away the index, and poke in a bad 'oid' value by hand.
@@ -218,72 +221,11 @@ class FileStorageTests(
         self.open()
         self.assertEqual(self._storage._oid, true_max_oid)
 
-    def checkTimeTravelOnOpen(self):
-        from ZODB.DB import DB
-
-        # This tests what happens if we open a FileStorage and its
-        # last tid appears to be in the future.  This can happen if,
-        # e.g., the system clock has been set backwards.  Part of
-        # the test is to check that we continue to create increasing
-        # tids anyway.  The other part is to check that we log
-        # appropriate msgs.  In ZODB 3.3, that latter part *is* checked,
-        # by adding additional handlers to the logging package.  In
-        # this (3.2) version of the test, it isn't checked
-        # automatically.  You have to enable logging and eyeball the
-        # log file in 3.2.
-
-        # First check the normal case:  transactions are recorded with
-        # increasing tids, and time doesn't run backwards.
-
-        db = DB(self._storage)
-        conn = db.open()
-        conn.root()['xyz'] = 1
-        get_transaction().commit()
-        checkIncreasingTids(self._storage)
-        db.close()
-        StorageTestBase.removefs("FileStorageTests.fs")
-
-        # Now force the database to have transaction records with tids from
-        # the future.
-
-        self.open(create=1)
-        self._storage._ts = timestamp(15)  # 15 minutes in the future
-        db = DB(self._storage)
-        db.close()
-
-        self.open() # this should log a warning
-        db = DB(self._storage)
-        conn = db.open()
-        conn.root()['xyz'] = 1
-        get_transaction().commit()
-        checkIncreasingTids(self._storage)
-        db.close()
-        StorageTestBase.removefs("FileStorageTests.fs")
-
-        # And one more time, with transaction records far in the future.
-        # We expect to log a critical error then, as a time so far in the
-        # future probably indicates a real problem with the system.
-        # Shorter spans may be due to clock drift.
-
-        self.open(create=1)
-        self._storage._ts = timestamp(60)  # an hour in the future
-        db = DB(self._storage)
-        db.close()
-
-        self.open() # this should log a critical error
-        db = DB(self._storage)
-        conn = db.open()
-        conn.root()['xyz'] = 1
-        get_transaction().commit()
-        checkIncreasingTids(self._storage)
-        db.close()
-        StorageTestBase.removefs("FileStorageTests.fs")
-
     def checkStoreBumpsOid(self):
         # If .store() is handed an oid bigger than the storage knows
         # about already, it's crucial that the storage bump its notion
         # of the largest oid in use.
-        t = Transaction()
+        t = transaction.Transaction()
         self._storage.tpc_begin(t)
         giant_oid = '\xee' * 8
         # Store an object.
@@ -300,7 +242,7 @@ class FileStorageTests(
         # knows about already, it's crucial that the storage bump its notion
         # of the largest oid in use.  Because copyTransactionsFrom(), and
         # ZRS recovery, use the .restore() method, this is plain critical.
-        t = Transaction()
+        t = transaction.Transaction()
         self._storage.tpc_begin(t)
         giant_oid = '\xee' * 8
         # Store an object.
@@ -320,14 +262,13 @@ class FileStorageTests(
         # global.
         import time
 
-        from ZODB.DB import DB
         from ZODB.utils import U64, p64
-        from ZODB.fspack import CorruptedError
+        from ZODB.FileStorage.format import CorruptedError
 
         db = DB(self._storage)
         conn = db.open()
         conn.root()['xyz'] = 1
-        get_transaction().commit()
+        transaction.commit()
 
         # Ensure it's all on disk.
         db.close()
@@ -362,6 +303,69 @@ class FileStorageTests(
         else:
             self.fail("expected CorruptedError")
 
+    def check_record_iternext(self):
+
+        db = DB(self._storage)
+        conn = db.open()
+        conn.root()['abc'] = MinPO('abc')
+        conn.root()['xyz'] = MinPO('xyz')
+        transaction.commit()
+
+        # Ensure it's all on disk.
+        db.close()
+        self._storage.close()
+
+        self.open()
+
+        key = None
+        for x in ('\000', '\001', '\002'):
+            oid, tid, data, next_oid = self._storage.record_iternext(key)
+            self.assertEqual(oid, ('\000' * 7) + x)
+            key = next_oid
+            expected_data, expected_tid = self._storage.load(oid, '')
+            self.assertEqual(expected_data, data)
+            self.assertEqual(expected_tid, tid)
+            if x == '\002':
+                self.assertEqual(next_oid, None)
+            else:
+                self.assertNotEqual(next_oid, None)
+
+class FileStorageRecoveryTest(
+    StorageTestBase.StorageTestBase,
+    RecoveryStorage.RecoveryStorage,
+    ):
+
+    def setUp(self):
+        self._storage = ZODB.FileStorage.FileStorage("Source.fs", create=True)
+        self._dst = ZODB.FileStorage.FileStorage("Dest.fs", create=True)
+
+    def tearDown(self):
+        self._storage.close()
+        self._dst.close()
+        self._storage.cleanup()
+        self._dst.cleanup()
+
+    def new_dest(self):
+        return ZODB.FileStorage.FileStorage('Dest.fs')
+
+class SlowFileStorageTest(BaseFileStorageTests):
+
+    level = 2
+
+    def check10Kstores(self):
+        # The _get_cached_serial() method has a special case
+        # every 8000 calls.  Make sure it gets minimal coverage.
+        oids = [[self._storage.new_oid(), None] for i in range(100)]
+        for i in range(100):
+            t = transaction.Transaction()
+            self._storage.tpc_begin(t)
+            for j in range(100):
+                o = MinPO(j)
+                oid, revid = oids[j]
+                serial = self._storage.store(oid, revid, zodb_pickle(o), "", t)
+                oids[j][1] = serial
+            self._storage.tpc_vote(t)
+            self._storage.tpc_finish(t)
 
 # Raise an exception if the tids in FileStorage fs aren't
 # strictly increasing.
@@ -375,40 +379,199 @@ def checkIncreasingTids(fs):
 # Return a TimeStamp object 'minutes' minutes in the future.
 def timestamp(minutes):
     import time
-    from ZODB.TimeStamp import TimeStamp
+    from persistent.TimeStamp import TimeStamp
 
     t = time.time() + 60 * minutes
     return TimeStamp(*time.gmtime(t)[:5] + (t % 60,))
 
+def testTimeTravelOnOpen():
+    """
+    >>> from ZODB.FileStorage import FileStorage
+    >>> from zope.testing.loggingsupport import InstalledHandler
 
-class FileStorageRecoveryTest(
-    StorageTestBase.StorageTestBase,
-    RecoveryStorage.RecoveryStorage,
-    ):
+    Arrange to capture log messages -- they're an important part of
+    this test!
 
-    def setUp(self):
-        StorageTestBase.removefs("Source.fs")
-        StorageTestBase.removefs("Dest.fs")
-        self._storage = ZODB.FileStorage.FileStorage('Source.fs')
-        self._dst = ZODB.FileStorage.FileStorage('Dest.fs')
+    >>> handler = InstalledHandler('ZODB.FileStorage')
 
-    def tearDown(self):
-        self._storage.close()
-        self._dst.close()
-        StorageTestBase.removefs("Source.fs")
-        StorageTestBase.removefs("Dest.fs")
+    Create a new file storage.
 
-    def new_dest(self):
-        StorageTestBase.removefs('Dest.fs')
-        return ZODB.FileStorage.FileStorage('Dest.fs')
+    >>> st = FileStorage('temp.fs', create=True)
+    >>> db = DB(st)
+    >>> db.close()
 
+    First check the normal case:  transactions are recorded with
+    increasing tids, and time doesn't run backwards.
+
+    >>> st = FileStorage('temp.fs')
+    >>> db = DB(st)
+    >>> conn = db.open()
+    >>> conn.root()['xyz'] = 1
+    >>> transaction.get().commit()
+    >>> checkIncreasingTids(st)
+    >>> db.close()
+    >>> st.cleanup() # remove .fs, .index, etc files
+    >>> handler.records   # i.e., no log messages
+    []
+
+    Now force the database to have transaction records with tids from
+    the future.
+
+    >>> st = FileStorage('temp.fs', create=True)
+    >>> st._ts = timestamp(15)  # 15 minutes in the future
+    >>> db = DB(st)
+    >>> db.close()
+
+    >>> st = FileStorage('temp.fs') # this should log a warning
+    >>> db = DB(st)
+    >>> conn = db.open()
+    >>> conn.root()['xyz'] = 1
+    >>> transaction.get().commit()
+    >>> checkIncreasingTids(st)
+    >>> db.close()
+    >>> st.cleanup()
+
+    >>> [record.levelname for record in handler.records]
+    ['WARNING']
+    >>> handler.clear()
+
+    And one more time, with transaction records far in the future.
+    We expect to log a critical error then, as a time so far in the
+    future probably indicates a real problem with the system.  Shorter
+    spans may be due to clock drift.
+
+    >>> st = FileStorage('temp.fs', create=True)
+    >>> st._ts = timestamp(60)  # an hour in the future
+    >>> db = DB(st)
+    >>> db.close()
+
+    >>> st = FileStorage('temp.fs') # this should log a critical error
+    >>> db = DB(st)
+    >>> conn = db.open()
+    >>> conn.root()['xyz'] = 1
+    >>> transaction.get().commit()
+    >>> checkIncreasingTids(st)
+    >>> db.close()
+    >>> st.cleanup()
+
+    >>> [record.levelname for record in handler.records]
+    ['CRITICAL']
+    >>> handler.clear()
+    >>> handler.uninstall()
+    """
+
+def lastInvalidations():
+    """
+
+The last invalidations method is used by a storage server to populate
+it's data structure of recent invalidations.  The lastInvalidations
+method is passed a count and must return up to count number of the
+most recent transactions.
+
+We'll create a FileStorage and populate it with some data, keeping
+track of the transactions along the way:
+
+    >>> fs = ZODB.FileStorage.FileStorage('t.fs', create=True)
+    >>> db = DB(fs)
+    >>> conn = db.open()
+    >>> from persistent.dict import PersistentDict
+    >>> last = []
+    >>> for i in range(100):
+    ...     conn.root()[i] = PersistentDict()
+    ...     transaction.commit()
+    ...     last.append(fs.lastTransaction())
+
+Now, we can call lastInvalidations on it:
+
+    >>> invalidations = fs.lastInvalidations(10)
+    >>> [t for (t, oids) in invalidations] == last[-10:]
+    True
+
+    >>> from ZODB.utils import u64
+    >>> [[int(u64(oid)) for (oid, version) in oids]
+    ...  for (i, oids) in invalidations]
+    ... # doctest: +NORMALIZE_WHITESPACE
+    [[0, 91], [0, 92], [0, 93], [0, 94], [0, 95],
+     [0, 96], [0, 97], [0, 98], [0, 99], [0, 100]]
+
+If we ask for more transactions than there are, we'll get as many as
+there are:
+
+    >>> len(fs.lastInvalidations(1000))
+    101
+
+Of course, calling lastInvalidations on an empty storage refturns no data:
+
+    >>> fs.close()
+    >>> fs = ZODB.FileStorage.FileStorage('t.fs', create=True)
+    >>> list(fs.lastInvalidations(10))
+    []
+
+    """
+
+def deal_with_finish_failures():
+    r"""
+    
+    It's really bad to get errors in FileStorage's _finish method, as
+    that can cause the file storage to be in an inconsistent
+    state. The data file will be fine, but the internal data
+    structures might be hosed. For this reason, FileStorage will close
+    if there is an error after it has finished writing transaction
+    data.  It bothers to do very little after writing this data, so
+    this should rarely, if ever, happen.
+
+    >>> fs = ZODB.FileStorage.FileStorage('data.fs')
+    >>> db = DB(fs)
+    >>> conn = db.open()
+    >>> conn.root()[1] = 1
+    >>> transaction.commit()
+
+    Now, we'll indentially break the file storage. It provides a hook
+    for this purpose. :)
+
+    >>> fs._finish_finish = lambda : None
+    >>> conn.root()[1] = 1
+
+    >>> import zope.testing.loggingsupport
+    >>> handler = zope.testing.loggingsupport.InstalledHandler(
+    ...     'ZODB.FileStorage')
+    >>> transaction.commit()
+    Traceback (most recent call last):
+    ...
+    TypeError: <lambda>() takes no arguments (1 given)
+
+    
+    >>> print handler
+    ZODB.FileStorage CRITICAL
+      Failure in _finish. Closing.
+
+    >>> handler.uninstall()
+
+    >>> fs.load('\0'*8, '')
+    Traceback (most recent call last):
+    ...
+    ValueError: I/O operation on closed file
+
+    >>> db.close()
+    >>> fs = ZODB.FileStorage.FileStorage('data.fs')
+    >>> db = DB(fs)
+    >>> conn = db.open()
+    >>> conn.root()
+    {1: 1}
+
+    >>> transaction.abort()
+    >>> db.close()
+    """
 
 def test_suite():
-    suite = unittest.makeSuite(FileStorageTests, 'check')
-    suite2 = unittest.makeSuite(Corruption.FileStorageCorruptTests, 'check')
-    suite3 = unittest.makeSuite(FileStorageRecoveryTest, 'check')
-    suite.addTest(suite2)
-    suite.addTest(suite3)
+    from zope.testing import doctest
+
+    suite = unittest.TestSuite()
+    for klass in [FileStorageTests, Corruption.FileStorageCorruptTests,
+                  FileStorageRecoveryTest, SlowFileStorageTest]:
+        suite.addTest(unittest.makeSuite(klass, "check"))
+    suite.addTest(doctest.DocTestSuite(setUp=ZODB.tests.util.setUp,
+                                       tearDown=ZODB.tests.util.tearDown))
     return suite
 
 if __name__=='__main__':
