@@ -13,18 +13,24 @@
 
 __version__='$Revision: 1.96 $'[11:-2]
 
-import re, sys, os, time, random, codecs
+import re, sys, os, time, random, codecs, tempfile
 from types import StringType, UnicodeType
-from BaseRequest import BaseRequest
+from BaseRequest import BaseRequest, quote
 from HTTPResponse import HTTPResponse
 from cgi import FieldStorage, escape
-from urllib import quote, unquote, splittype, splitport
+from urllib import unquote, splittype, splitport
 from copy import deepcopy
 from Converters import get_converter
 from TaintedString import TaintedString
 from maybe_lock import allocate_lock
 xmlrpc=None # Placeholder for module that we'll import if we have to.
 
+from zope.i18n.interfaces import IUserPreferredLanguages
+from zope.i18n.locales import locales, LoadLocaleError
+from zope.interface import implements
+from zope.publisher.base import DebugFlags
+from zope.publisher.interfaces.browser import IBrowserRequest
+
 # This may get overwritten during configuration
 default_encoding = 'iso-8859-15'
 
@@ -115,12 +121,19 @@ class HTTPRequest(BaseRequest):
     values will be looked up in the order: environment variables,
     other variables, form data, and then cookies.
     """
+
+    # The claim to implement IBrowserRequest has been made during the Zope3
+    # integration project called Five but hasn't been completed in full.
+
+    implements(IBrowserRequest)
+
     _hacked_path=None
     args=()
     _file=None
     _urls = ()
 
     retry_max_count=3
+
     def supports_retry(self):
         if self.retry_count < self.retry_max_count:
             time.sleep(random.uniform(0, 2**(self.retry_count)))
@@ -238,6 +251,26 @@ class HTTPRequest(BaseRequest):
         """
         return self._client_addr
 
+    def setupLocale(self):
+        envadapter = IUserPreferredLanguages(self, None)
+        if envadapter is None:
+            self._locale = None
+            return
+
+        langs = envadapter.getPreferredLanguages()
+        for httplang in langs:
+            parts = (httplang.split('-') + [None, None])[:3]
+            try:
+                self._locale = locales.getLocale(*parts)
+                return
+            except LoadLocaleError:
+                # Just try the next combination
+                pass
+        else:
+            # No combination gave us an existing locale, so use the default,
+            # which is guaranteed to exist
+            self._locale = locales.getLocale(None, None, None)
+
     def __init__(self, stdin, environ, response, clean=0):
         self._orig_env=environ
         # Avoid the overhead of scrubbing the environment in the
@@ -262,14 +295,24 @@ class HTTPRequest(BaseRequest):
         self.steps=[]
         self._steps=[]
         self._lazies={}
-
+        self._debug = DebugFlags()
+        # We don't set up the locale initially but just on first access
+        self._locale = _marker
 
         if environ.has_key('REMOTE_ADDR'):
             self._client_addr = environ['REMOTE_ADDR']
             if environ.has_key('HTTP_X_FORWARDED_FOR') and self._client_addr in trusted_proxies:
                 # REMOTE_ADDR is one of our trusted local proxies. Not really very remote at all.
                 # The proxy can tell us the IP of the real remote client in the forwarded-for header
-                self._client_addr = environ['HTTP_X_FORWARDED_FOR'].split(',')[-1].strip()
+                # Skip the proxy-address itself though
+                forwarded_for = [
+                    e.strip()
+                    for e in environ['HTTP_X_FORWARDED_FOR'].split(',')]
+                forwarded_for.reverse()
+                for entry in forwarded_for:
+                    if entry not in trusted_proxies:
+                        self._client_addr = entry
+                        break
         else:
             self._client_addr = ''
 
@@ -328,6 +371,7 @@ class HTTPRequest(BaseRequest):
         if script: script="%s/%s" % (server_url,script)
         else:      script=server_url
         other['URL']=self.script=script
+        other['method'] = environ.get('REQUEST_METHOD', 'GET').upper()
 
         ################################################################
         # Cookie values should *not* be appended to existing form
@@ -383,7 +427,7 @@ class HTTPRequest(BaseRequest):
         taintedform=self.taintedform
 
         meth=None
-        fs=FieldStorage(fp=fp,environ=environ,keep_blank_values=1)
+        fs=ZopeFieldStorage(fp=fp,environ=environ,keep_blank_values=1)
         if not hasattr(fs,'list') or fs.list is None:
             # Hm, maybe it's an XML-RPC
             if (fs.headers.has_key('content-type') and
@@ -1213,9 +1257,21 @@ class HTTPRequest(BaseRequest):
             raise KeyError, key
         return v
 
+    # Using the getattr protocol to retrieve form values and similar
+    # is discouraged and is likely to be deprecated in the future.
+    # request.get(key) or request[key] should be used instead
     def __getattr__(self, key, default=_marker, returnTaints=0):
         v = self.get(key, default, returnTaints=returnTaints)
         if v is _marker:
+            if key == 'locale':
+                # we only create the _locale on first access, as setting it
+                # up might be slow and we don't want to slow down every
+                # request
+                if self._locale is _marker:
+                    self.setupLocale()
+                return self._locale
+            if key == 'debug':
+                return self._debug
             raise AttributeError, key
         return v
 
@@ -1395,6 +1451,10 @@ def sane_environment(env):
         except: pass
     return dict
 
+class ZopeFieldStorage(FieldStorage):
+
+    def make_file(self, binary=None):
+        return tempfile.NamedTemporaryFile("w+b")
 
 class FileUpload:
     '''\
@@ -1420,7 +1480,7 @@ class FileUpload:
         else: methods= ['close', 'fileno', 'flush', 'isatty',
                         'read', 'readline', 'readlines', 'seek',
                         'tell', 'truncate', 'write', 'writelines',
-                        '__iter__','next'] # see Collector 1837
+                        '__iter__','next', 'name'] # see Collector 1837
 
         d=self.__dict__
         for m in methods:
@@ -1449,7 +1509,7 @@ def parse_cookie(text,
                  qparmre=re.compile(
                     '([\x00- ]*([^\x00- ;,="]+)="([^"]*)"([\x00- ]*[;,])?[\x00- ]*)'),
                  parmre=re.compile(
-                    '([\x00- ]*([^\x00- ;,="]+)=([^\x00- ;,"]*)([\x00- ]*[;,])?[\x00- ]*)'),
+                    '([\x00- ]*([^\x00- ;,="]+)=([^;,"]*)([\x00- ]*[;,])?[\x00- ]*)'),
                  paramlessre=re.compile(
                     '([\x00- ]*([^\x00- ;,="]+)[\x00- ]*[;,][\x00- ]*)'),
 
@@ -1484,11 +1544,11 @@ def parse_cookie(text,
 
             else:
                 # Broken Cookie without = nor value.
- 		broken_p = paramlessre.match(text)
- 		if broken_p:
- 		    l = len(broken_p.group(1))
- 		    name = broken_p.group(2)
- 		    value = ''
+                broken_p = paramlessre.match(text)
+                if broken_p:
+                    l = len(broken_p.group(1))
+                    name = broken_p.group(2)
+                    value = ''
 
                 else:
                     return result
@@ -1507,7 +1567,7 @@ class record:
     _guarded_writes = 1
 
     def __getattr__(self, key, default=None):
-        if key in ('get', 'keys', 'items', 'values', 'copy', 'has_key'):
+        if key in ('get', 'keys', 'items', 'values', 'copy', 'has_key', '__contains__', '__iter__', '__len__'):
             return getattr(self.__dict__, key)
         raise AttributeError, key
 
