@@ -12,7 +12,7 @@
 ##############################################################################
 '''CGI Response Output formatter
 
-$Id: HTTPResponse.py 99502 2009-04-26 00:12:23Z ldr $'''
+$Id: HTTPResponse.py 92519 2008-10-23 19:24:59Z tseaver $'''
 __version__ = '$Revision: 1.81 $'[11:-2]
 
 import types, os, sys, re
@@ -221,7 +221,7 @@ class HTTPResponse(BaseResponse):
             # It has already been determined.
             return
 
-        if (isinstance(status, (type, types.ClassType))
+        if (isinstance(status, types.ClassType)
          and issubclass(status, Exception)):
             status = status.__name__
 
@@ -246,18 +246,17 @@ class HTTPResponse(BaseResponse):
         if lock:
              self._locked_status = 1
 
-    def setHeader(self, name, value, literal=0, scrubbed=False):
+    def setHeader(self, name, value, literal=0):
         '''\
         Sets an HTTP return header "name" with value "value", clearing
         the previous value set for the header, if one exists. If the
         literal flag is true, the case of the header name is preserved,
         otherwise the header name will be lowercased.'''
-        if not scrubbed:
-            name, value = _scrubHeader(name, value)
+        name, value = _scrubHeader(name, value)
         key = name.lower()
         if accumulate_header(key):
             self.accumulated_headers = (
-                "%s%s: %s\r\n" % (self.accumulated_headers, name, value))
+                "%s%s: %s\n" % (self.accumulated_headers, name, value))
             return
         name = literal and name or key
         self.headers[name] = value
@@ -280,7 +279,7 @@ class HTTPResponse(BaseResponse):
         any previously set headers with the same name.'''
         name, value = _scrubHeader(name, value)
         self.accumulated_headers = (
-            "%s%s: %s\r\n" % (self.accumulated_headers, name, value))
+            "%s%s: %s\n" % (self.accumulated_headers, name, value))
 
     __setitem__ = setHeader
 
@@ -398,9 +397,7 @@ class HTTPResponse(BaseResponse):
                         # was ignored anyway, so cache should not
                         # vary on it. Otherwise if not forced, cache should
                         # respect Accept-Encoding client header
-                        vary = self.getHeader('Vary')
-                        if vary is None or 'Accept-Encoding' not in vary: 
-                            self.appendHeader('Vary','Accept-Encoding')
+                        self.appendHeader('Vary','Accept-Encoding')
         return self
 
     def enableHTTPCompression(self,REQUEST={},force=0,disable=0,query=0):
@@ -595,10 +592,10 @@ class HTTPResponse(BaseResponse):
         headers = self.headers
         if headers.has_key(name):
             h = headers[name]
-            h = "%s%s\r\n\t%s" % (h,delimiter,value)
+            h = "%s%s\n\t%s" % (h,delimiter,value)
         else:
             h = value
-        self.setHeader(name,h, scrubbed=True)
+        self.setHeader(name,h)
 
     def isHTML(self, s):
         s = s.lstrip()
@@ -897,7 +894,7 @@ class HTTPResponse(BaseResponse):
         if self.cookies:
             headersl = headersl+self._cookie_list()
         headersl[len(headersl):] = [self.accumulated_headers, body]
-        return '\r\n'.join(headersl)
+        return '\n'.join(headersl)
 
     def write(self,data):
         """\
