@@ -1,7 +1,16 @@
 import unittest
 from urllib import quote_plus
+ 
+TEST_LARGEFILE_DATA = '''
+--12345
+Content-Disposition: form-data; name="file"; filename="file"
+Content-Type: application/octet-stream
 
-class AuthCredentialsTestsa( unittest.TestCase ):
+test %s
+
+''' % ('test' * 1000)
+
+class AuthCredentialsTests( unittest.TestCase ):
 
     def _getTargetClass(self):
         from ZPublisher.HTTPRequest import HTTPRequest
@@ -77,6 +86,28 @@ class RecordTests( unittest.TestCase ):
         d = eval( r )
         self.assertEqual( d, record.__dict__ )
 
+    def test_contains(self):
+        from ZPublisher.HTTPRequest import record
+        record = record()
+        record.a = 1
+        self.assertTrue('a' in record)
+
+    def test_iter(self):
+        from ZPublisher.HTTPRequest import record
+        record = record()
+        record.a = 1
+        record.b = 2
+        record.c = 3
+        for k in record:
+            self.assertTrue(k in ('a','b','c'))
+
+    def test_len(self):
+        from ZPublisher.HTTPRequest import record
+        record = record()
+        record.a = 1
+        record.b = 2
+        record.c = 3
+        self.assertEqual(len(record), 3)
 
 class ProcessInputsTests(unittest.TestCase):
     def _getHTTPRequest(self, env):
@@ -649,6 +680,20 @@ class ProcessInputsTests(unittest.TestCase):
         self.assertEquals(req.cookies['hmm'], '')
         self.assertEquals(req.cookies['baz'], 'gee')
 
+	# Unquoted multi-space cookies
+        env['HTTP_COOKIE'] = 'single=cookie data; ' \
+	                     'quoted="cookie data with unquoted spaces"; ' \
+			     'multi=cookie data with unquoted spaces; ' \
+			     'multi2=cookie data with unquoted spaces'
+        req = self._getHTTPRequest(env)
+        self.assertEquals(req.cookies['single'], 'cookie data')
+        self.assertEquals(req.cookies['quoted'], 
+	                              'cookie data with unquoted spaces')
+        self.assertEquals(req.cookies['multi'], 
+	                              'cookie data with unquoted spaces')
+        self.assertEquals(req.cookies['multi2'], 
+	                              'cookie data with unquoted spaces')
+
 TEST_ENVIRON = {
     'CONTENT_TYPE': 'multipart/form-data; boundary=12345',
     'REQUEST_METHOD': 'POST',
@@ -684,6 +729,17 @@ class RequestTests( unittest.TestCase ):
         req.close()
         self.assertEqual(start_count, sys.getrefcount(s))  # The test
 
+    def testFileName(self):
+        # checks fileupload object supports the filename
+        from StringIO import StringIO
+        s = StringIO(TEST_LARGEFILE_DATA)
+        env = TEST_ENVIRON.copy()
+        from ZPublisher.HTTPRequest import HTTPRequest
+        req = HTTPRequest(s, env, None)
+        req.processInputs()
+        f = req.form.get('file')
+        self.assert_(f.name)
+
     def testFileIterator(self):
         # checks fileupload object supports the iterator protocol
         # collector entry 1837
@@ -700,11 +756,179 @@ class RequestTests( unittest.TestCase ):
         self.assertEqual(f.next(),'test\n')
         f.seek(0)
         self.assertEqual(f.xreadlines(),f)
-	
+
+    def testDebug(self):
+        TEST_ENVIRON = {
+            'REQUEST_METHOD': 'GET',
+            'SERVER_NAME': 'localhost',
+            'SERVER_PORT': '80',
+            }
+        from StringIO import StringIO
+        from ZPublisher.HTTPRequest import HTTPRequest
+        from zope.publisher.base import DebugFlags
+        s = StringIO('')
+
+        # when accessing request.debug we will see the DebugFlags instance
+        env = TEST_ENVIRON.copy()
+        request = HTTPRequest(s, env, None)
+        self.assert_(isinstance(request.debug, DebugFlags))
+        # It won't be available through dictonary lookup, though
+        self.assert_(request.get('debug') is None)
+
+        # request.debug will actually yield a 'debug' form variable
+        # if it exists
+        env = TEST_ENVIRON.copy()
+        env['QUERY_STRING'] = 'debug=1'
+        request = HTTPRequest(s, env, None)
+        request.processInputs()
+        self.assertEqual(request.debug, '1')
+        self.assertEqual(request.get('debug'), '1')
+        self.assertEqual(request['debug'], '1')
+
+        # we can still override request.debug with a form variable or directly
+        env = TEST_ENVIRON.copy()
+        request = HTTPRequest(s, env, None)
+        request.processInputs()
+        self.assert_(isinstance(request.debug, DebugFlags))
+        request.form['debug'] = '1'
+        self.assertEqual(request.debug, '1')
+        request['debug'] = '2'
+        self.assertEqual(request.debug, '2')
+
+    def testLocale(self):
+        TEST_ENVIRON = {
+            'HTTP_ACCEPT_LANGUAGE': 'en',
+            'REQUEST_METHOD': 'GET',
+            'SERVER_NAME': 'localhost',
+            'SERVER_PORT': '80',
+            }
+        from StringIO import StringIO
+        from ZPublisher.HTTPRequest import HTTPRequest
+        from zope.component import provideAdapter
+        from zope.publisher.browser import BrowserLanguages
+        from zope.publisher.interfaces.http import IHTTPRequest
+        from zope.i18n.interfaces import IUserPreferredLanguages
+        from zope.i18n.interfaces.locales import ILocale
+
+        provideAdapter(BrowserLanguages, [IHTTPRequest],
+                       IUserPreferredLanguages)
+        s = StringIO('')
+
+        # before accessing request.locale for the first time, request._locale
+        # is still a marker
+        from ZPublisher.HTTPRequest import _marker
+        env = TEST_ENVIRON.copy()
+        request = HTTPRequest(s, env, None)
+        self.assert_(request._locale is _marker)
+        # when accessing request.locale we will see an ILocale
+        self.assert_(ILocale.providedBy(request.locale))
+        # and request._locale has been set
+        self.assert_(request._locale is request.locale)
+        # It won't be available through dictonary lookup, though
+        self.assert_(request.get('locale') is None)
+
+        # request.locale will actually yield a 'locale' form variable
+        # if it exists
+        env = TEST_ENVIRON.copy()
+        env['QUERY_STRING'] = 'locale=1'
+        request = HTTPRequest(s, env, None)
+        request.processInputs()
+        self.assertEqual(request.locale, '1')
+        self.assertEqual(request.get('locale'), '1')
+        self.assertEqual(request['locale'], '1')
+
+        # we can still override request.locale with a form variable
+        env = TEST_ENVIRON.copy()
+        request = HTTPRequest(s, env, None)
+        request.processInputs()
+        self.assert_(ILocale.providedBy(request.locale))
+        request.form['locale'] = '1'
+        self.assertEqual(request.locale, '1')
+        request['locale'] = '2'
+        self.assertEqual(request.locale, '2')
+
+        # we should also test the correct semantics of the locale
+        for httplang in ('it', 'it-ch', 'it-CH', 'IT', 'IT-CH', 'IT-ch'):
+            env = TEST_ENVIRON.copy()
+            env['HTTP_ACCEPT_LANGUAGE'] = httplang
+            request = HTTPRequest(s, env, None)
+            locale = request.locale
+            self.assert_(ILocale.providedBy(locale))
+            parts = httplang.split('-')
+            lang = parts.pop(0).lower()
+            territory = variant = None
+            if parts:
+                territory = parts.pop(0).upper()
+            if parts:
+                variant = parts.pop(0).upper()
+            self.assertEqual(locale.id.language, lang)
+            self.assertEqual(locale.id.territory, territory)
+            self.assertEqual(locale.id.variant, variant)
+
+        # Now test for non-existant locale fallback
+        env = TEST_ENVIRON.copy()
+        env['HTTP_ACCEPT_LANGUAGE'] = 'xx'
+        request = HTTPRequest(s, env, None)
+        locale = request.locale
+        self.assert_(ILocale.providedBy(locale))
+        self.assert_(locale.id.language is None)
+        self.assert_(locale.id.territory is None)
+        self.assert_(locale.id.variant is None)
+
+    def testMethod(self):
+        TEST_ENVIRON = {
+            'REQUEST_METHOD': 'GET',
+            'SERVER_NAME': 'localhost',
+            'SERVER_PORT': '80',
+            }
+        from StringIO import StringIO
+        from ZPublisher.HTTPRequest import HTTPRequest
+        s = StringIO('')
+
+        env = TEST_ENVIRON.copy()
+        request = HTTPRequest(s, env, None)
+        self.assertEqual(request.method, 'GET')
+        
+        env = TEST_ENVIRON.copy()
+        env['REQUEST_METHOD'] = 'post'
+        request = HTTPRequest(s, env, None)
+        self.assertEqual(request.method, 'POST')
+
+    def testTrustedProxies(self):
+        TEST_ENVIRON = {
+            'REQUEST_METHOD': 'GET',
+            'SERVER_NAME': 'localhost',
+            'SERVER_PORT': '80',
+            'REMOTE_ADDR': '127.0.0.1',
+            'HTTP_X_FORWARDED_FOR': '10.1.20.30, 192.168.1.100',
+            }
+        from StringIO import StringIO
+        from ZPublisher.HTTPRequest import HTTPRequest, trusted_proxies
+        s = StringIO('')
+
+        env = TEST_ENVIRON.copy()
+        request = HTTPRequest(s, env, None)
+        self.assertEqual(request.getClientAddr(), '127.0.0.1')
+
+        trusted_proxies.append('127.0.0.1')
+        request = HTTPRequest(s, env, None)
+        self.assertEqual(request.getClientAddr(), '192.168.1.100')
+
+        trusted_proxies[0] = '192.168.1.100' 
+        env = TEST_ENVIRON.copy()
+        env['REMOTE_ADDR'] = '192.168.1.100'
+        request = HTTPRequest(s, env, None)
+        self.assertEqual(request.getClientAddr(), '10.1.20.30')
+
+        env = TEST_ENVIRON.copy()
+        del env['REMOTE_ADDR']
+        request = HTTPRequest(s, env, None)
+        self.assertEqual(request.getClientAddr(), '')
+
 
 def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(AuthCredentialsTestsa, 'test'))
+    suite.addTest(unittest.makeSuite(AuthCredentialsTests, 'test'))
     suite.addTest(unittest.makeSuite(RecordTests, 'test'))
     suite.addTest(unittest.makeSuite(ProcessInputsTests, 'test'))
     suite.addTest(unittest.makeSuite(RequestTests, 'test'))
