@@ -12,7 +12,7 @@
 ##############################################################################
 """WebDAV support - resource objects.
 
-$Id: Resource.py 77126 2007-06-27 12:59:31Z sidnei $
+$Id: Resource.py 76767 2007-06-18 16:04:25Z sidnei $
 """
 
 import mimetypes
@@ -116,11 +116,19 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
     def dav__simpleifhandler(self, request, response, method='PUT',
                              col=0, url=None, refresh=0):
         ifhdr = request.get_header('If', None)
-        if Lockable.wl_isLocked(self) and (not ifhdr):
-            raise Locked, "Resource is locked."
 
-        if not ifhdr: return None
-        if not Lockable.wl_isLocked(self): return None
+        lockable = Lockable.wl_isLockable(self)
+        if not lockable:
+            # degenerate case, we shouldnt have even called this method.
+            return None
+
+        locked = self.wl_isLocked()
+
+        if locked and (not ifhdr):
+            raise Locked('Resource is locked.')
+
+        if not ifhdr:
+            return None
 
         # Since we're a simple if handler, and since some clients don't
         # pass in the port information in the resource part of an If
@@ -167,7 +175,6 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         else:
             return 0
 
-
     # WebDAV class 1 support
     security.declareProtected(View, 'HEAD')
     def HEAD(self, REQUEST, RESPONSE):
@@ -468,8 +475,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
         except 'Not Found':
             raise Conflict, 'The resource %s must exist.' % parent_path
         except:
-            t, v, tb=sys.exc_info()
-            raise t, v
+            raise
         if hasattr(parent, '__null_resource__'):
             raise Conflict, 'The resource %s must exist.' % parent_path
         existing=hasattr(aq_base(parent), name)
@@ -523,8 +529,7 @@ class Resource(ExtensionClass.Base, Lockable.LockableItem):
                 if not itrue:
                     raise PreconditionFailed, 'Condition failed.'
             else:
-                raise PreconditionFailed, 'Source is locked and no '\
-                      'condition was passed in.'
+                raise Locked('Source is locked and no condition was passed in')
 
         orig_container = aq_parent(aq_inner(self))
         orig_id = self.getId()
