@@ -12,7 +12,7 @@
 ##############################################################################
 """WebDAV xml request objects.
 
-$Id: davcmds.py 69164 2006-07-17 23:34:32Z sidnei $
+$Id: davcmds.py 76724 2007-06-17 01:06:40Z chrism $
 """
 
 import sys
@@ -219,8 +219,7 @@ class PropPatch:
                     else:
                         # xml property
                         attrs={}
-                        prop.remap({ns:'n'})
-                        prop.del_attr('xmlns:n')
+                        prop.remove_namespace_attrs()
                         for attr in prop.attrs():
                             attrs[attr.qname()]=attr.value()
                         md={'__xml_attrs__':attrs}
@@ -437,14 +436,18 @@ class Unlock:
         islockable = IWriteLock.providedBy(obj) or \
                      WriteLockInterface.isImplementedBy(obj)
 
-        if islockable and obj.wl_hasLock(token):
-            method = getattr(obj, 'wl_delLock')
-            vld = getSecurityManager().validate(None,obj,'wl_delLock',method)
-            if vld:
-                obj.wl_delLock(token)
+        if islockable:
+            if obj.wl_hasLock(token):
+                method = getattr(obj, 'wl_delLock')
+                vld = getSecurityManager().validate(None,obj,
+                                                    'wl_delLock',method)
+                if vld:
+                    obj.wl_delLock(token)
+                else:
+                    errmsg = "403 Forbidden"
             else:
-                errmsg = "403 Forbidden"
-        elif not islockable:
+                errmsg = '400 Bad Request'
+        else:
             # Only set an error message if the command is being applied
             # to a top level object.  Otherwise, we're descending a tree
             # which may contain many objects that don't implement locking,
