@@ -10,6 +10,12 @@ class TestLockItem(unittest.TestCase):
 
         verifyClass(LockItemInterface, LockItem)
 
+    def test_z3interfaces(self):
+        from webdav.interfaces import ILockItem
+        from webdav.LockItem import LockItem
+        from zope.interface.verify import verifyClass
+
+        verifyClass(ILockItem, LockItem)
 
 def test_suite():
     return unittest.TestSuite((
