@@ -13,10 +13,16 @@
 ##############################################################################
 """Zope Container Package
 
-This package uses pkgutil so that the package can be split over
+This package uses setuptools / pkgutil so that the package can be split over
 multiple directories.
 
-$Id: __init__.py 39579 2005-10-24 17:08:50Z jim $
+See http://peak.telecommunity.com/DevCenter/setuptools#namespace-packages
+
+$Id: __init__.py 79263 2007-08-25 14:38:28Z hannosch $
 """
-from pkgutil import extend_path
-__path__ = extend_path(__path__, __name__)
+
+try:
+    __import__('pkg_resources').declare_namespace(__name__)
+except ImportError:
+    from pkgutil import extend_path
+    __path__ = extend_path(__path__, __name__)
