@@ -13,5 +13,16 @@
 ##############################################################################
 """Zope application package.
 
-$Id: __init__.py 68989 2006-07-05 21:21:49Z tseaver $
+This package uses setuptools / pkgutil so that the package can be split over
+multiple directories.
+
+See http://peak.telecommunity.com/DevCenter/setuptools#namespace-packages
+
+$Id: __init__.py 79263 2007-08-25 14:38:28Z hannosch $
 """
+
+try:
+    __import__('pkg_resources').declare_namespace(__name__)
+except ImportError:
+    from pkgutil import extend_path
+    __path__ = extend_path(__path__, __name__)
