@@ -13,25 +13,25 @@
 ##############################################################################
 """Component Inspection Utilities
 
-$Id: component.py 70020 2006-09-07 09:08:16Z flox $
+$Id: component.py 81215 2007-10-30 18:43:39Z srichter $
 """
 __docformat__ = 'restructuredtext'
 import types
+import zope.interface.declarations
 
 from zope.component import getGlobalSiteManager
 from zope.component.interfaces import IFactory
 from zope.component.registry import (
     AdapterRegistration,
-    SubscriptionRegistration,
     HandlerRegistration,
+    SubscriptionRegistration,
+    UtilityRegistration
     )
-from zope.component.registry import UtilityRegistration
+from zope.i18nmessageid import ZopeMessageFactory as _
 from zope.interface import Interface
 from zope.interface.interface import InterfaceClass
-import zope.interface.declarations
 from zope.publisher.interfaces import IRequest
 
-from zope.app.i18n import ZopeMessageFactory as _
 from zope.app.apidoc.classregistry import classRegistry
 from zope.app.apidoc.utilities import relativizePath, truncateSysPath
 from zope.app.apidoc.utilities import getPythonPath, isReferencable, renderText
@@ -177,6 +177,33 @@ def getInterfaceInfoDictionary(iface):
     return {'module': getattr(iface, '__module__', _('<unknown>')),
             'name': getattr(iface, '__name__', _('<unknown>'))}
 
+def getInterfaceInfoDictionary(iface):
+    """Return a PT-friendly info dictionary for an interface."""
+    if isinstance(iface, zope.interface.declarations.Implements):
+        iface = iface.inherit
+    if iface is None:
+        return None
+    return {'module': getattr(iface, '__module__', _('<unknown>')),
+            'name': getattr(iface, '__name__', _('<unknown>'))}
+
+
+def getTypeInfoDictionary(type):
+    """Return a PT-friendly info dictionary for a type."""
+    path = getPythonPath(type)
+    return {'name': type.__name__,
+            'module': type.__module__,
+            'url': isReferencable(path) and path.replace('.', '/') or None}
+
+def getSpecificationInfoDictionary(spec):
+    """Return an info dictionary for one specification."""
+    info = {'isInterface': False, 'isType': False}
+    if zope.interface.interfaces.IInterface.providedBy(spec):
+        info.update(getInterfaceInfoDictionary(spec))
+        info['isInterface'] = True
+    else:
+        info.update(getTypeInfoDictionary(spec.inherit))
+        info['isType'] = True
+    return info
 
 def getAdapterInfoDictionary(reg):
     """Return a PT-friendly info dictionary for an adapter registration."""
@@ -196,7 +223,7 @@ def getAdapterInfoDictionary(reg):
 
     return {
         'provided': getInterfaceInfoDictionary(reg.provided),
-        'required': [getInterfaceInfoDictionary(iface)
+        'required': [getSpecificationInfoDictionary(iface)
                      for iface in reg.required
                      if iface is not None],
         'name': unicode(getattr(reg, 'name', u'')),
