@@ -13,26 +13,21 @@
 ##############################################################################
 """Broken-object support
 
-$Id: broken.py 38178 2005-08-30 21:50:19Z mj $
+$Id: broken.py 72735 2007-02-21 05:02:33Z baijum $
 """
 __docformat__ = 'restructuredtext'
 
 import ZODB.broken
 import zope.interface
-import zope.app.location.interfaces
+import zope.location.interfaces
 import zope.security.checker
-
-
-from zope.app.annotation.interfaces import IAnnotations
-
-class IBroken(zope.interface.Interface):
-    """Marker interface for broken objects
-    """
+from zope.app.broken.interfaces import IBroken
+from zope.annotation.interfaces import IAnnotations
 
 class Broken(ZODB.broken.Broken):
     zope.interface.implements(
         IBroken,
-        zope.app.location.interfaces.ILocation,
+        zope.location.interfaces.ILocation,
         IAnnotations,
         )
 
@@ -96,7 +91,7 @@ def installBroken(event):
 
     It implements ILocation and IAnnotations::
 
-      >>> zope.app.location.interfaces.ILocation.implementedBy(cls)
+      >>> zope.location.interfaces.ILocation.implementedBy(cls)
       True
       >>> IAnnotations.implementedBy(cls)
       True
