@@ -16,31 +16,32 @@
 The Adding View is used to add new objects to a container. It is sort of a
 factory screen.
 
-$Id: adding.py 65722 2006-03-02 14:44:16Z hdima $
+$Id: adding.py 73547 2007-03-25 09:03:04Z dobe $
 """
+
 __docformat__ = 'restructuredtext'
-from warnings import warn
 
 import zope.security.checker
 from zope.component.interfaces import IFactory
 from zope.event import notify
 from zope.interface import implements
 from zope.publisher.interfaces import IPublishTraverse
+from zope.publisher.browser import BrowserView
 from zope.security.proxy import removeSecurityProxy
+from zope.exceptions.interfaces import UserError
+from zope.location import LocationProxy
+from zope.lifecycleevent import ObjectCreatedEvent
 
-from zope.app.exception.interfaces import UserError
 from zope.app.container.interfaces import IAdding, INameChooser
 from zope.app.container.interfaces import IContainerNamesContainer
 from zope.app.container.constraints import checkFactory, checkObject
 
 from zope.app import zapi
-from zope.app.event.objectevent import ObjectCreatedEvent
-from zope.app.i18n import ZopeMessageFactory as _
-from zope.app.location import LocationProxy
+from zope.app.container.i18n import ZopeMessageFactory as _
 from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
-from zope.app.publisher.browser import BrowserView
 from zope.app.publisher.browser.menu import getMenu
 
+
 class Adding(BrowserView):
     implements(IAdding, IPublishTraverse)
 
@@ -82,11 +83,6 @@ class Adding(BrowserView):
     request = None
     context = None
 
-    def renderAddButton(self):
-        warn("The renderAddButton method is deprecated, use nameAllowed",
-            DeprecationWarning, 2)
-
-
     def publishTraverse(self, request, name):
         """See zope.app.container.interfaces.IAdding"""
         if '=' in name:
@@ -199,9 +195,3 @@ class Adding(BrowserView):
            if 'has_custom_add_view' in menu_item:
                return True
        return False
-
-
-class ContentAdding(Adding):
-
-    menu_id = "add_content"
-
