@@ -13,58 +13,57 @@
 ##############################################################################
 """Browser widgets for items
 
-$Id: itemswidgets.py 70410 2006-09-28 07:30:15Z yuppie $
+$Id: itemswidgets.py 77294 2007-07-02 09:52:32Z mgedmin $
 """
 __docformat__ = 'restructuredtext'
 import sets
 from xml.sax.saxutils import escape
 
+from zope import component
 from zope.interface import implements
 from zope.i18n import translate
 from zope.schema.interfaces import ValidationError, InvalidValue
 from zope.schema.interfaces import ConstraintNotSatisfied, ITitledTokenizedTerm
 
-from zope.app import zapi
 from zope.app.form.browser.widget import SimpleInputWidget, renderElement
 from zope.app.form.interfaces import IInputWidget, IDisplayWidget
 from zope.app.form.interfaces import ConversionError
+from zope.app.form.browser.i18n import _
 from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
 
-from zope.app.i18n import ZopeMessageFactory as _
-
 
 # For choices, we want to make the widget a view of the field and vocabulary.
 
 def ChoiceDisplayWidget(field, request):
-    return zapi.getMultiAdapter((field, field.vocabulary, request),
-                                IDisplayWidget)
+    return component.getMultiAdapter((field, field.vocabulary, request),
+                                     IDisplayWidget)
 
 def ChoiceInputWidget(field, request):
-    return zapi.getMultiAdapter((field, field.vocabulary, request),
-                                IInputWidget)
+    return component.getMultiAdapter((field, field.vocabulary, request),
+                                     IInputWidget)
 
 # for collections, we want to make the widget a view of the field and the
 # value_type.  If the value_type is None we may fall over.  We may
 # not be able to do any better than that.
 
 def CollectionDisplayWidget(field, request):
-    return zapi.getMultiAdapter((field, field.value_type, request),
-                                IDisplayWidget)
+    return component.getMultiAdapter((field, field.value_type, request),
+                                     IDisplayWidget)
 
 def CollectionInputWidget(field, request):
-    return zapi.getMultiAdapter((field, field.value_type, request),
-                                IInputWidget)
+    return component.getMultiAdapter((field, field.value_type, request),
+                                     IInputWidget)
 
 # for collections of choices, we want to make the widget a view of the field,
 # the value type, and the vocabulary.
 
 def ChoiceCollectionDisplayWidget(field, value_type, request):
-    return zapi.getMultiAdapter((field, value_type.vocabulary, request),
-                                IDisplayWidget)
+    return component.getMultiAdapter((field, value_type.vocabulary, request),
+                                     IDisplayWidget)
 
 def ChoiceCollectionInputWidget(field, value_type, request):
-    return zapi.getMultiAdapter((field, value_type.vocabulary, request),
-                                IInputWidget)
+    return component.getMultiAdapter((field, value_type.vocabulary, request),
+                                     IInputWidget)
 
 class TranslationHook(object):
     """A mixin class that provides the translation capabilities."""
@@ -107,9 +106,10 @@ class ItemsWidgetBase(TranslationHook, SimpleInputWidget):
         known to be a string, or str()able.
 
         """
-        if ITitledTokenizedTerm.providedBy(term):
-            return self.translate(term.title)
-        return term.token
+        titled = ITitledTokenizedTerm(term, None)
+        if titled is None:
+            return term.token
+        return self.translate(titled.title)
 
     def convertTokensToValues(self, tokens):
         """Convert term tokens to the terms themselves.
@@ -190,8 +190,8 @@ class MultiDataHelper(object):
     def _toFieldValue(self, input):
         """See SimpleInputWidget"""
         if input is None:
-            return []
-        if not isinstance(input, list):
+            input = []
+        elif not isinstance(input, list):
             input = [input]
         try:
             values = self.convertTokensToValues(input)
@@ -200,9 +200,12 @@ class MultiDataHelper(object):
 
         # All AbstractCollection fields have a `_type` attribute specifying
         # the type of collection. Use it to generate the correct type,
-        # otherwise return a list. 
+        # otherwise return a list.  TODO: this breaks encapsulation.
         if hasattr(self.context, '_type'):
-            return self.context._type(values)
+            _type = self.context._type
+            if isinstance(_type, tuple):
+                _type = _type[0]
+            return _type(values)
         else:
             return values
 
@@ -255,8 +258,6 @@ class ItemsMultiDisplayWidget(MultiDataHelper, ItemsWidgetBase):
         if value:
             rendered_items = self.renderItems(value)
             return renderElement(self.tag,
-                                 type=self.type,
-                                 name=self.name,
                                  id=self.name,
                                  cssClass=self.cssClass,
                                  contents="\n".join(rendered_items),
@@ -444,30 +445,30 @@ class RadioWidget(SelectWidget):
     _messageNoValue = _("vocabulary-missing-single-value-for-edit",
                         "(no value)")
 
-    _joinButtonToMessageTemplate = u"<label>%s&nbsp;%s</label>"
-
     def renderItem(self, index, text, value, name, cssClass):
         """Render an item of the list."""
-        id = '%s.%s' % (name, index)
-        elem = renderElement(u'input',
-                             value=value,
-                             name=name,
-                             id=id,
-                             cssClass=cssClass,
-                             type='radio')
-        return self._joinButtonToMessageTemplate % (elem, text)
+        return self._renderItem(index, text, value, name, cssClass)
 
     def renderSelectedItem(self, index, text, value, name, cssClass):
         """Render a selected item of the list."""
+        return self._renderItem(index, text, value, name, cssClass,
+                                checked=True)
+
+    def _renderItem(self, index, text, value, name, cssClass, checked=False):
+        kw = {}
+        if checked:
+            kw['checked'] = 'checked'
         id = '%s.%s' % (name, index)
         elem = renderElement(u'input',
                              value=value,
                              name=name,
                              id=id,
                              cssClass=cssClass,
-                             checked="checked",
-                             type='radio')
-        return self._joinButtonToMessageTemplate %(elem, text)
+                             type='radio',
+                             **kw)
+        return renderElement(u'label',
+                             contents='%s&nbsp;%s' % (elem, text),
+                             **{'for': id})
 
     def renderValue(self, value):
         rendered_items = self.renderItems(value)
@@ -493,7 +494,7 @@ class ItemsMultiEditWidgetBase(MultiDataHelper, ItemsEditWidgetBase):
 
     def renderValue(self, value):
         # All we really add here is the ':list' in the name argument
-        # and mutliple=None to renderElement().
+        # and mutliple='multiple' to renderElement().
         rendered_items = self.renderItems(value)
         return renderElement(self.tag,
                              name=self.name + ':list',
@@ -531,6 +532,15 @@ class MultiSelectSetWidget(MultiSelectWidget):
         return value
 
 
+class MultiSelectFrozenSetWidget(MultiSelectWidget):
+    """Provide a selection list for the set to be selected."""
+
+    def _toFieldValue(self, input):
+        value = super(MultiSelectFrozenSetWidget, self)._toFieldValue(input)
+        if isinstance(value, list):
+            value = frozenset(value)
+        return value
+
 class OrderedMultiSelectWidget(ItemsMultiEditWidgetBase):
     """A multi-selection widget with ordering support."""
 
@@ -553,7 +563,7 @@ class OrderedMultiSelectWidget(ItemsMultiEditWidgetBase):
         values = self._getFormValue()
         # Not all content objects must necessarily support the attributes
         if hasattr(self.context.context, self.context.__name__):
-            # merge in values from content 
+            # merge in values from content
             for value in self.context.get(self.context.context):
                 if value not in values:
                     values.append(value)
@@ -582,16 +592,18 @@ class MultiCheckBoxWidget(ItemsMultiEditWidgetBase):
             return "<br />".join(rendered_items)
 
     def renderItem(self, index, text, value, name, cssClass):
-        id = '%s.%s' % (name, index)
-        elem = renderElement('input',
-                             type="checkbox",
-                             cssClass=cssClass,
-                             name=name,
-                             id=id,
-                             value=value)
-        return self._joinButtonToMessageTemplate %(elem, text)
+        """Render an item of the list."""
+        return self._renderItem(index, text, value, name, cssClass)
 
     def renderSelectedItem(self, index, text, value, name, cssClass):
+        """Render a selected item of the list."""
+        return self._renderItem(index, text, value, name, cssClass,
+                                checked=True)
+
+    def _renderItem(self, index, text, value, name, cssClass, checked=False):
+        kw = {}
+        if checked:
+            kw['checked'] = 'checked'
         id = '%s.%s' % (name, index)
         elem = renderElement('input',
                              type="checkbox",
@@ -599,5 +611,9 @@ class MultiCheckBoxWidget(ItemsMultiEditWidgetBase):
                              name=name,
                              id=id,
                              value=value,
-                             checked="checked")
-        return self._joinButtonToMessageTemplate %(elem, text)
+                             **kw)
+        contents = self._joinButtonToMessageTemplate % (elem, text)
+        return renderElement(u'label',
+                             contents=contents,
+                             **{'for': id})
+
