@@ -11,11 +11,11 @@
 ##############################################################################
 """HTTP DELETE verb
 
-$Id: delete.py 40824 2005-12-16 18:58:49Z mgedmin $
+$Id: delete.py 77110 2007-06-26 17:53:39Z mkerrin $
 """
 __docformat__ = 'restructuredtext'
 
-from zope.app.filerepresentation.interfaces import IWriteDirectory
+from zope.filerepresentation.interfaces import IWriteDirectory
 from zope.app.publication.http import MethodNotAllowed
 
 
@@ -28,13 +28,10 @@ class DELETE(object):
         self.request = request
 
     def DELETE(self):
-        request = self.request
-
         victim = self.context
         container = victim.__parent__
         name = victim.__name__
 
-
         # Get a "directory" surrogate for the container
         dir = IWriteDirectory(container, None)
         if dir is None:
@@ -43,5 +40,3 @@ class DELETE(object):
         del dir[name]
 
         return ''
-
-
