@@ -13,27 +13,34 @@
 ##############################################################################
 """Tests for the unique id utility.
 
-$Id: tests.py 68719 2006-06-17 21:51:14Z ctheune $
+$Id: tests.py 81076 2007-10-25 00:03:15Z srichter $
 """
+import re
 import unittest
+import BTrees
 
 from persistent import Persistent
 from persistent.interfaces import IPersistent
+from transaction import commit
 from ZODB.interfaces import IConnection
 
 from zope.interface import implements
 from zope.interface.verify import verifyObject
 from zope.location.interfaces import ILocation
 
+from zope.component.interfaces import IFactory
+
 from zope.app.testing import setup, ztapi
 from zope.app import zapi
 from zope.app.component.hooks import setSite
 
-from zope.app.intid.interfaces import IIntIds
 from zope.app.intid import IntIds
+from zope.app.intid.interfaces import IIntIds
+from zope.app.intid.testing import IntIdLayer
 from zope.app.keyreference.persistent import KeyReferenceToPersistent
 from zope.app.keyreference.persistent import connectionOfPersistent
 from zope.app.keyreference.interfaces import IKeyReference
+from zope.app.testing.functional import BrowserTestCase
 
 
 class P(Persistent):
@@ -47,7 +54,7 @@ class ConnectionStub(object):
         return self
 
     database_name = 'ConnectionStub'
-    
+
     def add(self, ob):
         ob._p_jar = self
         ob._p_oid = self.next
@@ -68,11 +75,13 @@ class ReferenceSetupMixin(object):
 
 class TestIntIds(ReferenceSetupMixin, unittest.TestCase):
 
+    createIntIds = IntIds
+
     def test_interface(self):
-        verifyObject(IIntIds, IntIds())
+        verifyObject(IIntIds, self.createIntIds())
 
     def test_non_keyreferences(self):
-        u = IntIds()
+        u = self.createIntIds()
         obj = object()
 
         self.assert_(u.queryId(obj) is None)
@@ -80,9 +89,9 @@ class TestIntIds(ReferenceSetupMixin, unittest.TestCase):
         self.assertRaises(KeyError, u.getId, obj)
 
     def test(self):
-        u = IntIds()
+        u = self.createIntIds()
         obj = P()
-        
+
         obj._p_jar = ConnectionStub()
 
         self.assertRaises(KeyError, u.getId, obj)
@@ -110,7 +119,7 @@ class TestIntIds(ReferenceSetupMixin, unittest.TestCase):
     def test_btree_long(self):
         # This is a somewhat arkward test, that *simulates* the border case
         # behaviour of the _generateId method
-        u = IntIds()
+        u = self.createIntIds()
         u._randrange = lambda x,y:int(2**31-1)
 
         # The chosen int is exactly the largest number possible that is
@@ -124,7 +133,7 @@ class TestIntIds(ReferenceSetupMixin, unittest.TestCase):
         self.failUnless(2**31-1 in tuple(u.refs.keys()))
 
     def test_len_items(self):
-        u = IntIds()
+        u = self.createIntIds()
         obj = P()
         obj._p_jar = ConnectionStub()
 
@@ -162,7 +171,7 @@ class TestIntIds(ReferenceSetupMixin, unittest.TestCase):
         self.assertEquals(u.items(), [])
 
     def test_getenrateId(self):
-        u = IntIds()
+        u = self.createIntIds()
         self.assertEquals(u._v_nextid, None)
         id1 = u._generateId()
         self.assert_(u._v_nextid is not None)
@@ -242,10 +251,67 @@ class TestSubscribers(ReferenceSetupMixin, unittest.TestCase):
         self.assertEquals(events[0].original_event.object, parent_folder)
         self.assertEquals(events[0].object, folder)
 
+
+class TestIntIds64(TestIntIds):
+
+    def createIntIds(self):
+        return IntIds(family=BTrees.family64)
+
+
+class TestFunctionalIntIds(BrowserTestCase):
+
+    def setUp(self):
+        from zope.app.intid import IntIds
+        from zope.app.intid.interfaces import IIntIds
+
+        BrowserTestCase.setUp(self)
+
+        self.basepath = '/++etc++site/default'
+        root = self.getRootFolder()
+
+        sm = zapi.traverse(root, '/++etc++site')
+        setup.addUtility(sm, 'intid', IIntIds, IntIds())
+        commit()
+
+        type_name = 'BrowserAdd__zope.app.intid.IntIds'
+
+        response = self.publish(
+            self.basepath + '/contents.html',
+            basic='mgr:mgrpw',
+            form={'type_name': type_name,
+                  'new_value': 'mgr' })
+
+    def test(self):
+        response = self.publish(self.basepath + '/intid/@@index.html',
+                                basic='mgr:mgrpw')
+        self.assertEquals(response.getStatus(), 200)
+        # The utility registers in itself when it is being added
+        self.assert_(response.getBody().find('1 objects') > 0)
+        self.assert_('<a href="/++etc++site">/++etc++site</a>'
+                     not in response.getBody())
+
+        response = self.publish(self.basepath + '/intid/@@populate',
+                                basic='mgr:mgrpw')
+        self.assertEquals(response.getStatus(), 302)
+
+        response = self.publish(self.basepath
+                                + '/intid/@@index.html?testing=1',
+                                basic='mgr:mgrpw')
+        self.assertEquals(response.getStatus(), 200)
+        body = response.getBody()
+        self.assert_('3 objects' in body)
+        self.assert_('<a href="/++etc++site">/++etc++site</a>' in body)
+        self.checkForBrokenLinks(body, response.getPath(), basic='mgr:mgrpw')
+
+
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(TestIntIds))
+    suite.addTest(unittest.makeSuite(TestIntIds64))
     suite.addTest(unittest.makeSuite(TestSubscribers))
+    # Functional Tests
+    TestFunctionalIntIds.layer = IntIdLayer
+    suite.addTest(unittest.makeSuite(TestFunctionalIntIds))
     return suite
 
 if __name__ == '__main__':
