@@ -13,12 +13,13 @@
 ##############################################################################
 """'browser' namespace directive tests
 
-$Id: test_directives.py 40517 2005-12-03 13:12:15Z ctheune $
+$Id: test_directives.py 79176 2007-08-23 21:05:22Z philikon $
 """
 import os
 import unittest
 from cStringIO import StringIO
 
+from zope import component
 from zope.interface import Interface, implements, directlyProvides, providedBy
 
 import zope.security.management
@@ -26,26 +27,23 @@ from zope.component.interfaces import IDefaultViewName
 from zope.configuration.xmlconfig import xmlconfig, XMLConfig
 from zope.configuration.exceptions import ConfigurationError
 from zope.publisher.browser import TestRequest
-from zope.publisher.interfaces import ILayer
 from zope.publisher.interfaces.browser import IBrowserPublisher
 from zope.publisher.interfaces.browser import IBrowserRequest
-from zope.publisher.interfaces.browser import ISkin, IDefaultSkin
+from zope.publisher.interfaces.browser import IBrowserSkinType, IDefaultSkin
 from zope.security.proxy import removeSecurityProxy, ProxyFactory
+from zope.security.permission import Permission 
+from zope.security.interfaces import IPermission 
 from zope.testing.doctestunit import DocTestSuite
+from zope.traversing.adapters import DefaultTraversable
+from zope.traversing.interfaces import ITraversable
 
 import zope.app.publisher.browser
-from zope.app import zapi
 from zope.app.component.tests.views import IC, V1, VZMI, R1, IV
 from zope.app.publisher.browser.fileresource import FileResource
 from zope.app.publisher.browser.i18nfileresource import I18nFileResource
 from zope.app.publisher.browser.menu import getFirstMenuItem
 from zope.app.publisher.interfaces.browser import IMenuItemType
-from zope.app.security.permission import Permission 
-from zope.app.security.interfaces import IPermission 
 from zope.app.testing import placelesssetup, ztapi
-from zope.app.traversing.adapters import DefaultTraversable
-from zope.app.traversing.interfaces import ITraversable
-
 
 tests_path = os.path.join(
     os.path.dirname(zope.app.publisher.browser.__file__),
@@ -104,10 +102,12 @@ class ITestMenu(Interface):
 
 directlyProvides(ITestMenu, IMenuItemType)
 
+
 class ITestLayer(IBrowserRequest):
     """Test Layer."""
 
-directlyProvides(ITestLayer, ILayer)
+class ITestSkin(ITestLayer):
+    """Test Skin."""
 
 
 class MyResource(object):
@@ -123,32 +123,10 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         XMLConfig('meta.zcml', zope.app.publisher.browser)()
         ztapi.provideAdapter(None, ITraversable, DefaultTraversable)
 
-    def testLayer(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
-        xmlconfig(StringIO(
-            template % '<browser:layer name="testlayer" />'
-            ))
-        testlayer = zapi.getUtility(ILayer, "testlayer")
-        import zope.app.layers
-        self.assert_(zope.app.layers.testlayer is testlayer)
-
-    def testSkin(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
-        xmlconfig(StringIO(template % (
-            '''
-            <browser:layer name="default" />
-            <browser:skin name="testskin" layers="default" />
-            '''
-            )))
-        testskin = zapi.getUtility(ISkin, "testskin")
-        import zope.app.skins
-        self.assert_(zope.app.skins.testskin is testskin)
-
     def testPage(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template % (
             '''
@@ -162,12 +140,13 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             )))
 
-        v = zapi.queryMultiAdapter((ob, request), name='test')
+        v = component.queryMultiAdapter((ob, request), name='test')
         self.assert_(issubclass(v.__class__, V1))
 
     def testPageWithClassWithMenu(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
         testtemplate = os.path.join(tests_path, 'testfiles', 'test.pt')
                          
 
@@ -189,13 +168,14 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
         self.assertEqual(menuItem["title"], "Test View")
         self.assertEqual(menuItem["action"], "@@test")
-        v = zapi.queryMultiAdapter((ob, request), name='test')
+        v = component.queryMultiAdapter((ob, request), name='test')
         self.assertEqual(v(), "<html><body><p>test</p></body></html>\n")
 
 
     def testPageWithTemplateWithMenu(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
         testtemplate = os.path.join(tests_path, 'testfiles', 'test.pt')
                          
         xmlconfig(StringIO(template % (
@@ -216,13 +196,14 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
         self.assertEqual(menuItem["title"], "Test View")
         self.assertEqual(menuItem["action"], "@@test")
-        v = zapi.queryMultiAdapter((ob, request), name='test')
+        v = component.queryMultiAdapter((ob, request), name='test')
         self.assertEqual(v(), "<html><body><p>test</p></body></html>\n")
 
 
     def testPageInPagesWithTemplateWithMenu(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
         testtemplate = os.path.join(tests_path, 'testfiles', 'test.pt')
 
         xmlconfig(StringIO(template % (
@@ -245,13 +226,14 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
         self.assertEqual(menuItem["title"], "Test View")
         self.assertEqual(menuItem["action"], "@@test")
-        v = zapi.queryMultiAdapter((ob, request), name='test')
+        v = component.queryMultiAdapter((ob, request), name='test')
         self.assertEqual(v(), "<html><body><p>test</p></body></html>\n")
 
 
     def testPageInPagesWithClassWithMenu(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
         testtemplate = os.path.join(tests_path, 'testfiles', 'test.pt')
                          
 
@@ -276,12 +258,12 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         menuItem = getFirstMenuItem('test_menu', ob, TestRequest())
         self.assertEqual(menuItem["title"], "Test View")
         self.assertEqual(menuItem["action"], "@@test")
-        v = zapi.queryMultiAdapter((ob, request), name='test')
+        v = component.queryMultiAdapter((ob, request), name='test')
         self.assertEqual(v(), "<html><body><p>test</p></body></html>\n")
 
     def testDefaultView(self):
         self.assertEqual(
-            zapi.queryMultiAdapter((ob, request), IDefaultViewName),
+            component.queryMultiAdapter((ob, request), IDefaultViewName),
             None)
 
         xmlconfig(StringIO(template % (
@@ -292,7 +274,9 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             )))
 
-        self.assertEqual(zapi.getDefaultViewName(ob, request), 'test')
+        self.assertEqual(
+            zope.app.publisher.browser.getDefaultViewName(ob, request),
+            'test')
 
     def testDefaultViewWithLayer(self):
         class FakeRequest(TestRequest):
@@ -300,7 +284,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         request2 = FakeRequest()
 
         self.assertEqual(
-            zapi.queryMultiAdapter((ob, request2), IDefaultViewName),
+            component.queryMultiAdapter((ob, request2), IDefaultViewName),
             None)
 
         xmlconfig(StringIO(template % (
@@ -318,48 +302,79 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             )))
 
-        self.assertEqual(zapi.getDefaultViewName(ob, request2), 'test2')
-        self.assertEqual(zapi.getDefaultViewName(ob, request), 'test')
-
-
-    def testSkinResource(self):
         self.assertEqual(
-            zapi.queryAdapter(Request(IV), name='test'), None)
+            zope.app.publisher.browser.getDefaultViewName(ob, request2),
+            'test2')
+        self.assertEqual(
+            zope.app.publisher.browser.getDefaultViewName(ob, request),
+            'test')
+
+    def testDefaultViewForClass(self):
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), IDefaultViewName),
+            None)
 
         xmlconfig(StringIO(template % (
             '''
-            <browser:layer name="zmi" />
-            <browser:skin name="zmi" layers="zmi" />
-            <browser:resource
+            <browser:defaultView
+                for="zope.app.publisher.browser.tests.test_directives.Ob"
                 name="test"
-                factory="zope.app.component.tests.views.RZMI"
-                layer="zmi" />
-            <browser:resource
-                name="test"
-                factory="zope.app.component.tests.views.R1" />
+                />
             '''
             )))
 
         self.assertEqual(
-            zapi.queryAdapter(request, name='test').__class__, R1)
-        zmi = zapi.getUtility(ISkin, 'zmi')
+            zope.app.publisher.browser.getDefaultViewName(ob, request),
+            'test')
+
+
+    def testSkinResource(self):
         self.assertEqual(
-            zapi.queryAdapter(TestRequest(skin=zmi), name='test').__class__,
+            component.queryAdapter(Request(IV), name='test'), None)
+
+        xmlconfig(StringIO(template % (
+            '''
+            <browser:resource
+                name="test"
+                factory="zope.app.component.tests.views.RZMI"
+                layer="
+                  zope.app.publisher.browser.tests.test_directives.ITestLayer"
+                />
+            <browser:resource
+                name="test"
+                factory="zope.app.component.tests.views.R1"
+                />
+            '''
+            )))
+
+        self.assertEqual(
+            component.queryAdapter(request, name='test').__class__, R1)
+        self.assertEqual(
+            component.queryAdapter(
+                TestRequest(skin=ITestSkin), name='test').__class__,
             RZMI)
 
     def testDefaultSkin(self):
         request = TestRequest()
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        XMLConfig('meta.zcml', zope.app.component)()        
         xmlconfig(StringIO(template % (
             '''
-            <browser:layer name="zmi" />
-            <browser:skin name="zmi" layers="zmi" />
-            <browser:defaultSkin name="zmi" />
-            <browser:page name="test"
+            <interface
+                interface="
+                  zope.app.publisher.browser.tests.test_directives.ITestSkin"
+                type="zope.publisher.interfaces.browser.IBrowserSkinType"
+                name="Test Skin"
+                />
+            <browser:defaultSkin name="Test Skin" />
+            <browser:page
+                name="test"
                 class="zope.app.component.tests.views.VZMI"
-                layer="zmi"
+                layer="
+                  zope.app.publisher.browser.tests.test_directives.ITestLayer"
                 for="zope.app.component.tests.views.IC"
                 permission="zope.Public"
                 attribute="index"
@@ -374,24 +389,24 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             )))
 
         # Simulate Zope Publication behavior in beforeTraversal()
-        adapters = zapi.getSiteManager().adapters
+        adapters = component.getSiteManager().adapters
         skin = adapters.lookup((providedBy(request),), IDefaultSkin, '')
         directlyProvides(request, skin)
 
-        v = zapi.queryMultiAdapter((ob, request), name='test')
+        v = component.queryMultiAdapter((ob, request), name='test')
         self.assert_(issubclass(v.__class__, VZMI))
 
     def testSkinPage(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template % (
             '''
-            <browser:layer name="zmi" />
-            <browser:skin name="zmi" layers="zmi" />
             <browser:page name="test"
                 class="zope.app.component.tests.views.VZMI"
-                layer="zmi"
+                layer="
+                  zope.app.publisher.browser.tests.test_directives.ITestLayer"
                 for="zope.app.component.tests.views.IC"
                 permission="zope.Public"
                 attribute="index"
@@ -405,14 +420,14 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             )))
 
-        v = zapi.queryMultiAdapter((ob, request), name='test')
+        v = component.queryMultiAdapter((ob, request), name='test')
         self.assert_(issubclass(v.__class__, V1))
-        zmi = zapi.getUtility(ISkin, 'zmi')
-        v = zapi.queryMultiAdapter((ob, TestRequest(skin=zmi)), name='test')
+        v = component.queryMultiAdapter(
+            (ob, TestRequest(skin=ITestSkin)), name='test')
         self.assert_(issubclass(v.__class__, VZMI))
 
     def testI18nResource(self):
-        self.assertEqual(zapi.queryAdapter(request, name='test'), None)
+        self.assertEqual(component.queryAdapter(request, name='test'), None)
 
         path1 = os.path.join(tests_path, 'testfiles', 'test.pt')
         path2 = os.path.join(tests_path, 'testfiles', 'test2.pt')
@@ -426,9 +441,9 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % (path1, path2)
             )))
 
-        v = zapi.getAdapter(request, name='test')
+        v = component.getAdapter(request, name='test')
         self.assertEqual(
-            zapi.queryAdapter(request, name='test').__class__,
+            component.queryAdapter(request, name='test').__class__,
             I18nFileResource)
         self.assertEqual(v._testData('en'), open(path1, 'rb').read())
         self.assertEqual(v._testData('fr'), open(path2, 'rb').read())
@@ -468,7 +483,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='test')
+        v = component.getMultiAdapter((ob, request), name='test')
         v = ProxyFactory(v)
         self.assertEqual(v.index(), 'V1 here')
         self.assertRaises(Exception, getattr, v, 'action')
@@ -486,7 +501,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='test')
+        v = component.getMultiAdapter((ob, request), name='test')
         v = ProxyFactory(v)
         self.assertEqual(v.action(), 'done')
         self.assertEqual(v.action2(), 'done')
@@ -506,7 +521,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='test')
+        v = component.getMultiAdapter((ob, request), name='test')
         v = ProxyFactory(v)
         page = v.publishTraverse(request, 'index.html')
         self.assertEqual(page(), 'done')
@@ -527,7 +542,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='test')
+        v = component.getMultiAdapter((ob, request), name='test')
         self.assertEqual(v.index(), 'V1 here')
         self.assertEqual(v.action(), 'done')
 
@@ -545,7 +560,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='test')
+        v = component.getMultiAdapter((ob, request), name='test')
         self.assertEqual(v.index(), 'V1 here')
         self.assertEqual(v.action(), 'done')
 
@@ -563,7 +578,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='test')
+        v = component.getMultiAdapter((ob, request), name='test')
         self.assertEqual(v.index(), v)
         self.assert_(IBrowserPublisher.providedBy(v))
 
@@ -584,8 +599,9 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
 
 
     def testPageViews(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
         test3 = os.path.join(tests_path, 'testfiles', 'test3.pt')
 
         xmlconfig(StringIO(template %
@@ -603,16 +619,17 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % test3
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='index.html')
+        v = component.getMultiAdapter((ob, request), name='index.html')
         self.assertEqual(v(), 'V1 here')
-        v = zapi.getMultiAdapter((ob, request), name='action.html')
+        v = component.getMultiAdapter((ob, request), name='action.html')
         self.assertEqual(v(), 'done')
-        v = zapi.getMultiAdapter((ob, request), name='test.html')
+        v = component.getMultiAdapter((ob, request), name='test.html')
         self.assertEqual(str(v()), '<html><body><p>done</p></body></html>\n')
 
     def testNamedViewPageViewsCustomTraversr(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
@@ -629,7 +646,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        view = zapi.getMultiAdapter((ob, request), name='test')
+        view = component.getMultiAdapter((ob, request), name='test')
         view = removeSecurityProxy(view)
         self.assertEqual(view.browserDefault(request)[1], (u'index.html', ))
 
@@ -643,8 +660,9 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
 
 
     def testNamedViewNoPagesForCallable(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
@@ -657,13 +675,14 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        view = zapi.getMultiAdapter((ob, request), name='test')
+        view = component.getMultiAdapter((ob, request), name='test')
         view = removeSecurityProxy(view)
         self.assertEqual(view.browserDefault(request), (view, ()))
 
     def testNamedViewNoPagesForNonCallable(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
@@ -676,13 +695,14 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        view = zapi.getMultiAdapter((ob, request), name='test')
+        view = component.getMultiAdapter((ob, request), name='test')
         view = removeSecurityProxy(view)
         self.assertEqual(getattr(view, 'browserDefault', None), None)
 
     def testNamedViewPageViewsNoDefault(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
         test3 = os.path.join(tests_path, 'testfiles', 'test3.pt')
 
         xmlconfig(StringIO(template %
@@ -701,7 +721,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % test3
             ))
 
-        view = zapi.getMultiAdapter((ob, request), name='test')
+        view = component.getMultiAdapter((ob, request), name='test')
         view = removeSecurityProxy(view)
         self.assertEqual(view.browserDefault(request)[1], (u'index.html', ))
 
@@ -717,8 +737,9 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         self.assertEqual(str(v()), '<html><body><p>done</p></body></html>\n')
 
     def testNamedViewPageViewsWithDefault(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
         test3 = os.path.join(tests_path, 'testfiles', 'test3.pt')
 
         xmlconfig(StringIO(template %
@@ -738,7 +759,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % test3
             ))
 
-        view = zapi.getMultiAdapter((ob, request), name='test')
+        view = component.getMultiAdapter((ob, request), name='test')
         view = removeSecurityProxy(view)
         self.assertEqual(view.browserDefault(request)[1], (u'test.html', ))
 
@@ -771,7 +792,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        view = zapi.getMultiAdapter((ob, request), name='test')
+        view = component.getMultiAdapter((ob, request), name='test')
         view = removeSecurityProxy(view)
         view.publishTraverse(request, 'index.html')
         
@@ -798,7 +819,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        view = zapi.getMultiAdapter((ob, request), name='test')
+        view = component.getMultiAdapter((ob, request), name='test')
         view = removeSecurityProxy(view)
         view.publishTraverse(request, 'index.html')
 
@@ -806,8 +827,9 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         ztapi.provideUtility(IPermission, Permission('p', 'P'), 'p')
 
         request = TestRequest()
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
@@ -827,17 +849,18 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='index.html')
+        v = component.getMultiAdapter((ob, request), name='index.html')
         v = ProxyFactory(v)
         zope.security.management.getInteraction().add(request)
         self.assertRaises(Exception, v)
-        v = zapi.getMultiAdapter((ob, request), name='action.html')
+        v = component.getMultiAdapter((ob, request), name='action.html')
         v = ProxyFactory(v)
         self.assertRaises(Exception, v)
 
     def testProtectedNamedViewPageViews(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
@@ -858,49 +881,48 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        view = zapi.getMultiAdapter((ob, request), name='test')
+        view = component.getMultiAdapter((ob, request), name='test')
         self.assertEqual(view.browserDefault(request)[1], (u'index.html', ))
 
         v = view.publishTraverse(request, 'index.html')
         self.assertEqual(v(), 'V1 here')
 
     def testSkinnedPageView(self):
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
-            <browser:layer name="layer" />
-            <browser:skin name="skinny" layers="layer" />
             <browser:pages
                 for="*"
                 class="zope.app.component.tests.views.V1"
                 permission="zope.Public"
-                >
-             
+                >             
               <browser:page name="index.html" attribute="index" />
             </browser:pages>
+
             <browser:pages
                 for="*"
                 class="zope.app.component.tests.views.V1"
-                layer="layer"
+                layer="
+                  zope.app.publisher.browser.tests.test_directives.ITestLayer"
                 permission="zope.Public"
                 >
-            
               <browser:page name="index.html" attribute="action" />
             </browser:pages>
             '''
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='index.html')
+        v = component.getMultiAdapter((ob, request), name='index.html')
         self.assertEqual(v(), 'V1 here')
-        skinny = zapi.getUtility(ISkin, 'skinny')
-        v = zapi.getMultiAdapter((ob, TestRequest(skin=skinny)),
+        v = component.getMultiAdapter((ob, TestRequest(skin=ITestSkin)),
                                  name='index.html')
         self.assertEqual(v(), 'done')
 
     def testFactory(self):
-        self.assertEqual(zapi.queryAdapter(request, name='index.html'), None)
+        self.assertEqual(
+            component.queryAdapter(request, name='index.html'), None)
 
         xmlconfig(StringIO(template %
             '''
@@ -912,7 +934,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        r = zapi.getAdapter(request, name='index.html')
+        r = component.getAdapter(request, name='index.html')
         self.assertEquals(r.__class__, MyResource)
         r = ProxyFactory(r)
         self.assertEqual(r.__name__, "index.html")
@@ -920,7 +942,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
     def testFile(self):
         path = os.path.join(tests_path, 'testfiles', 'test.pt')
 
-        self.assertEqual(zapi.queryAdapter(request, name='test'), None)
+        self.assertEqual(component.queryAdapter(request, name='test'), None)
 
         xmlconfig(StringIO(template %
             '''
@@ -931,7 +953,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % path
             ))
 
-        r = zapi.getAdapter(request, name='index.html')
+        r = component.getAdapter(request, name='index.html')
         self.assertEquals(r.__class__, FileResource)
         r = ProxyFactory(r)
         self.assertEqual(r.__name__, "index.html")
@@ -949,23 +971,23 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
 
 
     def testSkinResource(self):
-        self.assertEqual(zapi.queryAdapter(request, name='test'), None)
+        self.assertEqual(component.queryAdapter(request, name='test'), None)
 
         path = os.path.join(tests_path, 'testfiles', 'test.pt')
-
         xmlconfig(StringIO(template % (
             '''
-            <browser:layer name="zmi" />
-            <browser:skin name="zmi" layers="zmi" />
-            <browser:resource name="test" file="%s" 
-                  layer="zmi" />
+            <browser:resource
+                name="test"
+                file="%s" 
+                layer="
+                  zope.app.publisher.browser.tests.test_directives.ITestLayer"
+                />
             ''' % path
             )))
 
-        self.assertEqual(zapi.queryAdapter(request, name='test'), None)
+        self.assertEqual(component.queryAdapter(request, name='test'), None)
 
-        zmi = zapi.getUtility(ISkin, 'zmi')
-        r = zapi.getAdapter(TestRequest(skin=zmi), name='test')
+        r = component.getAdapter(TestRequest(skin=ITestSkin), name='test')
         r = removeSecurityProxy(r)
         self.assertEqual(r._testData(), open(path, 'rb').read())
 
@@ -973,7 +995,8 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         path = os.path.join(tests_path, 'testfiles', 'test.pt')
 
         self.assertEqual(
-            zapi.queryMultiAdapter((ob, request), name='index.html'), None)
+            component.queryMultiAdapter((ob, request), name='index.html'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
@@ -985,13 +1008,13 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % path
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='index.html')
+        v = component.getMultiAdapter((ob, request), name='index.html')
         self.assertEqual(v().strip(), '<html><body><p>test</p></body></html>')
 
     def test_page_menu_within_different_layers(self):
         path = os.path.join(tests_path, 'testfiles', 'test.pt')
         self.assertEqual(
-            zapi.queryMultiAdapter((ob, request), name='index.html'),
+            component.queryMultiAdapter((ob, request), name='index.html'),
             None)
 
         xmlconfig(StringIO(template %
@@ -1001,11 +1024,6 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
                 title="Test menu"
                 interface="zope.app.publisher.browser.tests.test_directives.ITestMenu"/>
 
-            <browser:layer 
-                name="test_layer"
-                interface="zope.app.publisher.browser.tests.test_directives.ITestLayer"
-                />
-
             <browser:page
                 name="index.html"
                 permission="zope.Public"
@@ -1023,14 +1041,15 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % (path, path)
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='index.html')
+        v = component.getMultiAdapter((ob, request), name='index.html')
         self.assertEqual(v().strip(), '<html><body><p>test</p></body></html>')
 
     def testtemplateWClass(self):
         path = os.path.join(tests_path, 'testfiles', 'test2.pt')
 
         self.assertEqual(
-            zapi.queryMultiAdapter((ob, request), name='index.html'), None)
+            component.queryMultiAdapter((ob, request), name='index.html'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
@@ -1043,7 +1062,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % path
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='index.html')
+        v = component.getMultiAdapter((ob, request), name='index.html')
         self.assertEqual(v().strip(), '<html><body><p>42</p></body></html>')
 
     def testProtectedtemplate(self):
@@ -1051,8 +1070,9 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
         path = os.path.join(tests_path, 'testfiles', 'test.pt')
 
         request = TestRequest()
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), name='test'),
-                         None)
+        self.assertEqual(
+            component.queryMultiAdapter((ob, request), name='test'),
+            None)
 
         xmlconfig(StringIO(template %
             '''
@@ -1078,12 +1098,12 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             ''' % path
             ))
 
-        v = zapi.getMultiAdapter((ob, request), name='xxx.html')
+        v = component.getMultiAdapter((ob, request), name='xxx.html')
         v = ProxyFactory(v)
         zope.security.management.getInteraction().add(request)
         self.assertRaises(Exception, v)
 
-        v = zapi.getMultiAdapter((ob, request), name='index.html')
+        v = component.getMultiAdapter((ob, request), name='index.html')
         v = ProxyFactory(v)
         self.assertEqual(v().strip(), '<html><body><p>test</p></body></html>')
 
@@ -1123,7 +1143,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
     def testViewThatProvidesAnInterface(self):
         request = TestRequest()
         self.assertEqual(
-            zapi.queryMultiAdapter((ob, request), IV, name='test'), None)
+            component.queryMultiAdapter((ob, request), IV, name='test'), None)
 
         xmlconfig(StringIO(template %
             '''
@@ -1136,7 +1156,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.queryMultiAdapter((ob, request), IV, name='test')
+        v = component.queryMultiAdapter((ob, request), IV, name='test')
         self.assertEqual(v, None)
 
         xmlconfig(StringIO(template %
@@ -1151,12 +1171,12 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.queryMultiAdapter((ob, request), IV, name='test')
+        v = component.queryMultiAdapter((ob, request), IV, name='test')
         self.assert_(isinstance(v, V1))
 
     def testUnnamedViewThatProvidesAnInterface(self):
         request = TestRequest()
-        self.assertEqual(zapi.queryMultiAdapter((ob, request), IV),
+        self.assertEqual(component.queryMultiAdapter((ob, request), IV),
                          None)
 
         xmlconfig(StringIO(template %
@@ -1169,7 +1189,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.queryMultiAdapter((ob, request), IV)
+        v = component.queryMultiAdapter((ob, request), IV)
         self.assertEqual(v, None)
 
         xmlconfig(StringIO(template %
@@ -1183,7 +1203,7 @@ class Test(placelesssetup.PlacelessSetup, unittest.TestCase):
             '''
             ))
 
-        v = zapi.queryMultiAdapter((ob, request), IV)
+        v = component.queryMultiAdapter((ob, request), IV)
 
         self.assert_(isinstance(v, V1))
 
