@@ -13,17 +13,16 @@
 ##############################################################################
 """A WSGI Application wrapper for zope
 
-$Id: __init__.py 69639 2006-08-18 10:19:40Z philikon $
+$Id: __init__.py 79640 2007-09-14 16:43:17Z philikon $
 """
 import os
 import sys
-import ZConfig
 import logging
+import ZConfig
 
 from zope.event import notify
 from zope.interface import implements
 from zope.publisher.publish import publish
-from zope.publisher.interfaces.http import IHeaderOutput
 
 from zope.app.appsetup import appsetup
 from zope.app.publication.httpfactory import HTTPPublicationRequestFactory
@@ -84,14 +83,11 @@ class PMDBWSGIPublisherApplication(WSGIPublisherApplication):
                 pass #zope.security.management.endInteraction()
 
 
-
-def getWSGIApplication(configfile, schemafile=None,
-                       features=(),
-                       requestFactory=HTTPPublicationRequestFactory):
+def config(configfile, schemafile=None, features=()):
     # Load the configuration schema
     if schemafile is None:
         schemafile = os.path.join(
-            os.path.dirname(appsetup.__file__), 'schema.xml')
+            os.path.dirname(appsetup.__file__), 'schema', 'schema.xml')
 
     # Let's support both, an opened file and path
     if isinstance(schemafile, basestring):
@@ -124,14 +120,16 @@ def getWSGIApplication(configfile, schemafile=None,
             "and should NOT be enabled on production servers. Developer mode "
             "can be turned off in etc/zope.conf")
 
-    # Configure the application
+    # Execute the ZCML configuration.
     appsetup.config(options.site_definition, features=features)
 
-    # Connect to and open the database
+    # Connect to and open the database, notify subscribers.
     db = appsetup.multi_database(options.databases)[0][0]
-
-    # Send out an event that the database has been opened
     notify(appsetup.interfaces.DatabaseOpened(db))
 
-    # Create the WSGI application
+    return db
+
+def getWSGIApplication(configfile, schemafile=None, features=(),
+                       requestFactory=HTTPPublicationRequestFactory):
+    db = config(configfile, schemafile, features)
     return WSGIPublisherApplication(db, requestFactory)
