@@ -13,23 +13,29 @@
 ##############################################################################
 """WSGI tests
 
-$Id: tests.py 41673 2006-02-18 20:49:44Z benji_york $
+$Id: tests.py 78771 2007-08-12 12:29:58Z nikhil_n $
 """
 import tempfile
 import unittest
+import re
 
 from zope import component, interface
 from zope.testing import doctest
+from zope.testing import renormalizing
 
-import zope.app.testing.functional
 import zope.publisher.interfaces.browser
-from zope.app.testing import placelesssetup
+from zope.app.testing import placelesssetup, ztapi
 from zope.app.publication.requestpublicationregistry import factoryRegistry
 from zope.app.publication.requestpublicationfactories import BrowserFactory
+from zope.app.wsgi.testing import AppWSGILayer
+from zope.app.security.interfaces import IAuthentication
+from zope.app.security.principalregistry import principalRegistry
+
 
 def setUp(test):
     placelesssetup.setUp(test)
     factoryRegistry.register('GET', '*', 'browser', 0, BrowserFactory())
+    ztapi.provideUtility(IAuthentication, principalRegistry)
 
 
 
@@ -96,14 +102,18 @@ Clean up:
 
 def test_suite():
 
+    checker = renormalizing.RENormalizing([
+        (re.compile(r"&lt;class 'zope.component.interfaces.ComponentLookupError'&gt;"),
+                    r'ComponentLookupError'),
+    ])
     functional_suite = doctest.DocTestSuite()
-    functional_suite.layer = zope.app.testing.functional.Functional
+    functional_suite.layer = AppWSGILayer
 
     return unittest.TestSuite((
         functional_suite,
         doctest.DocFileSuite(
             'README.txt', 'fileresult.txt',
-            setUp=setUp,
+            setUp=setUp, checker=checker,
             tearDown=placelesssetup.tearDown,
             optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS),
         ))
