@@ -13,12 +13,14 @@
 
 See the CachedProperty class.
 
-$Id: property.py 38680 2005-09-29 09:12:36Z jim $
+$Id: property.py 75652 2007-05-09 13:11:30Z zagy $
 """
+
 ncaches = 0l
 
+
 class CachedProperty(object):
-    """Cached Properties
+    """Cached Properties.
     """
 
     def __init__(self, func, *names):
@@ -42,18 +44,18 @@ class CachedProperty(object):
             if key == getattr(inst, key_name, self):
                 # Cache is still good!
                 return value
-            
+
         # We need to compute and cache the value
 
         value = func(inst)
         setattr(inst, key_name, key)
         setattr(inst, value_name, value)
-        
+
         return value
 
 
 class Lazy(object):
-    """Lazy Attributes
+    """Lazy Attributes.
     """
 
     def __init__(self, func, name=None):
@@ -68,9 +70,10 @@ class Lazy(object):
         func, name = self.data
         value = func(inst)
         inst.__dict__[name] = value
-        
+
         return value
 
+
 class readproperty(object):
 
     def __init__(self, func):
@@ -82,4 +85,22 @@ class readproperty(object):
 
         func = self.func
         return func(inst)
-    
+
+
+class cachedIn(object):
+    """Cached property with given cache attribute."""
+
+    def __init__(self, attribute_name):
+        self.attribute_name = attribute_name
+
+    def __call__(self, func):
+
+        def get(instance):
+            try:
+                value = getattr(instance, self.attribute_name)
+            except AttributeError:
+                value = func(instance)
+                setattr(instance, self.attribute_name, value)
+            return value
+
+        return property(get)
