@@ -15,9 +15,9 @@
 
 Note, for a detailed description of the way that conflicting
 configuration actions are resolved, see the detailed example in
-test_includeOverrides in tests/text_xmlconfig.py
+test_includeOverrides in tests/test_xmlconfig.py
 
-$Id: xmlconfig.py 68962 2006-07-04 01:26:28Z tseaver $
+$Id: xmlconfig.py 74015 2007-04-05 09:40:20Z jinty $
 """
 __docformat__ = 'restructuredtext'
 
@@ -169,7 +169,15 @@ class ParserInfo(object):
                 # Remove text before start if it's noy whitespace
                 lines[0] = lines[0][self.column:]
 
-            src = ''.join([u"  "+l for l in lines])
+            try:
+                src = u''.join([u"  "+l for l in lines])
+            except UnicodeDecodeError:
+                # XXX:
+                # I hope so most internation zcml will use UTF-8 as encoding
+                # otherwise this code must be made more clever
+                src = u''.join([u"  "+l.decode('utf-8') for l in lines])
+                # unicode won't be printable, at least on my console
+                src = src.encode('ascii','replace')
 
         return "%s\n%s" % (`self`, src)
 
@@ -235,7 +243,8 @@ class ConfigurationHandler(ContentHandler):
 
         `expression` is a string of the form "verb arguments".
 
-        Currently the two supported verba are 'have' and 'installed'.
+        Currently the supported verbs are 'have', 'not-have',
+        'installed' and 'not-installed'.
 
         The 'have' verb takes one argument: the name of a feature.
 
@@ -245,8 +254,12 @@ class ConfigurationHandler(ContentHandler):
         >>> c = ConfigurationHandler(context, testing=True)
         >>> c.evaluateCondition("have apidoc")
         True
+        >>> c.evaluateCondition("not-have apidoc")
+        False
         >>> c.evaluateCondition("have onlinehelp")
         False
+        >>> c.evaluateCondition("not-have onlinehelp")
+        True
 
         Ill-formed expressions raise an error
 
@@ -275,8 +288,12 @@ class ConfigurationHandler(ContentHandler):
         >>> c = ConfigurationHandler(context, testing=True)
         >>> c.evaluateCondition('installed zope.interface')
         True
+        >>> c.evaluateCondition('not-installed zope.interface')
+        False
         >>> c.evaluateCondition('installed zope.foo')
         False
+        >>> c.evaluateCondition('not-installed zope.foo')
+        True
 
         Ill-formed expressions raise an error
 
@@ -292,22 +309,34 @@ class ConfigurationHandler(ContentHandler):
         """
         arguments = expression.split(None)
         verb = arguments.pop(0)
-        if verb == 'have':
+
+        if verb in ('have', 'not-have'):
             if not arguments:
                 raise ValueError("Feature name missing: %r" % expression)
             if len(arguments) > 1:
                 raise ValueError("Only one feature allowed: %r" % expression)
-            return self.context.hasFeature(arguments[0])
-        elif verb == 'installed':
+
+            if verb == 'have':
+                return self.context.hasFeature(arguments[0])
+            elif verb == 'not-have':
+                return not self.context.hasFeature(arguments[0])
+
+        elif verb in ('installed', 'not-installed'):
             if not arguments:
                 raise ValueError("Package name missing: %r" % expression)
             if len(arguments) > 1:
                 raise ValueError("Only one package allowed: %r" % expression)
+
             try:
                 __import__(arguments[0])
+                installed = True
             except ImportError:
-                return False
-            return True
+                installed = False
+
+            if verb == 'installed':
+                return installed
+            elif verb == 'not-installed':
+                return not installed
         else:
             raise ValueError("Invalid ZCML condition: %r" % expression)
 
@@ -467,6 +496,29 @@ def include(_context, file=None, package=None, files=None):
     elif not file:
         file = 'configure.zcml'
 
+    # BBB 2006/12/19 -- to be removed after 12 months
+    # This is a backward-compatibility support for old site.conf
+
+    if package and (package.__name__ == 'zope.app'):
+        try:
+            import zope.app.zcmlfiles
+        except ImportError:
+            pass # maybe this is an old zope without zope.app.zcmlfiles
+        else:
+            dirpath, filename = os.path.split(file)
+            # be careful, because zope.app is a namespace package
+            # we can't assume that zcmlfiles is a subdirectory of the 
+            # zope.app package
+            dirpath = os.path.dirname(zope.app.zcmlfiles.__file__)
+            file = os.path.join(dirpath, filename)
+            import warnings
+            warnings.warn('In configuration file: %s '
+                          'replace: <include package="zope.app" /> '
+                          'with: <include package="zope.app.zcmlfiles" /> '
+                          'This will go away in Zope 3.6.' % os.path.abspath(file),
+                          DeprecationWarning,
+                          2)
+
     # This is a tad tricky. We want to behave as a grouping directive.
 
     context = config.GroupingContextDecorator(_context)
