@@ -13,13 +13,12 @@
 ##############################################################################
 """Field index
 
-$Id: index.py 71194 2006-11-20 09:43:58Z adamg $
+$Id: index.py 74783 2007-04-26 14:52:38Z fdrake $
 """
 import persistent
 
-from BTrees.IOBTree import IOBTree
-from BTrees.OOBTree import OOBTree
-from BTrees.IFBTree import IFTreeSet, IFSet, multiunion
+import BTrees
+
 from BTrees.Length import Length
 
 import zope.interface
@@ -34,15 +33,19 @@ class FieldIndex(persistent.Persistent):
         interfaces.IIndexSearch,
         )
 
-    def __init__(self):
+    family = BTrees.family32
+
+    def __init__(self, family=None):
+        if family is not None:
+            self.family = family
         self.clear()
 
     def clear(self):
         """Initialize forward and reverse mappings."""
         # The forward index maps indexed values to a sequence of docids
-        self._fwd_index = OOBTree()
+        self._fwd_index = self.family.OO.BTree()
         # The reverse index maps a docid to its index value
-        self._rev_index = IOBTree()
+        self._rev_index = self.family.IO.BTree()
         self._num_docs = Length(0)
 
     def documentCount(self):
@@ -57,13 +60,16 @@ class FieldIndex(persistent.Persistent):
         """See interface IInjection"""
         rev_index = self._rev_index
         if docid in rev_index:
+            if docid in self._fwd_index.get(value, ()):
+                # no need to index the doc, its already up to date
+                return
             # unindex doc if present
             self.unindex_doc(docid)
 
         # Insert into forward index.
         set = self._fwd_index.get(value)
         if set is None:
-            set = IFTreeSet()
+            set = self.family.IF.TreeSet()
             self._fwd_index[value] = set
         set.insert(docid)
 
@@ -99,4 +105,5 @@ class FieldIndex(persistent.Persistent):
     def apply(self, query):
         if len(query) != 2 or not isinstance(query, tuple):
             raise TypeError("two-length tuple expected", query)
-        return multiunion(self._fwd_index.values(*query))        
+        return self.family.IF.multiunion(
+            self._fwd_index.values(*query))
