@@ -13,7 +13,7 @@
 ##############################################################################
 """HTTP Publisher
 
-$Id: http.py 96308 2009-02-09 12:44:38Z hannosch $
+$Id: http.py 89074 2008-07-31 07:29:29Z andreasjung $
 """
 import re, time, random
 from cStringIO import StringIO
@@ -34,6 +34,7 @@ from zope.publisher.interfaces.http import IHTTPRequest
 from zope.publisher.interfaces.http import IHTTPApplicationRequest
 from zope.publisher.interfaces.http import IHTTPPublisher
 from zope.publisher.interfaces.http import IHTTPVirtualHostChangedEvent
+from zope.publisher.interfaces.http import IResult
 
 from zope.publisher.interfaces import Redirect
 from zope.publisher.interfaces.http import IHTTPResponse
@@ -52,6 +53,8 @@ from zope.publisher.base import RequestDataGetter
 # Default Encoding
 ENCODING = 'UTF-8'
 
+eventlog = logging.getLogger('eventlog')
+
 class CookieMapper(RequestDataMapper):
     _mapname = '_cookies'
 
@@ -76,9 +79,9 @@ def sane_environment(env):
 
 class HTTPVirtualHostChangedEvent(object):
     interface.implements(IHTTPVirtualHostChangedEvent)
-    
+
     request = None
-    
+
     def __init__(self, request):
         self.request = request
 
@@ -87,7 +90,7 @@ status_reasons = {
 100: 'Continue',
 101: 'Switching Protocols',
 102: 'Processing',
-200: 'OK',
+200: 'Ok',
 201: 'Created',
 202: 'Accepted',
 203: 'Non-Authoritative Information',
@@ -188,8 +191,12 @@ class HTTPInputStream(object):
 
     def __init__(self, stream, environment):
         self.stream = stream
-        size = environment.get('HTTP_CONTENT_LENGTH')
-        if size is None or int(size) < 65536:
+        size = environment.get('CONTENT_LENGTH')
+        # There can be no size in the environment (None) or the size
+        # can be an empty string, in which case we treat it as absent.
+        if not size:
+            size = environment.get('HTTP_CONTENT_LENGTH')
+        if not size or int(size) < 65536:
             self.cacheStream = StringIO()
         else:
             self.cacheStream = TemporaryFile()
@@ -288,21 +295,7 @@ class HTTPRequest(BaseRequest):
 
     retry_max_count = 3    # How many times we're willing to retry
 
-    def __init__(self, body_instream, environ, response=None, outstream=None):
-        # BBB: This is backward-compatibility support for the deprecated
-        # output stream.
-        try:
-            environ.get
-        except AttributeError:
-            import warnings
-            warnings.warn("Can't pass output streams to requests anymore. "
-                          "This will go away in Zope 3.4.",
-                          DeprecationWarning,
-                          # HTTPRequest.__init__() mostly called through
-                          # BrowserRequest.__init__() so we need to shift
-                          # to another one step backward.
-                          3)
-            environ, response = response, outstream
+    def __init__(self, body_instream, environ, response=None):
 
         super(HTTPRequest, self).__init__(
             HTTPInputStream(body_instream, environ), environ, response)
@@ -408,8 +401,7 @@ class HTTPRequest(BaseRequest):
         try:
             c = SimpleCookie(text)
         except CookieError, e:
-            log = logging.getLogger('eventlog')
-            log.warn(e)
+            eventlog.warn(e)
             return result
 
         for k,v in c.items():
@@ -594,14 +586,12 @@ class HTTPRequest(BaseRequest):
 
     def get(self, key, default=None):
         'See Interface.Common.Mapping.IReadMapping'
+        marker = object()
+        result = self._cookies.get(key, marker)
+        if result is not marker:
+            return result
 
-        result = self._cookies.get(key, self)
-        if result is not self: return result
-
-        result = self._environ.get(key, self)
-        if result is not self: return result
-
-        return default
+        return super(HTTPRequest, self).get(key, default)
 
     def keys(self):
         'See Interface.Common.Mapping.IEnumerableMapping'
@@ -611,28 +601,6 @@ class HTTPRequest(BaseRequest):
         return d.keys()
 
 
-class IResult(interface.Interface):
-    """HTTP result.
-
-    WARNING! This is a PRIVATE interface and VERY LIKELY TO CHANGE!
-
-    The result provides the result in a form suitable for delivery to HTTP
-    clients.
-
-    IMPORTANT: The result object may be held indefinitely by a server and may
-    be accessed by arbitrary threads. For that reason the result should not
-    hold on to any application resources and should be prepared to be invoked
-    from any thread.
-    """
-
-    headers = interface.Attribute(
-        'A sequence of tuples of result headers, such as '
-        '"Content-Type" and "Content-Length", etc.')
-
-    body = interface.Attribute(
-        'An iterable that provides the body data of the response.')
-
-
 
 class HTTPResponse(BaseResponse):
     interface.implements(IHTTPResponse, IHTTPApplicationResponse)
@@ -651,29 +619,7 @@ class HTTPResponse(BaseResponse):
         )
 
 
-    def __init__(self, header_output=None, http_transaction=None):
-        # BBB: Both, header_output and http_transaction have been deprecated.
-        if header_output is not None:
-            import warnings
-            warnings.warn(
-                "The header output API is completely deprecated. It's "
-                "intentions were not clear and it duplicated APIs in the "
-                "response, which you should use instead. "
-                "This will go away in Zope 3.4.",
-                DeprecationWarning, 2)
-
-        if http_transaction is not None:
-            import warnings
-            warnings.warn(
-                "Storing the HTTP transaction here was a *huge* hack to "
-                "support transporting the authenticated user string "
-                "to the server. You should never rely on this variable "
-                "anyways. "
-                "This will go away in Zope 3.4.",
-                DeprecationWarning, 2)
-
-        self._header_output = header_output
-
+    def __init__(self):
         super(HTTPResponse, self).__init__()
         self.reset()
 
@@ -696,19 +642,12 @@ class HTTPResponse(BaseResponse):
         else:
             if type(status) in StringTypes:
                 status = status.lower()
-            if status in status_codes:
-                status = status_codes[status]
-            else:
-                status = 500
+            status = status_codes.get(status, 500)
         self._status = status
 
         if reason is None:
-            if status == 200:
-                reason = 'Ok'
-            elif status in status_reasons:
-                reason = status_reasons[status]
-            else:
-                reason = 'Unknown'
+            reason = status_reasons.get(status, "Unknown")
+
         self._reason = reason
         self._status_set = True
 
@@ -812,43 +751,38 @@ class HTTPResponse(BaseResponse):
 
 
     def setResult(self, result):
+        'See IHTTPResponse'
         if IResult.providedBy(result):
             r = result
         else:
             r = component.queryMultiAdapter((result, self._request), IResult)
             if r is None:
                 if isinstance(result, basestring):
-                    body, headers = self._implicitResult(result)
-                    r = DirectResult((body,), headers)
+                    r = result
                 elif result is None:
-                    body, headers = self._implicitResult('')
-                    r = DirectResult((body,), headers)
+                    r = ''
                 else:
                     raise TypeError(
-                        'The result should be adaptable to IResult.')
+                        'The result should be None, a string, or adaptable to '
+                        'IResult.')
+            if isinstance(r, basestring):
+                r, headers = self._implicitResult(r)
+                self._headers.update(dict((k, [v]) for (k, v) in headers))
+                r = (r,) # chunking should be much larger than per character
 
         self._result = r
-        self._headers.update(dict([(k, [v]) for (k, v) in r.headers]))
         if not self._status_set:
             self.setStatus(200)
 
 
     def consumeBody(self):
         'See IHTTPResponse'
-        return ''.join(self._result.body)
+        return ''.join(self._result)
 
 
     def consumeBodyIter(self):
         'See IHTTPResponse'
-        return self._result.body
-
-
-    # BBB: Backward-compatibility for old body API
-    _body = property(consumeBody)
-    _body = deprecation.deprecated(
-        _body,
-        '`_body` has been deprecated in favor of `consumeBody()`. '
-        'This will go away in Zope 3.4.')
+        return self._result
 
 
     def _implicitResult(self, body):
@@ -888,7 +822,7 @@ class HTTPResponse(BaseResponse):
         Calls self.setBody() with an error response.
         """
         t, v = exc_info[:2]
-        if isinstance(t, ClassType):
+        if isinstance(t, (ClassType, type)):    
             if issubclass(t, Redirect):
                 self.redirect(v.getLocation())
                 return
@@ -901,14 +835,13 @@ class HTTPResponse(BaseResponse):
         self.setStatus(tname)
 
         body = self._html(title, "A server error occurred." )
+        self.setHeader("Content-Type", "text/html")
         self.setResult(body)
 
-
     def internalError(self):
         'See IPublisherResponse'
         self.setStatus(500, u"The engines can't take any more, Jim!")
 
-
     def _html(self, title, content):
         t = escape(title)
         return (
@@ -919,14 +852,12 @@ class HTTPResponse(BaseResponse):
             (t, t, content)
             )
 
-
     def retry(self):
         """
         Returns a response object to be used in a retry attempt
         """
         return self.__class__()
 
-
     def redirect(self, location, status=None):
         """Causes a redirection without raising an error"""
         if status is None:
@@ -946,8 +877,7 @@ class HTTPResponse(BaseResponse):
         try:
             c = SimpleCookie()
         except CookieError, e:
-            log = logging.getLogger('eventlog')
-            log.warn(e)
+            eventlog.warn(e)
             return []
         for name, attrs in self._cookies.items():
             name = str(name)
@@ -998,7 +928,10 @@ class HTTPCharsets(object):
             charset = charset.strip().lower()
             if charset:
                 if ';' in charset:
-                    charset, quality = charset.split(';')
+                    try:
+                        charset, quality = charset.split(';')
+                    except ValueError:
+                        continue
                     if not quality.startswith('q='):
                         # not a quality parameter
                         quality = 1.0
@@ -1055,22 +988,14 @@ def getCharsetUsingRequest(request):
 class DirectResult(object):
     """A generic result object.
 
-    The result's body can be any iteratable. It is the responsibility of the
+    The result's body can be any iterable. It is the responsibility of the
     application to specify all headers related to the content, such as the
     content type and length.
     """
     interface.implements(IResult)
 
-    def __init__(self, body, headers=()):
+    def __init__(self, body):
         self.body = body
-        self.headers = headers
-
-
-def StrResult(body, headers=()):
-    """A simple string result that represents any type of data.
-
-    It is the responsibility of the application to specify all the headers,
-    including content type and length.
-    """
-    return DirectResult((body,), headers)
 
+    def __iter__(self):
+        return iter(self.body)
