@@ -13,11 +13,16 @@
 ##############################################################################
 """Browser Request Tests
 
-$Id: test_browserrequest.py 29942 2005-04-11 13:26:38Z hdima $
+$Id: test_browserrequest.py 78536 2007-08-02 00:36:05Z philikon $
 """
 import sys
 import unittest
+from StringIO import StringIO
 
+from zope.interface import implements, directlyProvides, Interface
+from zope.interface.verify import verifyObject
+
+from zope.publisher.publish import publish as publish_
 from zope.publisher.http import HTTPCharsets
 from zope.publisher.browser import BrowserRequest
 from zope.publisher.interfaces import NotFound
@@ -25,14 +30,25 @@ from zope.publisher.interfaces import NotFound
 from zope.publisher.base import DefaultPublication
 from zope.publisher.interfaces.browser import IBrowserApplicationRequest
 from zope.publisher.interfaces.browser import IBrowserRequest
-from zope.interface.verify import verifyObject
-
-from StringIO import StringIO
+from zope.publisher.interfaces.browser import IBrowserPublication
 
 from zope.publisher.tests.test_http import HTTPTests
+from zope.publisher.tests.publication import TestPublication
 
-from zope.publisher.publish import publish as publish_
+from zope.publisher.tests.basetestipublicationrequest \
+     import BaseTestIPublicationRequest
+from zope.publisher.tests.basetestipublisherrequest \
+     import BaseTestIPublisherRequest
+from zope.publisher.tests.basetestiapplicationrequest \
+     import BaseTestIApplicationRequest
 
+LARGE_FILE_BODY = """-----------------------------1
+Content-Disposition: form-data; name="upload"; filename="test"
+Content-Type: text/plain
+
+Here comes some text! %s
+-----------------------------1--
+""" % ('test' * 1000)
 
 def publish(request):
     publish_(request, handle_errors=0)
@@ -79,12 +95,12 @@ class BrowserTests(HTTPTests):
         class Item(object):
             """Required docstring for the publisher."""
             def __call__(self, a, b):
-                return "%s, %s" % (`a`, `b`)
+                return u"%s, %s" % (`a`, `b`)
 
         class Item3(object):
             """Required docstring for the publisher."""
             def __call__(self, *args):
-                return "..."
+                return u"..."
 
         class View(object):
             """Required docstring for the publisher."""
@@ -93,7 +109,7 @@ class BrowserTests(HTTPTests):
 
             def index(self, a, b):
                 """Required docstring for the publisher."""
-                return "%s, %s" % (`a`, `b`)
+                return u"%s, %s" % (`a`, `b`)
 
         class Item2(object):
             """Required docstring for the publisher."""
@@ -109,17 +125,15 @@ class BrowserTests(HTTPTests):
         self.app.folder.item2 = Item2()
         self.app.folder.item3 = Item3()
 
-    def _createRequest(self, extra_env={}, body="", outstream=None):
+    def _createRequest(self, extra_env={}, body=""):
         env = self._testEnv.copy()
         env.update(extra_env)
         if len(body):
             env['CONTENT_LENGTH'] = str(len(body))
 
         publication = Publication(self.app)
-        if outstream is None:
-            outstream = StringIO()
         instream = StringIO(body)
-        request = TestBrowserRequest(instream, outstream, env)
+        request = TestBrowserRequest(instream, env)
         request.setPublication(publication)
         return request
 
@@ -130,18 +144,11 @@ class BrowserTests(HTTPTests):
             "Status: 200 Ok\r\n"
             "Content-Length: 7\r\n"
             "Content-Type: text/plain;charset=utf-8\r\n"
+            "X-Content-Type-Warning: guessed from content\r\n"
             "X-Powered-By: Zope (www.zope.org), Python (www.python.org)\r\n"
             "\r\n"
             "u'5', 6")
 
-    def testIPresentationRequest(self):
-        # test the IView request
-        r = self._createRequest()
-
-        self.assertEqual(r.getPresentationSkin(), '')
-        r.setPresentationSkin('morefoo')
-        self.assertEqual(r.getPresentationSkin(), 'morefoo')
-
     def testNoDefault(self):
         request = self._createRequest()
         response = request.response
@@ -164,6 +171,38 @@ class BrowserTests(HTTPTests):
         self.assertEqual(response.getBase(),
                          'http://foobar.com/folder/item2/view/index')
 
+    def testNoneFieldNamePost(self):
+
+        """Produce a Fieldstorage with a name wich is None, this
+        should be catched"""
+        
+        extra = {'REQUEST_METHOD':'POST',
+                 'PATH_INFO': u'/',
+                 'CONTENT_TYPE': 'multipart/form-data;\
+                 boundary=---------------------------1'}
+
+        body = """-----------------------------1
+        Content-Disposition: form-data; name="field.contentType"
+        ...
+        application/octet-stream
+        -----------------------------1--
+        """
+        request  = self._createRequest(extra,body=body)
+        request.processInputs()
+
+    def testFileUploadPost(self):
+        """Produce a Fieldstorage with a file handle that exposes
+        its filename."""
+
+        extra = {'REQUEST_METHOD':'POST',
+                 'PATH_INFO': u'/',
+                 'CONTENT_TYPE': 'multipart/form-data;\
+                 boundary=---------------------------1'}
+        
+        request  = self._createRequest(extra, body=LARGE_FILE_BODY)
+        request.processInputs()
+        self.assert_(request.form['upload'].name)
+
     def testDefault2(self):
         extra = {'PATH_INFO': '/folder/item2/view'}
         request = self._createRequest(extra)
@@ -246,8 +285,9 @@ class BrowserTests(HTTPTests):
         self.assertEqual(request.form[u'b'], u'1')
         self.assertEqual(request.form[u'a'].keys(), [u'x'])
         self.assertEqual(request.form[u'a'][u'x'], (u'5',u'6'))
-        self.assertEqual(str(request.form[u'a']), "x: (u'5', u'6')")
-        self.assertEqual(repr(request.form[u'a']), "x: (u'5', u'6')")
+        self.assertEqual(request.form[u'a'].x, (u'5',u'6'))
+        self.assertEqual(str(request.form[u'a']), "{x: (u'5', u'6')}")
+        self.assertEqual(repr(request.form[u'a']), "{x: (u'5', u'6')}")
 
     def testFormRecordsTypes(self):
         extra = {'QUERY_STRING':'a.x:records=5&a.x:records=6&b=1'}
@@ -259,9 +299,34 @@ class BrowserTests(HTTPTests):
         self.assertEqual(request.form[u'b'], u'1')
         self.assertEqual(len(request.form[u'a']), 2)
         self.assertEqual(request.form[u'a'][0][u'x'], u'5')
+        self.assertEqual(request.form[u'a'][0].x, u'5')
         self.assertEqual(request.form[u'a'][1][u'x'], u'6')
-        self.assertEqual(str(request.form[u'a']), "[x: u'5', x: u'6']")
-        self.assertEqual(repr(request.form[u'a']), "[x: u'5', x: u'6']")
+        self.assertEqual(request.form[u'a'][1].x, u'6')
+        self.assertEqual(str(request.form[u'a']), "[{x: u'5'}, {x: u'6'}]")
+        self.assertEqual(repr(request.form[u'a']), "[{x: u'5'}, {x: u'6'}]")
+
+    def testFormMultipleRecordsTypes(self):
+        extra = {'QUERY_STRING':'a.x:records:int=5&a.y:records:int=51'
+            '&a.x:records:int=6&a.y:records:int=61&b=1'}
+        request = self._createRequest(extra)
+        publish(request)
+        keys = request.form.keys()
+        keys.sort()
+        self.assertEqual(keys, [u'a',u'b'])
+        self.assertEqual(request.form[u'b'], u'1')
+        self.assertEqual(len(request.form[u'a']), 2)
+        self.assertEqual(request.form[u'a'][0][u'x'], 5)
+        self.assertEqual(request.form[u'a'][0].x, 5)
+        self.assertEqual(request.form[u'a'][0][u'y'], 51)
+        self.assertEqual(request.form[u'a'][0].y, 51)
+        self.assertEqual(request.form[u'a'][1][u'x'], 6)
+        self.assertEqual(request.form[u'a'][1].x, 6)
+        self.assertEqual(request.form[u'a'][1][u'y'], 61)
+        self.assertEqual(request.form[u'a'][1].y, 61)
+        self.assertEqual(str(request.form[u'a']),
+            "[{x: 5, y: 51}, {x: 6, y: 61}]")
+        self.assertEqual(repr(request.form[u'a']),
+            "[{x: 5, y: 51}, {x: 6, y: 61}]")
 
     def testFormListRecordTypes(self):
         extra = {'QUERY_STRING':'a.x:list:record=5&a.x:list:record=6&b=1'}
@@ -273,8 +338,9 @@ class BrowserTests(HTTPTests):
         self.assertEqual(request.form[u'b'], u'1')
         self.assertEqual(request.form[u'a'].keys(), [u'x'])
         self.assertEqual(request.form[u'a'][u'x'], [u'5',u'6'])
-        self.assertEqual(str(request.form[u'a']), "x: [u'5', u'6']")
-        self.assertEqual(repr(request.form[u'a']), "x: [u'5', u'6']")
+        self.assertEqual(request.form[u'a'].x, [u'5',u'6'])
+        self.assertEqual(str(request.form[u'a']), "{x: [u'5', u'6']}")
+        self.assertEqual(repr(request.form[u'a']), "{x: [u'5', u'6']}")
 
     def testFormListTypes2(self):
         extra = {'QUERY_STRING':'a=5&a=6&b=1'}
@@ -404,9 +470,76 @@ class BrowserTests(HTTPTests):
         finally:
             sys.argv = argv
 
+    def testIssue559(self):
+        extra = {'QUERY_STRING': 'HTTP_REFERER=peter',
+                 'HTTP_REFERER':'http://localhost/',
+                 'PATH_INFO': '/folder/item3/'}
+        request = self._createRequest(extra)
+        publish(request)
+        self.assertEqual(request.headers.get('HTTP_REFERER'), 'http://localhost/')
+        self.assertEqual(request.form, {u'HTTP_REFERER': u'peter'})
+
+
+class TestBrowserPublication(TestPublication):
+    implements(IBrowserPublication)
+
+    def getDefaultTraversal(self, request, ob):
+        return ob, ()
+
+class APITests(BaseTestIPublicationRequest,
+               BaseTestIApplicationRequest,
+               BaseTestIPublisherRequest,
+               unittest.TestCase):
+
+    def _Test__new(self, environ=None, **kw):
+        if environ is None:
+            environ = kw
+        return BrowserRequest(StringIO(''), environ)
+
+    def test_IApplicationRequest_bodyStream(self):
+        request = BrowserRequest(StringIO('spam'), {})
+        self.assertEqual(request.bodyStream.read(), 'spam')
+
+    # Needed by BaseTestIEnumerableMapping tests:
+    def _IEnumerableMapping__stateDict(self):
+        return {'id': 'ZopeOrg', 'title': 'Zope Community Web Site',
+                'greet': 'Welcome to the Zope Community Web site'}
+
+    def _IEnumerableMapping__sample(self):
+        return self._Test__new(**(self._IEnumerableMapping__stateDict()))
+
+    def _IEnumerableMapping__absentKeys(self):
+        return 'foo', 'bar'
+
+    def test_IPublicationRequest_getPositionalArguments(self):
+        self.assertEqual(self._Test__new().getPositionalArguments(), ())
+
+    def test_IPublisherRequest_retry(self):
+        self.assertEqual(self._Test__new().supportsRetry(), True)
+
+    def test_IPublisherRequest_processInputs(self):
+        self._Test__new().processInputs()
+
+    def test_IPublisherRequest_traverse(self):
+        request = self._Test__new()
+        request.setPublication(TestBrowserPublication())
+        app = request.publication.getApplication(request)
+
+        request.setTraversalStack([])
+        self.assertEqual(request.traverse(app).name, '')
+        self.assertEqual(request._last_obj_traversed, app)
+        request.setTraversalStack(['ZopeCorp'])
+        self.assertEqual(request.traverse(app).name, 'ZopeCorp')
+        self.assertEqual(request._last_obj_traversed, app.ZopeCorp)
+        request.setTraversalStack(['Engineering', 'ZopeCorp'])
+        self.assertEqual(request.traverse(app).name, 'Engineering')
+        self.assertEqual(request._last_obj_traversed, app.ZopeCorp.Engineering)
+
 def test_suite():
-    loader = unittest.TestLoader()
-    return loader.loadTestsFromTestCase(BrowserTests)
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.makeSuite(BrowserTests))
+    suite.addTest(unittest.makeSuite(APITests))
+    return suite
 
 
 if __name__ == '__main__':
