@@ -14,27 +14,38 @@
 ##############################################################################
 """HTTP Publisher Tests
 
-$Id: test_http.py 69340 2006-08-02 14:16:04Z mj $
+$Id: test_http.py 82179 2007-12-07 15:03:15Z ctheune $
 """
+import sys
+import tempfile
 import unittest
+from StringIO import StringIO
+from Cookie import CookieError
 
 import zope.event
+import zope.testing.cleanup
+from zope.testing import doctest
+from zope.i18n.interfaces.locales import ILocale
+from zope.interface.verify import verifyObject
+
 from zope.interface import implements
 from zope.publisher.interfaces.logginginfo import ILoggingInfo
 from zope.publisher.http import HTTPRequest, HTTPResponse
-from zope.publisher.http import HTTPInputStream, StrResult
+from zope.publisher.http import HTTPInputStream, DirectResult
 from zope.publisher.publish import publish
 from zope.publisher.base import DefaultPublication
 from zope.publisher.interfaces.http import IHTTPRequest, IHTTPResponse
 from zope.publisher.interfaces.http import IHTTPApplicationResponse
 from zope.publisher.interfaces import IResponse
+from zope.publisher.tests.publication import TestPublication
 
-from zope.i18n.interfaces.locales import ILocale
+from zope.publisher.tests.basetestipublicationrequest \
+     import BaseTestIPublicationRequest
+from zope.publisher.tests.basetestipublisherrequest \
+     import BaseTestIPublisherRequest
+from zope.publisher.tests.basetestiapplicationrequest \
+     import BaseTestIApplicationRequest
 
-from zope.interface.verify import verifyObject
-
-from StringIO import StringIO
-from Cookie import CookieError
 
 
 class UserStub(object):
@@ -55,54 +66,85 @@ line 1
 line 2
 line 3'''
 
+# tempfiles have different types on different platforms, therefore use
+# this "canonical" way of finding out the type.
+TempFileType = tempfile.TemporaryFile().__class__
 
 class HTTPInputStreamTests(unittest.TestCase):
 
-    def setUp(self):
-        self.stream = HTTPInputStream(StringIO(data), {})
-
-    def getCacheStreamValue(self):
-        self.stream.cacheStream.seek(0)
-        result = self.stream.cacheStream.read()
+    def getCacheStreamValue(self, stream):
+        stream.cacheStream.seek(0)
+        result = stream.cacheStream.read()
         # We just did a read on a file opened for update.  If the next
         # operation on that file is a write, behavior is 100% undefined,
         # and it in fact frequently (but not always) blows up on Windows.
         # Behavior is 100% defined instead if we explictly seek.  Since
         # we expect to be at EOF now, explicitly seek to the end.
-        self.stream.cacheStream.seek(0, 2)
+        stream.cacheStream.seek(0, 2)
         return result
 
     def testRead(self):
+        stream = HTTPInputStream(StringIO(data), {})
         output = ''
-        self.assertEqual(output, self.getCacheStreamValue())
-        output += self.stream.read(5)
-        self.assertEqual(output, self.getCacheStreamValue())
-        output += self.stream.read()
-        self.assertEqual(output, self.getCacheStreamValue())
-        self.assertEqual(data, self.getCacheStreamValue())
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        output += stream.read(5)
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        output += stream.read()
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        self.assertEqual(data, self.getCacheStreamValue(stream))
 
     def testReadLine(self):
-        output = self.stream.readline()
-        self.assertEqual(output, self.getCacheStreamValue())
-        output += self.stream.readline()
-        self.assertEqual(output, self.getCacheStreamValue())
-        output += self.stream.readline()
-        self.assertEqual(output, self.getCacheStreamValue())
-        output += self.stream.readline()
-        self.assertEqual(output, self.getCacheStreamValue())
-        self.assertEqual(data, self.getCacheStreamValue())
+        stream = HTTPInputStream(StringIO(data), {})
+        output = stream.readline()
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        output += stream.readline()
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        output += stream.readline()
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        output += stream.readline()
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        self.assertEqual(data, self.getCacheStreamValue(stream))
 
     def testReadLines(self):
-        output = ''.join(self.stream.readlines(4))
-        self.assertEqual(output, self.getCacheStreamValue())
-        output += ''.join(self.stream.readlines())
-        self.assertEqual(output, self.getCacheStreamValue())
-        self.assertEqual(data, self.getCacheStreamValue())
+        stream = HTTPInputStream(StringIO(data), {})
+        output = ''.join(stream.readlines(4))
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        output += ''.join(stream.readlines())
+        self.assertEqual(output, self.getCacheStreamValue(stream))
+        self.assertEqual(data, self.getCacheStreamValue(stream))
 
-    def testGetChacheStream(self):
-        self.stream.read(5)
-        self.assertEqual(data, self.stream.getCacheStream().read())
+    def testGetCacheStream(self):
+        stream = HTTPInputStream(StringIO(data), {})
+        stream.read(5)
+        self.assertEqual(data, stream.getCacheStream().read())
 
+    def testCachingWithContentLength(self):
+        # The HTTPInputStream implementation will cache the input
+        # stream in a temporary file (instead of in memory) when the
+        # reported content length is over a certain number (100000 is
+        # definitely over that).
+
+        # HTTPInputStream understands both CONTENT_LENGTH...
+        stream = HTTPInputStream(StringIO(data), {'CONTENT_LENGTH': '100000'})
+        self.assert_(isinstance(stream.getCacheStream(), TempFileType))
+
+        # ... and HTTP_CONTENT_LENGTH.
+        stream = HTTPInputStream(StringIO(data), {'HTTP_CONTENT_LENGTH':
+                                                  '100000'})
+        self.assert_(isinstance(stream.getCacheStream(), TempFileType))
+
+        # If CONTENT_LENGTH is absent or empty, it takes the value
+        # given in HTTP_CONTENT_LENGTH:
+        stream = HTTPInputStream(StringIO(data),
+                                 {'CONTENT_LENGTH': '',
+                                  'HTTP_CONTENT_LENGTH': '100000'})
+        self.assert_(isinstance(stream.getCacheStream(), TempFileType))
+
+        # In fact, HTTPInputStream can be instantiated with both an
+        # empty CONTENT_LENGTH and an empty HTTP_CONTENT_LENGTH:
+        stream = HTTPInputStream(StringIO(data),
+                                 {'CONTENT_LENGTH': '',
+                                  'HTTP_CONTENT_LENGTH': ''})
 
 class HTTPTests(unittest.TestCase):
 
@@ -482,6 +524,11 @@ class HTTPTests(unittest.TestCase):
                           'some output',
                           )
 
+    def test_PathTrailingWhitespace(self):
+        request = self._createRequest({'PATH_INFO': '/test '})
+        self.assertEqual(['test '], request.getTraversalStack())
+
+
 class ConcreteHTTPTests(HTTPTests):
     """Tests that we don't have to worry about subclasses inheriting and
     breaking.
@@ -539,7 +586,7 @@ class TestHTTPResponse(unittest.TestCase):
 
         # Output the data
         data = 'a'*10
-        response.setResult(StrResult(data))
+        response.setResult(DirectResult(data))
 
         headers, body = self._parseResult(response)
         # Check that the data have been written, and that the header
@@ -606,19 +653,19 @@ class TestHTTPResponse(unittest.TestCase):
         c = self._getCookieFromResponse([
                 ('foo', 'bar', {}),
                 ])
-        self.failUnless('foo=bar;' in c, 'foo=bar; not in %r' % c)
+        self.failUnless('foo=bar;' in c or 'foo=bar' in c, 'foo=bar; not in %r' % c)
 
         c = self._getCookieFromResponse([
                 ('foo', 'bar', {}),
                 ('alpha', 'beta', {}),
                 ])
-        self.failUnless('foo=bar;' in c)
-        self.failUnless('alpha=beta;' in c)
+        self.failUnless('foo=bar;' in c or 'foo=bar' in c)
+        self.failUnless('alpha=beta;' in c or 'alpha=beta' in c)
 
         c = self._getCookieFromResponse([
                 ('sign', u'\N{BIOHAZARD SIGN}', {}),
                 ])
-        self.failUnless(r'sign="\342\230\243";' in c)
+        self.failUnless((r'sign="\342\230\243";' in c) or (r'sign="\342\230\243"' in c))
 
         self.assertRaises(
                 CookieError,
@@ -636,24 +683,102 @@ class TestHTTPResponse(unittest.TestCase):
                     'seCure': True,
                     }),
                 ])[0]
-        self.failUnless('foo=bar;' in c)
+        self.failUnless('foo=bar;' in c or 'foo=bar' in c)
         self.failUnless('expires=Sat, 12 Jul 2014 23:26:28 GMT;' in c, repr(c))
         self.failUnless('Domain=example.com;' in c)
         self.failUnless('Path=/froboz;' in c)
         self.failUnless('Max-Age=3600;' in c)
         self.failUnless('Comment=blah%3B%E2%98%A3?;' in c, repr(c))
-        self.failUnless('secure;' in c)
+        self.failUnless('secure;' in c or 'secure' in c)
 
         c = self._getCookieFromResponse([('foo', 'bar', {'secure': False})])[0]
-        self.failUnless('foo=bar;' in c)
+        self.failUnless('foo=bar;' in c or 'foo=bar' in c)
         self.failIf('secure' in c)
 
+    def test_handleException(self):
+        response = HTTPResponse()
+        try:
+            raise ValueError(1)
+        except:
+            exc_info = sys.exc_info()
+
+        response.handleException(exc_info)
+        self.assertEquals(response.getHeader("content-type"),
+            "text/html;charset=utf-8")
+        self.assertEquals(response.getStatus(), 500)
+        self.assert_(response.consumeBody() in
+            ["<html><head><title>&lt;type 'exceptions.ValueError'&gt;</title></head>\n"
+            "<body><h2>&lt;type 'exceptions.ValueError'&gt;</h2>\n"
+            "A server error occurred.\n"
+            "</body></html>\n",
+            "<html><head><title>ValueError</title></head>\n"
+            "<body><h2>ValueError</h2>\n"
+            "A server error occurred.\n"
+            "</body></html>\n"]
+            )
+
+
+class APITests(BaseTestIPublicationRequest,
+               BaseTestIApplicationRequest,
+               BaseTestIPublisherRequest,
+               unittest.TestCase):
+
+    def _Test__new(self, environ=None, **kw):
+        if environ is None:
+            environ = kw
+        return HTTPRequest(StringIO(''), environ)
+
+    def test_IApplicationRequest_bodyStream(self):
+        request = HTTPRequest(StringIO('spam'), {})
+        self.assertEqual(request.bodyStream.read(), 'spam')
+
+    # Needed by BaseTestIEnumerableMapping tests:
+    def _IEnumerableMapping__stateDict(self):
+        return {'id': 'ZopeOrg', 'title': 'Zope Community Web Site',
+                'greet': 'Welcome to the Zope Community Web site'}
+
+    def _IEnumerableMapping__sample(self):
+        return self._Test__new(**(self._IEnumerableMapping__stateDict()))
+
+    def _IEnumerableMapping__absentKeys(self):
+        return 'foo', 'bar'
+
+    def test_IPublicationRequest_getPositionalArguments(self):
+        self.assertEqual(self._Test__new().getPositionalArguments(), ())
+
+    def test_IPublisherRequest_retry(self):
+        self.assertEqual(self._Test__new().supportsRetry(), True)
+
+    def test_IPublisherRequest_processInputs(self):
+        self._Test__new().processInputs()
+
+    def test_IPublisherRequest_traverse(self):
+        request = self._Test__new()
+        request.setPublication(TestPublication())
+        app = request.publication.getApplication(request)
+
+        request.setTraversalStack([])
+        self.assertEqual(request.traverse(app).name, '')
+        self.assertEqual(request._last_obj_traversed, app)
+        request.setTraversalStack(['ZopeCorp'])
+        self.assertEqual(request.traverse(app).name, 'ZopeCorp')
+        self.assertEqual(request._last_obj_traversed, app.ZopeCorp)
+        request.setTraversalStack(['Engineering', 'ZopeCorp'])
+        self.assertEqual(request.traverse(app).name, 'Engineering')
+        self.assertEqual(request._last_obj_traversed, app.ZopeCorp.Engineering)
+
+def cleanUp(test):
+    zope.testing.cleanup.cleanUp()
+
 
 def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(ConcreteHTTPTests))
     suite.addTest(unittest.makeSuite(TestHTTPResponse))
     suite.addTest(unittest.makeSuite(HTTPInputStreamTests))
+    suite.addTest(doctest.DocFileSuite(
+        '../httpresults.txt', setUp=cleanUp, tearDown=cleanUp))
+    suite.addTest(unittest.makeSuite(APITests))
     return suite
 
 
