@@ -1,6 +1,5 @@
 # -*- coding: ISO-8859-1 -*-
 ##############################################################################
-#
 # Copyright (c) 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
 #
@@ -14,41 +13,49 @@
 ##############################################################################
 """Schema Fields
 
-$Id: _field.py 27056 2004-08-12 16:09:51Z srichter $
+$Id: _field.py 77624 2007-07-08 14:02:38Z ctheune $
 """
-import warnings
+
+__docformat__ = 'restructuredtext'
+
 import re
-from datetime import datetime, date
+import decimal
+from datetime import datetime, date, timedelta, time
 from sets import Set as SetType
 
-from zope.interface import classImplements, implements, directlyProvides
+from zope.event import notify
+
+from zope.interface import classImplements, implements
 from zope.interface.interfaces import IInterface, IMethod
 
 from zope.schema.interfaces import IField
 from zope.schema.interfaces import IMinMaxLen, IText, ITextLine
 from zope.schema.interfaces import ISourceText
 from zope.schema.interfaces import IInterfaceField
-from zope.schema.interfaces import IBytes, IASCII, IBytesLine
-from zope.schema.interfaces import IBool, IInt, IFloat, IDatetime
+from zope.schema.interfaces import IBytes, IASCII, IBytesLine, IASCIILine
+from zope.schema.interfaces import IBool, IInt, IFloat, IDatetime, IFrozenSet
 from zope.schema.interfaces import IChoice, ITuple, IList, ISet, IDict
-from zope.schema.interfaces import IPassword, IObject, IDate
+from zope.schema.interfaces import IPassword, IDate, ITimedelta
+from zope.schema.interfaces import IObject, IBeforeObjectAssignedEvent
+from zope.schema.interfaces import ITime, IDecimal
 from zope.schema.interfaces import IURI, IId, IFromUnicode
-from zope.schema.interfaces import IVocabulary
+from zope.schema.interfaces import ISource, IBaseVocabulary
+from zope.schema.interfaces import IContextSourceBinder
 
 from zope.schema.interfaces import ValidationError, InvalidValue
 from zope.schema.interfaces import WrongType, WrongContainedType, NotUnique
 from zope.schema.interfaces import SchemaNotProvided, SchemaNotFullyImplemented
 from zope.schema.interfaces import InvalidURI, InvalidId, InvalidDottedName
 from zope.schema.interfaces import ConstraintNotSatisfied
-from zope.schema.interfaces import Unbound
 
 from zope.schema._bootstrapfields import Field, Container, Iterable, Orderable
-from zope.schema._bootstrapfields import MinMaxLen
 from zope.schema._bootstrapfields import Text, TextLine, Bool, Int, Password
-from zope.schema._bootstrapfields import MinMaxLen, ValidatedProperty
+from zope.schema._bootstrapfields import MinMaxLen
 from zope.schema.fieldproperty import FieldProperty
 from zope.schema.vocabulary import getVocabularyRegistry
-from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
+from zope.schema.vocabulary import VocabularyRegistryError
+from zope.schema.vocabulary import SimpleVocabulary
+
 
 # Fix up bootstrap field types
 Field.title       = FieldProperty(IField['title'])
@@ -133,6 +140,14 @@ class BytesLine(Bytes):
         # TODO: we should probably use a more general definition of newlines
         return '\n' not in value
 
+class ASCIILine(ASCII):
+    __doc__ = IASCIILine.__doc__
+
+    implements(IASCIILine)
+
+    def constraint(self, value):
+        # TODO: we should probably use a more general definition of newlines
+        return '\n' not in value
 
 class Float(Orderable, Field):
     __doc__ = IFloat.__doc__
@@ -156,6 +171,31 @@ class Float(Orderable, Field):
         self.validate(v)
         return v
 
+class Decimal(Orderable, Field):
+    __doc__ = IDecimal.__doc__
+    implements(IDecimal, IFromUnicode)
+    _type = decimal.Decimal
+
+    def __init__(self, *args, **kw):
+        super(Decimal, self).__init__(*args, **kw)
+
+    def fromUnicode(self, u):
+        """
+        >>> f = Decimal()
+        >>> f.fromUnicode("1.25")
+        Decimal("1.25")
+        >>> f.fromUnicode("1.25.6")
+        Traceback (most recent call last):
+        ...
+        ValueError: invalid literal for Decimal(): 1.25.6
+        """
+        try:
+            v = decimal.Decimal(u)
+        except decimal.InvalidOperation:
+            raise ValueError('invalid literal for Decimal(): %s' % u)
+        self.validate(v)
+        return v
+
 class Datetime(Orderable, Field):
     __doc__ = IDatetime.__doc__
     implements(IDatetime)
@@ -168,6 +208,21 @@ class Date(Orderable, Field):
     __doc__ = IDate.__doc__
     implements(IDate)
     _type = date
+    
+    def _validate(self, value):
+        super(Date, self)._validate(value)
+        if isinstance(value, datetime):
+            raise WrongType(value, self._type)
+
+class Timedelta(Orderable, Field):
+    __doc__ = ITimedelta.__doc__
+    implements(ITimedelta)
+    _type = timedelta
+    
+class Time(Orderable, Field):
+    __doc__ = ITime.__doc__
+    implements(ITime)
+    _type = time
 
 class Choice(Field):
     """Choice fields can have a value found in a constant or dynamic set of
@@ -175,13 +230,21 @@ class Choice(Field):
     """
     implements(IChoice)
 
-    def __init__(self, values=None, vocabulary=None, **kw):
+    def __init__(self, values=None, vocabulary=None, source=None, **kw):
         """Initialize object."""
-        assert not (values is None and vocabulary is None), \
-               "You must specify either values or vocabulary."
-        assert values is None or vocabulary is None, \
-               "You cannot specify both values and vocabulary."
-        
+        if vocabulary is not None:
+            assert (isinstance(vocabulary, basestring)
+                    or IBaseVocabulary.providedBy(vocabulary))
+            assert source is None, (
+                "You cannot specify both source and vocabulary.")
+        elif source is not None:
+            vocabulary = source
+
+        assert not (values is None and vocabulary is None), (
+               "You must specify either values or vocabulary.")
+        assert values is None or vocabulary is None, (
+               "You cannot specify both values and vocabulary.")
+
         self.vocabulary = None
         self.vocabularyName = None
         if values is not None:
@@ -189,7 +252,8 @@ class Choice(Field):
         elif isinstance(vocabulary, (unicode, str)):
             self.vocabularyName = vocabulary
         else:
-            assert IVocabulary.providedBy(vocabulary)
+            assert (ISource.providedBy(vocabulary) or 
+                    IContextSourceBinder.providedBy(vocabulary))
             self.vocabulary = vocabulary
         # Before a default value is checked, it is validated. However, a
         # named vocabulary is usually not complete when these fields are
@@ -200,13 +264,20 @@ class Choice(Field):
         super(Choice, self).__init__(**kw)
         self._init_field = False
 
+    source = property(lambda self: self.vocabulary)
+
     def bind(self, object):
         """See zope.schema._bootstrapinterfaces.IField."""
         clone = super(Choice, self).bind(object)
         # get registered vocabulary if needed:
-        if clone.vocabulary is None and self.vocabularyName is not None:
+        if IContextSourceBinder.providedBy(self.vocabulary):
+            clone.vocabulary = self.vocabulary(object)
+            assert ISource.providedBy(clone.vocabulary)
+        elif clone.vocabulary is None and self.vocabularyName is not None:
             vr = getVocabularyRegistry()
             clone.vocabulary = vr.get(object, self.vocabularyName)
+            assert ISource.providedBy(clone.vocabulary)
+
         return clone
 
     def fromUnicode(self, str):
@@ -223,7 +294,7 @@ class Choice(Field):
         """
         self.validate(str)
         return str
-        
+
     def _validate(self, value):
         # Pass all validations during initialization
         if self._init_field:
@@ -237,7 +308,7 @@ class Choice(Field):
             except VocabularyRegistryError:
                 raise ValueError("Can't validate value without vocabulary")
         if value not in vocabulary:
-            raise ConstraintNotSatisfied, value
+            raise ConstraintNotSatisfied(value)
 
 class InterfaceField(Field):
     __doc__ = IInterfaceField.__doc__
@@ -246,33 +317,33 @@ class InterfaceField(Field):
     def _validate(self, value):
         super(InterfaceField, self)._validate(value)
         if not IInterface.providedBy(value):
-            raise WrongType
+            raise WrongType("An interface is required")
 
 def _validate_sequence(value_type, value, errors=None):
     """Validates a sequence value.
-    
+
     Returns a list of validation errors generated during the validation. If
     no errors are generated, returns an empty list.
-    
+
     value_type is a field. value is the sequence being validated. errors is
     an optional list of errors that will be prepended to the return value.
-    
+
     To illustrate, we'll use a text value type. All values must be unicode.
-    
-	    >>> field = TextLine(required=True)
-	
-	To validate a sequence of various values:
-	
-	    >>> errors = _validate_sequence(field, ('foo', u'bar', 1))
-	    >>> errors
-	    [foo <type 'unicode'>, 1 <type 'unicode'>]
-	
-	The only valid value in the sequence is the second item. The others
-	generated errors.
-	
-	We can use the optional errors argument to collect additional errors
-	for a new sequence:
-	
+
+            >>> field = TextLine(required=True)
+
+        To validate a sequence of various values:
+
+            >>> errors = _validate_sequence(field, ('foo', u'bar', 1))
+            >>> errors
+            [foo <type 'unicode'>, 1 <type 'unicode'>]
+
+        The only valid value in the sequence is the second item. The others
+        generated errors.
+
+        We can use the optional errors argument to collect additional errors
+        for a new sequence:
+
         >>> errors = _validate_sequence(field, (2, u'baz'), errors)
         >>> errors
         [foo <type 'unicode'>, 1 <type 'unicode'>, 2 <type 'unicode'>]
@@ -293,11 +364,11 @@ def _validate_uniqueness(value):
     temp_values = []
     for item in value:
         if item in temp_values:
-            raise NotUnique, item
+            raise NotUnique(item)
 
         temp_values.append(item)
 
-class AbstractCollection(MinMaxLen, Iterable, Field):
+class AbstractCollection(MinMaxLen, Iterable):
     value_type = None
     unique = False
 
@@ -305,7 +376,7 @@ class AbstractCollection(MinMaxLen, Iterable, Field):
         super(AbstractCollection, self).__init__(**kw)
         # whine if value_type is not a field
         if value_type is not None and not IField.providedBy(value_type):
-            raise ValueError, "'value_type' must be field instance."
+            raise ValueError("'value_type' must be field instance.")
         self.value_type = value_type
         self.unique = unique
 
@@ -315,14 +386,14 @@ class AbstractCollection(MinMaxLen, Iterable, Field):
         # binding value_type is necessary for choices with named vocabularies,
         # and possibly also for other fields.
         if clone.value_type is not None:
-           clone.value_type = clone.value_type.bind(object) 
+           clone.value_type = clone.value_type.bind(object)
         return clone
 
     def _validate(self, value):
         super(AbstractCollection, self)._validate(value)
         errors = _validate_sequence(self.value_type, value)
         if errors:
-            raise WrongContainedType, errors
+            raise WrongContainedType(errors)
         if self.unique:
             _validate_uniqueness(value)
 
@@ -339,28 +410,37 @@ class List(AbstractCollection):
 class Set(AbstractCollection):
     """A field representing a set."""
     implements(ISet)
-    _type = SetType
+    _type = SetType, set
     def __init__(self, **kw):
         if 'unique' in kw: # set members are always unique
             raise TypeError(
                 "__init__() got an unexpected keyword argument 'unique'")
         super(Set, self).__init__(unique=True, **kw)
 
+class FrozenSet(AbstractCollection):
+    implements(IFrozenSet)
+    _type = frozenset
+    def __init__(self, **kw):
+        if 'unique' in kw: # set members are always unique
+            raise TypeError(
+                "__init__() got an unexpected keyword argument 'unique'")
+        super(FrozenSet, self).__init__(unique=True, **kw)
+
 def _validate_fields(schema, value, errors=None):
     if errors is None:
-        errors = []  
+        errors = []
     for name in schema.names(all=True):
         if not IMethod.providedBy(schema[name]):
             try:
                 attribute = schema[name]
                 if IField.providedBy(attribute):
-                    # validate attributes that are fields  
+                    # validate attributes that are fields
                     attribute.validate(getattr(value, name))
             except ValidationError, error:
                 errors.append(error)
             except AttributeError, error:
                 # property for the given name is not implemented
-                errors.append(SchemaNotFullyImplemented())
+                errors.append(SchemaNotFullyImplemented(error))
     return errors
 
 
@@ -371,24 +451,44 @@ class Object(Field):
     def __init__(self, schema, **kw):
         if not IInterface.providedBy(schema):
             raise WrongType
-            
+
         self.schema = schema
         super(Object, self).__init__(**kw)
-        
+
     def _validate(self, value):
         super(Object, self)._validate(value)
-        
+
         # schema has to be provided by value
         if not self.schema.providedBy(value):
             raise SchemaNotProvided
-            
+
         # check the value against schema
         errors = _validate_fields(self.schema, value)
         if errors:
             raise WrongContainedType(errors)
 
+    def set(self, object, value):
+        # Announce that we're going to assign the value to the object.
+        # Motivation: Widgets typically like to take care of policy-specific
+        # actions, like establishing location.
+        event = BeforeObjectAssignedEvent(value, self.__name__, object)
+        notify(event)
+        # The event subscribers are allowed to replace the object, thus we need
+        # to replace our previous value.
+        value = event.object
+        super(Object, self).set(object, value)
 
-class Dict(MinMaxLen, Iterable, Field):
+class BeforeObjectAssignedEvent(object):
+    """An object is going to be assigned to an attribute on another object."""
+
+    implements(IBeforeObjectAssignedEvent)
+
+    def __init__(self, object, name, context):
+        self.object = object
+        self.name = name
+        self.context = context
+
+class Dict(MinMaxLen, Iterable):
     """A field representing a Dict."""
     implements(IDict)
     _type = dict
@@ -399,9 +499,9 @@ class Dict(MinMaxLen, Iterable, Field):
         super(Dict, self).__init__(**kw)
         # whine if key_type or value_type is not a field
         if key_type is not None and not IField.providedBy(key_type):
-            raise ValueError, "'key_type' must be field instance."
+            raise ValueError("'key_type' must be field instance.")
         if value_type is not None and not IField.providedBy(value_type):
-            raise ValueError, "'value_type' must be field instance."
+            raise ValueError("'value_type' must be field instance.")
         self.key_type = key_type
         self.value_type = value_type
 
@@ -415,16 +515,28 @@ class Dict(MinMaxLen, Iterable, Field):
             errors = _validate_sequence(self.key_type, value, errors)
 
             if errors:
-                raise WrongContainedType, errors
+                raise WrongContainedType(errors)
 
         finally:
             errors = None
 
+    def bind(self, object):
+        """See zope.schema._bootstrapinterfaces.IField."""
+        clone = super(Dict, self).bind(object)
+        # binding value_type is necessary for choices with named vocabularies,
+        # and possibly also for other fields.
+        if clone.key_type is not None:
+           clone.key_type = clone.key_type.bind(object)
+        if clone.value_type is not None:
+           clone.value_type = clone.value_type.bind(object)
+        return clone
+
 
 _isuri = re.compile(
     r"[a-zA-z0-9+.-]+:"   # scheme
     r"\S*$"               # non space (should be pickier)
     ).match
+
 class URI(BytesLine):
     """URI schema field
     """
@@ -446,7 +558,7 @@ class URI(BytesLine):
         if _isuri(value):
             return
 
-        raise InvalidURI, value
+        raise InvalidURI(value)
 
     def fromUnicode(self, value):
         """
@@ -501,7 +613,7 @@ class Id(BytesLine):
         if _isdotted(value) and "." in value:
             return
 
-        raise InvalidId, value
+        raise InvalidId(value)
 
     def fromUnicode(self, value):
         """
@@ -572,10 +684,10 @@ class DottedName(BytesLine):
         >>> dotted_name = DottedName(__name__='test')
         >>> dotted_name.validate("a.b.c")
         >>> dotted_name.validate("a")
-        >>> dotted_name.validate("   a   ")
+        >>> dotted_name.validate("   a")
         Traceback (most recent call last):
         ...
-        InvalidDottedName:    a   
+        InvalidDottedName:    a
 
         >>> dotted_name = DottedName(__name__='test', min_dots=1)
         >>> dotted_name.validate('a.b')
@@ -618,12 +730,11 @@ class DottedName(BytesLine):
             raise InvalidDottedName(value)
         dots = value.count(".")
         if dots < self.min_dots:
-            raise InvalidDottedName, \
-                  ("too few dots; %d required" % self.min_dots, value)
+            raise InvalidDottedName("too few dots; %d required" % self.min_dots,
+                                    value)
         if self.max_dots is not None and dots > self.max_dots:
-            raise InvalidDottedName, \
-                  ("too many dots; no more than %d allowed" % self.max_dots,
-                   value)
+            raise InvalidDottedName("too many dots; no more than %d allowed" %
+                                    self.max_dots, value)
 
     def fromUnicode(self, value):
         v = str(value.strip())
