@@ -23,13 +23,14 @@ messages about granted attribute access.
 Note that the ZOPE_WATCH_CHECKERS mechanism will eventually be
 replaced with a more general security auditing mechanism.
 
-$Id: checker.py 40887 2005-12-19 18:03:34Z poster $
+$Id: checker.py 78815 2007-08-14 17:52:16Z jim $
 """
 import os
 import sys
 import sets
 import types
 import datetime
+import decimal
 import pytz
 import weakref
 
@@ -43,7 +44,7 @@ from zope.interface.interfaces import IInterface, IDeclaration
 from zope.security.interfaces import IChecker, INameBasedChecker
 from zope.security.interfaces import ISecurityProxyFactory
 from zope.security.interfaces import Unauthorized, ForbiddenAttribute
-from zope.security.management import thread_local
+from zope.security._definitions import thread_local
 from zope.security._proxy import _Proxy as Proxy, getChecker
 
 if os.environ.get('ZOPE_WATCH_CHECKERS'):
@@ -100,7 +101,7 @@ def canWrite(obj, name):
     except ForbiddenAttribute:
         # we are going to be a bit DWIM-y here: see
         # http://www.zope.org/Collectors/Zope3-dev/506
-        
+
         # generally, if the check is ForbiddenAttribute we want it to be
         # raised: it probably indicates a programming or configuration error.
         # However, we special case a write ForbiddenAttribute when one can
@@ -394,19 +395,6 @@ def defineChecker(type_, checker):
         raise DuplicationError(type_)
     _checkers[type_] = checker
 
-def defineChecker(type_, checker):
-    """Define a checker for a given type of object
-
-    The checker can be a Checker, or a function that, when called with
-    an object, returns a Checker.
-    """
-    if not isinstance(type_, (type, types.ClassType, types.ModuleType)):
-        raise TypeError(
-                'type_ must be a type, class or module, not a %s' % type_)
-    if type_ in _checkers:
-        raise DuplicationError(type_)
-    _checkers[type_] = checker
-
 def undefineChecker(type_):
     del _checkers[type_]
 
@@ -591,13 +579,14 @@ _setChecker = NamesChecker(['__iter__', '__len__', '__str__', '__contains__',
                             'copy', 'difference', 'intersection', 'issubset',
                             'issuperset', 'symmetric_difference', 'union',
                             '__and__', '__or__', '__sub__', '__xor__',
+                            '__rand__', '__ror__', '__rsub__', '__rxor__',
                             '__eq__', '__ne__', '__lt__', '__gt__',
                             '__le__', '__ge__'])
 
 class BasicTypes(dict):
     """Basic Types Dictionary
 
-    Make sure that the checkers a really updated, when a new type is added.
+    Make sure that checkers are really updated, when a new type is added.
     """
     def __setitem__(self, name, value):
         super(BasicTypes.__class__, self).__setitem__(name, value)
@@ -658,7 +647,10 @@ class _Sequence(object):
 _Declaration_checker = InterfaceChecker(
     IDeclaration,
     _implied=CheckerPublic,
-    subscribe=CheckerPublic)
+    subscribe=CheckerPublic,
+    unsubscribe=CheckerPublic,
+    __call__=CheckerPublic,
+    )
 
 def f():
     yield f
@@ -674,6 +666,22 @@ _default_checkers = {
                         '__add__', '__radd__', ]),
     sets.Set: _setChecker,
     sets.ImmutableSet: _setChecker,
+    set: _setChecker,
+    frozenset: _setChecker,
+    decimal.Decimal: NamesChecker(['__nonzero__', '__cmp__', '__eq__',
+                                   '__ne__', 'compare', '__hash__',
+                                   'as_tuple', '__str__', 'to_eng_string',
+                                   '__neg__', '__pos__', '__abs__',
+                                   '__add__', '__radd__', '__sub__',
+                                   '__rsub__', '__mul__', '__rmul__',
+                                   '__div__', '__rdiv__', '__rtruediv__',
+                                   '__divmod__', '__rdivmod__', '__mod__',
+                                   '__rmod__', 'remainder_near',
+                                   '__floordiv__', '__rfloordiv__',
+                                   '__float__', '__int__', '__long__',
+                                   '__pow__', '__rpow__', 'normalize',
+                                   'quantize', 'same_quantum', 'to_integral',
+                                   'sqrt', 'max', 'min', 'adjusted']),
 
     # YAGNI: () a rock
     tuple: NamesChecker(['__getitem__', '__getslice__', '__add__', '__radd__',
@@ -693,6 +701,7 @@ _default_checkers = {
     type(iter([])): _iteratorChecker, # Same types in Python 2.2.1,
     type(iter(())): _iteratorChecker, # different in Python 2.3.
     type(iter({})): _iteratorChecker,
+    type(iter(set())): _iteratorChecker,
     type({}.iteritems()): _iteratorChecker,
     type({}.iterkeys()): _iteratorChecker,
     type({}.itervalues()): _iteratorChecker,
@@ -701,10 +710,6 @@ _default_checkers = {
     type(Interface): InterfaceChecker(
         IInterface,
         __str__=CheckerPublic, _implied=CheckerPublic, subscribe=CheckerPublic,
-        # BBB 2004-07-13: backward compatability for deprecated
-        # interface methods
-        isImplementedByInstancesOf=CheckerPublic,
-        isImplementedBy=CheckerPublic,
         ),
     zope.interface.interface.Method: InterfaceChecker(
                                         zope.interface.interfaces.IMethod),
@@ -714,14 +719,6 @@ _default_checkers = {
     zope.interface.declarations.Declaration: _Declaration_checker,
 }
 
-# If we are running with Python 2.4+, setup security on the builtin
-# set and frozenset types.
-try:
-    _default_checkers[set] = _setChecker
-    _default_checkers[frozenset] = _setChecker
-except NameError:
-    pass
-
 def _clear():
     _checkers.clear()
     _checkers.update(_default_checkers)
@@ -729,5 +726,10 @@ def _clear():
 
 _clear()
 
-from zope.testing.cleanup import addCleanUp
-addCleanUp(_clear)
+try:
+    from zope.testing.cleanup import addCleanUp
+except ImportError:
+    pass
+else:
+    addCleanUp(_clear)
+
