@@ -13,9 +13,10 @@
 ##############################################################################
 """Zope 3-specific testing code
 
-$Id: testing.py 68954 2006-07-02 18:46:20Z benji_york $
+$Id: testing.py 79406 2007-08-31 16:24:37Z nikhil_n $
 """
 import re
+import sys
 import unittest
 import httplib
 import urllib2
@@ -59,7 +60,10 @@ class PublisherConnection(object):
 
         url = self._quote(url)
         # Extract the handle_error option header
-        handle_errors_key = 'X-zope-handle-errors'
+        if sys.version_info >= (2,5):
+            handle_errors_key = 'X-Zope-Handle-Errors'
+        else:
+            handle_errors_key = 'X-zope-handle-errors'
         handle_errors = headers.get(handle_errors_key, True)
         if handle_errors_key in headers:
             del headers[handle_errors_key]
@@ -111,6 +115,10 @@ class PublisherResponse(object):
     def read(self, amt=None):
         return self.content_as_file.read(amt)
 
+    def close(self):
+        """To overcome changes in urllib2 and socket in python2.5"""
+        pass
+
 
 class PublisherHTTPHandler(urllib2.HTTPHandler):
     """Special HTTP handler to use the Zope Publisher."""
@@ -130,13 +138,13 @@ class PublisherMechanizeBrowser(mechanize.Browser):
     default_others = ['_http_error', '_http_request_upgrade',
                       '_http_default_error']
     default_features = ['_redirect', '_cookies', '_referer', '_refresh',
-                        '_equiv', '_basicauth', '_digestauth', '_seek' ]
+                        '_equiv', '_basicauth', '_digestauth', ]
 
     def __init__(self, *args, **kws):
         inherited_handlers = ['_unknown', '_http_error',
             '_http_request_upgrade', '_http_default_error', '_basicauth',
             '_digestauth', '_redirect', '_cookies', '_referer',
-            '_refresh', '_equiv', '_seek', '_gzip']
+            '_refresh', '_equiv', '_gzip']
 
         self.handler_classes = {"http": PublisherHTTPHandler}
         for name in inherited_handlers:
@@ -175,7 +183,7 @@ class VirtualHostingPublisherHTTPHandler(urllib2.HTTPHandler):
     def http_open(self, req):
         """Open an HTTP connection having a ``urllib2`` request."""
         # Here we connect to the publisher.
-        return self.do_open(VirtualHostingPublisherConnection, req) 
+        return self.do_open(VirtualHostingPublisherConnection, req)
 
 class VirtualHostingPublisherMechanizeBrowser(PublisherMechanizeBrowser):
     handler_classes = PublisherMechanizeBrowser.handler_classes.copy()
@@ -200,7 +208,7 @@ def virtualHostingSetUp(test):
 def VirtualHostTestBrowserSuite(*paths, **kw):
 #    layer=None,
 #    globs=None, setUp=None, normalizers=None, **kw):
-    
+
     if 'checker' in kw:
         raise ValueError(
             'Must not supply custom checker.  To provide values for '
@@ -211,7 +219,10 @@ def VirtualHostTestBrowserSuite(*paths, **kw):
     layer = kw.pop('layer', None)
     normalizers = kw.pop('normalizers', None)
     vh_kw = kw.copy()
-    globs = kw.setdefault('globs', {})
+    if 'globs' in kw:
+        globs = kw['globs'] = kw['globs'].copy() # don't mutate the original
+    else:
+        globs = kw['globs'] = {}
     if 'Browser' in globs:
         raise ValueError('"Browser" must not be defined in globs')
     vh_kw['globs'] = globs.copy()
@@ -232,6 +243,7 @@ def VirtualHostTestBrowserSuite(*paths, **kw):
     suite = unittest.TestSuite()
     test = functional.FunctionalDocFileSuite(*paths, **kw)
     vh_test = functional.FunctionalDocFileSuite(*paths, **vh_kw)
+    vh_test.level = 2
     if layer is not None:
         test.layer = layer
         vh_test.layer = layer
